@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2004 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine.h.in,v 1.58 2003/01/29 02:33:35 miguelfreitas Exp $
+ * $Id: xine.h.in,v 1.136.2.4 2005/07/17 21:49:33 miguelfreitas Exp $
  *
  * public xine-lib (libxine) interface and documentation
  *
@@ -37,6 +37,8 @@
  *     use xlockdisplay() or use two seperate connections to the x-server
  *
  */
+/*_x_ Lines formatted like this one are xine-lib developer comments.  */
+/*_x_ They will be removed from the installed version of this header. */
 
 #ifndef HAVE_XINE_H
 #define HAVE_XINE_H
@@ -46,11 +48,35 @@ extern "C" {
 #endif
 
 #include <sys/types.h>
-#include <unistd.h>
-#include <inttypes.h>
 #include <sys/time.h>
 #include <time.h>
 
+#ifdef WIN32
+#include <windows.h>
+#include <windowsx.h>
+#endif
+
+#ifdef XINE_COMPILE
+#include <inttypes.h>
+#else
+#include <xine/os_types.h>
+#endif
+
+
+/* This enables some experimental features. These are not part of the
+ * official libxine API, so use them only, if you absolutely need them.
+ * Although we make efforts to keep even this part of the API as stable
+ * as possible, this is not guaranteed. Incompatible changes can occur.
+ */
+/* #define XINE_ENABLE_EXPERIMENTAL_FEATURES */
+
+/* This disables some deprecated features. These are still part of the
+ * official libxine API and you may still use them. During the current
+ * major release series, these will always be available and will stay
+ * compatible. However, removal is likely to occur as soon as possible.
+ */
+/* #define XINE_DISABLE_DEPRECATED_FEATURES */
+
 
 /*********************************************************************
  * xine opaque data types                                            *
@@ -61,9 +87,11 @@ typedef struct xine_stream_s xine_stream_t;
 typedef struct xine_audio_port_s xine_audio_port_t;
 typedef struct xine_video_port_s xine_video_port_t;
 
+#ifndef XINE_DISABLE_DEPRECATED_FEATURES
 /* convenience types: simple player UIs might want to call ports drivers */
 typedef xine_audio_port_t xine_ao_driver_t;
 typedef xine_video_port_t xine_vo_driver_t;
+#endif
 
 
 /*********************************************************************
@@ -85,8 +113,8 @@ int  xine_check_version (int major, int minor, int sub) ;
 /* static info - which libxine release this header came from */
 #define XINE_MAJOR_VERSION 1
 #define XINE_MINOR_VERSION 0
-#define XINE_SUB_VERSION   0
-#define XINE_VERSION       "1-beta4"
+#define XINE_SUB_VERSION   2
+#define XINE_VERSION       "1.0.2"
 
 /*
  * pre-init the xine engine
@@ -142,6 +170,9 @@ void xine_close_video_driver (xine_t *self, xine_video_port_t  *driver);
 #define XINE_VISUAL_TYPE_GTK               4
 #define XINE_VISUAL_TYPE_DFB               5
 #define XINE_VISUAL_TYPE_PM                6 /* used by the OS/2 port */
+#define XINE_VISUAL_TYPE_DIRECTX           7 /* used by the win32/msvc port */
+#define XINE_VISUAL_TYPE_CACA              8
+#define XINE_VISUAL_TYPE_MACOSX            9
 
 /*
  * free all resources, close all plugins, close engine.
@@ -194,6 +225,8 @@ int xine_stream_master_slave(xine_stream_t *master, xine_stream_t *slave,
 #define XINE_MASTER_SLAVE_PLAY     (1<<0)
 /* slave stops on master stop */
 #define XINE_MASTER_SLAVE_STOP     (1<<1)
+/* slave is synced to master's speed */
+#define XINE_MASTER_SLAVE_SPEED    (1<<2)
 
 /*
  * open a stream
@@ -258,14 +291,22 @@ int  xine_eject (xine_stream_t *stream);
 void xine_dispose (xine_stream_t *stream);
 
 /*
- * set/get xine engine parameters
+ * set/get engine parameters.
+ */
+void xine_engine_set_param(xine_t *self, int param, int value);
+int xine_engine_get_param(xine_t *self, int param);
+  
+#define XINE_ENGINE_PARAM_VERBOSITY        1
+
+/*
+ * set/get xine stream parameters
  * e.g. playback speed, constants see below
  */
 void xine_set_param (xine_stream_t *stream, int param, int value);
 int  xine_get_param (xine_stream_t *stream, int param);
 
 /*
- * xine engine parameters
+ * xine stream parameters
  */
 #define XINE_PARAM_SPEED                   1 /* see below                   */
 #define XINE_PARAM_AV_OFFSET               2 /* unit: 1/90000 sec           */
@@ -279,14 +320,41 @@ int  xine_get_param (xine_stream_t *stream, int param);
 #define XINE_PARAM_AUDIO_REPORT_LEVEL     10 /* 1=>send events, 0=> don't   */
 #define XINE_PARAM_VERBOSITY              11 /* control console output      */
 #define XINE_PARAM_SPU_OFFSET             12 /* unit: 1/90000 sec           */
+#define XINE_PARAM_IGNORE_VIDEO           13 /* disable video decoding      */
+#define XINE_PARAM_IGNORE_AUDIO           14 /* disable audio decoding      */
+#define XINE_PARAM_IGNORE_SPU             15 /* disable spu decoding        */
+#define XINE_PARAM_BROADCASTER_PORT       16 /* 0: disable, x: server port  */
+#define XINE_PARAM_METRONOM_PREBUFFER     17 /* unit: 1/90000 sec           */
+#define XINE_PARAM_EQ_30HZ                18 /* equalizer gains -100..100   */
+#define XINE_PARAM_EQ_60HZ                19 /* equalizer gains -100..100   */
+#define XINE_PARAM_EQ_125HZ               20 /* equalizer gains -100..100   */
+#define XINE_PARAM_EQ_250HZ               21 /* equalizer gains -100..100   */
+#define XINE_PARAM_EQ_500HZ               22 /* equalizer gains -100..100   */
+#define XINE_PARAM_EQ_1000HZ              23 /* equalizer gains -100..100   */
+#define XINE_PARAM_EQ_2000HZ              24 /* equalizer gains -100..100   */
+#define XINE_PARAM_EQ_4000HZ              25 /* equalizer gains -100..100   */
+#define XINE_PARAM_EQ_8000HZ              26 /* equalizer gains -100..100   */
+#define XINE_PARAM_EQ_16000HZ             27 /* equalizer gains -100..100   */
+#define XINE_PARAM_AUDIO_CLOSE_DEVICE     28 /* force closing audio device  */
+#define XINE_PARAM_AUDIO_AMP_MUTE         29 /* 1=>mute, 0=>unmute */
+#define XINE_PARAM_FINE_SPEED             30 /* 1.000.000 => normal speed   */ 
 
-/* speed values */
+/*
+ * speed values for XINE_PARAM_SPEED parameter.
+ *
+ * alternatively, one may use XINE_PARAM_FINE_SPEED for greater
+ * control of the speed value, where:
+ * XINE_PARAM_SPEED / 4 <-> XINE_PARAM_FINE_SPEED / 1000000
+ */
 #define XINE_SPEED_PAUSE                   0
 #define XINE_SPEED_SLOW_4                  1
 #define XINE_SPEED_SLOW_2                  2
 #define XINE_SPEED_NORMAL                  4
 #define XINE_SPEED_FAST_2                  8
 #define XINE_SPEED_FAST_4                  16
+ 
+/* normal speed value for XINE_PARAM_FINE_SPEED parameter */
+#define XINE_FINE_SPEED_NORMAL             1000000
 
 /* video parameters */
 #define XINE_PARAM_VO_DEINTERLACE          0x01000000 /* bool               */
@@ -299,6 +367,11 @@ int  xine_get_param (xine_stream_t *stream, int param);
 #define XINE_PARAM_VO_ZOOM_Y               0x0100000d /* percent            */
 #define XINE_PARAM_VO_PAN_SCAN             0x01000009 /* bool               */
 #define XINE_PARAM_VO_TVMODE               0x0100000a /* ???                */
+#define XINE_PARAM_VO_CROP_LEFT            0x01000020 /* crop frame pixels  */
+#define XINE_PARAM_VO_CROP_RIGHT           0x01000021 /* crop frame pixels  */
+#define XINE_PARAM_VO_CROP_TOP             0x01000022 /* crop frame pixels  */
+#define XINE_PARAM_VO_CROP_BOTTOM          0x01000023 /* crop frame pixels  */
+
 
 #define XINE_VO_ZOOM_STEP                  100
 #define XINE_VO_ZOOM_MAX                   400
@@ -306,13 +379,15 @@ int  xine_get_param (xine_stream_t *stream, int param);
 
 /* possible ratios for XINE_PARAM_VO_ASPECT_RATIO */
 #define XINE_VO_ASPECT_AUTO                0
-#define XINE_VO_ASPECT_SQUARE              1 /* 1:1  */
-#define XINE_VO_ASPECT_4_3                 2 /* 4:3  */
-#define XINE_VO_ASPECT_ANAMORPHIC          3 /* 16:9 */
-#define XINE_VO_ASPECT_DVB                 4 /* 1:2  */
+#define XINE_VO_ASPECT_SQUARE              1 /* 1:1    */
+#define XINE_VO_ASPECT_4_3                 2 /* 4:3    */
+#define XINE_VO_ASPECT_ANAMORPHIC          3 /* 16:9   */
+#define XINE_VO_ASPECT_DVB                 4 /* 2.11:1 */
 #define XINE_VO_ASPECT_NUM_RATIOS          5
+#ifndef XINE_DISABLE_DEPRECATED_FEATURES
 #define XINE_VO_ASPECT_PAN_SCAN            41
 #define XINE_VO_ASPECT_DONT_TOUCH          42
+#endif
 
 /* stream format detection strategies */
 
@@ -325,6 +400,11 @@ int  xine_get_param (xine_stream_t *stream, int param);
 /* recognize stream type by extension only.                  */
 #define XINE_DEMUX_EXTENSION_STRATEGY      3
 
+/* verbosity settings */
+#define XINE_VERBOSITY_NONE                0
+#define XINE_VERBOSITY_LOG                 1
+#define XINE_VERBOSITY_DEBUG               2
+
 /*
  * snapshot function
  *
@@ -342,6 +422,14 @@ int  xine_get_current_frame (xine_stream_t *stream,
 /* xine image formats */
 #define XINE_IMGFMT_YV12 (('2'<<24)|('1'<<16)|('V'<<8)|'Y')
 #define XINE_IMGFMT_YUY2 (('2'<<24)|('Y'<<16)|('U'<<8)|'Y')
+#define XINE_IMGFMT_XVMC (('C'<<24)|('M'<<16)|('v'<<8)|'X')
+#define XINE_IMGFMT_XXMC (('C'<<24)|('M'<<16)|('x'<<8)|'X')
+
+/* get current xine's virtual presentation timestamp (1/90000 sec)
+ * note: this is mostly internal data.
+ * one can use vpts with xine_osd_show() and xine_osd_hide().
+ */
+int64_t xine_get_current_vpts(xine_stream_t *stream);
 
 
 /*********************************************************************
@@ -355,7 +443,11 @@ int  xine_get_current_frame (xine_stream_t *stream,
  * these functions are intended to provide the basis for
  * re-encoding and other video processing applications
  *
- * warning: highly experimental
+ * note that the xine playback engine will block when
+ * rendering to a framegrab port: to unblock the stream,
+ * you must fetch the frames manually with the
+ * xine_get_next_* functions.  this ensures that a
+ * framegrab port is guaranteed to never miss a frame.
  *
  */
 
@@ -374,6 +466,8 @@ typedef struct {
 
   uint8_t *data;
   void    *xine_frame; /* used internally by xine engine */
+  
+  int      frame_number; /* frame number (may be unknown) */
 } xine_video_frame_t;
 
 int xine_get_next_video_frame (xine_video_port_t *port,
@@ -534,10 +628,10 @@ const char *const *xine_post_list_inputs(xine_post_t *self);
 const char *const *xine_post_list_outputs(xine_post_t *self);
 
 /* retrieve one specific input of a post plugin */
-const xine_post_in_t *xine_post_input(xine_post_t *self, char *name);
+xine_post_in_t *xine_post_input(xine_post_t *self, const char *name);
 
 /* retrieve one specific output of a post plugin */
-const xine_post_out_t *xine_post_output(xine_post_t *self, char *name);
+xine_post_out_t *xine_post_output(xine_post_t *self, const char *name);
 
 /*
  * wire an input to an output
@@ -565,7 +659,7 @@ int xine_post_wire_video_port(xine_post_out_t *source, xine_video_port_t *vo);
  *
  * returns 1 on success, 0 on failure
  */
-int xine_post_wire_audio_port(xine_post_out_t *source, xine_audio_port_t *vo);
+int xine_post_wire_audio_port(xine_post_out_t *source, xine_audio_port_t *ao);
 
 /*
  * Extracts an output for a stream. Use this to rewire the outputs of streams.
@@ -584,6 +678,7 @@ void xine_post_dispose(xine_t *xine, xine_post_t *self);
 /* post plugin types */
 #define XINE_POST_TYPE_VIDEO_FILTER		0x010000
 #define XINE_POST_TYPE_VIDEO_VISUALIZATION	0x010001
+#define XINE_POST_TYPE_VIDEO_COMPOSE		0x010002
 #define XINE_POST_TYPE_AUDIO_FILTER		0x020000
 #define XINE_POST_TYPE_AUDIO_VISUALIZATION	0x020001
 
@@ -614,6 +709,64 @@ void xine_post_dispose(xine_t *xine, xine_post_t *self);
  */
 #define XINE_POST_DATA_DOUBLE         4
 
+/* parameters api (used by frontends)
+ * input->data is xine_post_api_t*  (see below)
+ */
+#define XINE_POST_DATA_PARAMETERS     5
+
+/* defines a single parameter entry. */
+typedef struct {
+  int              type;             /* POST_PARAM_TYPE_xxx             */
+  char            *name;             /* name of this parameter          */
+  int              size;             /* sizeof(parameter)               */
+  int              offset;           /* offset in bytes from struct ptr */
+  char           **enum_values;      /* enumeration (first=0) or NULL   */
+  double           range_min;        /* minimum value                   */
+  double           range_max;        /* maximum value                   */
+  int              readonly;         /* 0 = read/write, 1=read-only     */
+  char            *description;      /* user-friendly description       */
+} xine_post_api_parameter_t;
+
+/* description of parameters struct (params). */
+typedef struct {
+  int struct_size;                       /* sizeof(params)     */
+  xine_post_api_parameter_t *parameter;  /* list of parameters */
+} xine_post_api_descr_t;
+
+typedef struct {
+  /*
+   * method to set all the read/write parameters.
+   * params is a struct * defined by xine_post_api_descr_t
+   */
+  int (*set_parameters) (xine_post_t *self, void *params);
+
+  /*
+   * method to get all parameters.
+   */
+  int (*get_parameters) (xine_post_t *self, void *params);
+
+  /*
+   * method to get params struct definition
+   */
+  xine_post_api_descr_t * (*get_param_descr) (void);
+
+  /*
+   * method to get plugin and parameters help (UTF-8)
+   * the help string must be word wrapped by the frontend.
+   * it might contain \n to mark paragraph breaks.
+   */
+  char * (*get_help) (void);
+} xine_post_api_t;
+
+/* post parameter types */
+#define POST_PARAM_TYPE_LAST       0  /* terminator of parameter list       */
+#define POST_PARAM_TYPE_INT        1  /* integer (or vector of integers)    */
+#define POST_PARAM_TYPE_DOUBLE     2  /* double (or vector of doubles)      */
+#define POST_PARAM_TYPE_CHAR       3  /* char (or vector of chars = string) */
+#define POST_PARAM_TYPE_STRING     4  /* (char *), ASCIIZ                   */
+#define POST_PARAM_TYPE_STRINGLIST 5  /* (char **) list, NULL terminated    */
+#define POST_PARAM_TYPE_BOOL       6  /* integer (0 or 1)                   */
+
 
 /*********************************************************************
  * information retrieval                                             *
@@ -630,8 +783,13 @@ int    xine_get_log_section_count(xine_t *self);
 const char *const *xine_get_log_names(xine_t *self);
 
 /* print some log information to <buf> section */
+#ifndef __GNUC__
 void   xine_log (xine_t *self, int buf,
 		 const char *format, ...);
+#else
+void   xine_log (xine_t *self, int buf,
+		 const char *format, ...) __attribute__ ((__format__(printf, 3, 4)));
+#endif
 
 /* get log messages of specified section */
 const char *const *xine_get_log (xine_t *self, int buf);
@@ -667,6 +825,7 @@ int  xine_get_status (xine_stream_t *stream);
 #define XINE_ERROR_NO_DEMUX_PLUGIN         2
 #define XINE_ERROR_DEMUX_FAILED            3
 #define XINE_ERROR_MALFORMED_MRL           4
+#define XINE_ERROR_INPUT_FAILED            5
 
 /*
  * try to find out audio/spu language of given channel
@@ -678,6 +837,8 @@ int xine_get_audio_lang (xine_stream_t *stream, int channel,
 			 char *lang);
 int xine_get_spu_lang   (xine_stream_t *stream, int channel,
 			 char *lang);
+/*_x_ increasing this number means an incompatible ABI breakage! */
+#define XINE_LANG_MAX                     32
 
 /*
  * get position / length information
@@ -697,6 +858,7 @@ int  xine_get_pos_length (xine_stream_t *stream,
 /*
  * get information about the stream such as
  * video width/height, codecs, audio format, title, author...
+ * strings are UTF-8 encoded.
  *
  * constants see below
  */
@@ -730,6 +892,35 @@ const char *xine_get_meta_info   (xine_stream_t *stream, int info);
 #define XINE_STREAM_INFO_VIDEO_HAS_STILL   23
 #define XINE_STREAM_INFO_MAX_AUDIO_CHANNEL 24
 #define XINE_STREAM_INFO_MAX_SPU_CHANNEL   25
+#define XINE_STREAM_INFO_AUDIO_MODE        26
+#define XINE_STREAM_INFO_SKIPPED_FRAMES    27 /* for 1000 frames delivered */
+#define XINE_STREAM_INFO_DISCARDED_FRAMES  28 /* for 1000 frames delivered */
+#define XINE_STREAM_INFO_VIDEO_AFD         29
+#define XINE_STREAM_INFO_DVD_TITLE_NUMBER   30
+#define XINE_STREAM_INFO_DVD_TITLE_COUNT    31
+#define XINE_STREAM_INFO_DVD_CHAPTER_NUMBER 32
+#define XINE_STREAM_INFO_DVD_CHAPTER_COUNT  33
+#define XINE_STREAM_INFO_DVD_ANGLE_NUMBER   34
+#define XINE_STREAM_INFO_DVD_ANGLE_COUNT    35
+
+/* possible values for XINE_STREAM_INFO_VIDEO_AFD */
+#define XINE_VIDEO_AFD_NOT_PRESENT         -1
+#define XINE_VIDEO_AFD_RESERVED_0          0
+#define XINE_VIDEO_AFD_RESERVED_1          1
+#define XINE_VIDEO_AFD_BOX_16_9_TOP        2
+#define XINE_VIDEO_AFD_BOX_14_9_TOP        3
+#define XINE_VIDEO_AFD_BOX_GT_16_9_CENTRE  4
+#define XINE_VIDEO_AFD_RESERVED_5          5
+#define XINE_VIDEO_AFD_RESERVED_6          6
+#define XINE_VIDEO_AFD_RESERVED_7          7
+#define XINE_VIDEO_AFD_SAME_AS_FRAME       8
+#define XINE_VIDEO_AFD_4_3_CENTRE          9
+#define XINE_VIDEO_AFD_16_9_CENTRE         10
+#define XINE_VIDEO_AFD_14_9_CENTRE         11
+#define XINE_VIDEO_AFD_RESERVED_12         12
+#define XINE_VIDEO_AFD_4_3_PROTECT_14_9    13
+#define XINE_VIDEO_AFD_16_9_PROTECT_14_9   14
+#define XINE_VIDEO_AFD_16_9_PROTECT_4_3    15
 
 /* xine_get_meta_info */
 #define XINE_META_INFO_TITLE               0
@@ -742,6 +933,8 @@ const char *xine_get_meta_info   (xine_stream_t *stream, int info);
 #define XINE_META_INFO_AUDIOCODEC          7
 #define XINE_META_INFO_SYSTEMLAYER         8
 #define XINE_META_INFO_INPUT_PLUGIN        9
+#define XINE_META_INFO_CDINDEX_DISCID      10
+#define XINE_META_INFO_TRACK_NUMBER        11
 
 
 /*********************************************************************
@@ -825,22 +1018,57 @@ char *xine_get_mime_types (xine_t *self);
  * returns NULL if no demuxer is available to handle this. */
 char *xine_get_demux_for_mime_type (xine_t *self, const char *mime_type);
 
-/* get a description string for an input plugin */
+/* get a description string for a plugin */
 const char *xine_get_input_plugin_description (xine_t *self,
 					       const char *plugin_id);
+const char *xine_get_demux_plugin_description (xine_t *self,
+					       const char *plugin_id);
+const char *xine_get_spu_plugin_description   (xine_t *self,
+					       const char *plugin_id);
+const char *xine_get_audio_plugin_description (xine_t *self,
+					       const char *plugin_id);
+const char *xine_get_video_plugin_description (xine_t *self,
+					       const char *plugin_id);
+const char *xine_get_audio_driver_plugin_description (xine_t *self,
+					       const char *plugin_id);
+const char *xine_get_video_driver_plugin_description (xine_t *self,
+					       const char *plugin_id);
+const char *xine_get_post_plugin_description  (xine_t *self,
+					       const char *plugin_id);
 
 /* get lists of available audio and video output plugins */
 const char *const *xine_list_audio_output_plugins (xine_t *self) ;
 const char *const *xine_list_video_output_plugins (xine_t *self) ;
 
+/* get list of available demultiplexor plugins */
+const char *const *xine_list_demuxer_plugins(xine_t *self);
+
+/* get list of available input plugins */
+const char *const *xine_list_input_plugins(xine_t *self);
+
+/* get list of available subpicture plugins */
+const char *const *xine_list_spu_plugins(xine_t *self);
+
+/* get list of available audio and video decoder plugins */
+const char *const *xine_list_audio_decoder_plugins(xine_t *self);
+const char *const *xine_list_video_decoder_plugins(xine_t *self);
+
+/* unload unused plugins */
+void xine_plugins_garbage_collector(xine_t *self);
 
 /*********************************************************************
  * visual specific gui <-> xine engine communication                 *
  *********************************************************************/
 
-/* talk to video output driver */
+#ifndef XINE_DISABLE_DEPRECATED_FEATURES
+/* talk to video output driver - old method */
 int    xine_gui_send_vo_data (xine_stream_t *self,
 			      int type, void *data);
+#endif
+
+/* new (preferred) method to talk to video driver. */
+int    xine_port_send_gui_data (xine_video_port_t *vo,
+			        int type, void *data);
 
 typedef struct {
 
@@ -926,11 +1154,60 @@ typedef struct {
 } x11_visual_t;
 
 /*
- * "type" constants for xine_gui_send_vo_data (...)
+ * this is the visual data struct any fb gui
+ * may supply to the xine_open_video_driver call
+ * ("data" parameter) to get frame_output_cd calls
  */
 
+typedef struct {
+
+ void (*frame_output_cb) (void *user_data,
+			   int video_width, int video_height,
+			   double video_pixel_aspect,
+			   int *dest_x, int *dest_y,
+			   int *dest_width, int *dest_height,
+			   double *dest_pixel_aspect,
+			   int *win_x, int *win_y);
+
+  void             *user_data;
+
+} fb_visual_t;
+
+#ifdef WIN32
+/*
+ * this is the visual data struct any win32 gui should supply
+ * (pass this to init_video_out_plugin or the xine_load_video_output_plugin
+ * utility function)
+ */
+
+typedef struct {
+
+  HWND      WndHnd;     /* handle of window associated with primary surface */
+  HINSTANCE HInst;      /* handle of windows application instance */
+  RECT      WndRect;    /* rect of window client points translated to screen 
+                         * cooridnates */
+  int       FullScreen; /* is window fullscreen */
+  HBRUSH    Brush;      /* window brush for background color */
+  COLORREF  ColorKey;   /* window brush color key */
+
+} win32_visual_t;
+
+/*
+ * constants for gui_data_exchange's data_type parameter
+ */
+
+#define GUI_WIN32_MOVED_OR_RESIZED	0
+
+#endif /* WIN32 */
+
+/*
+ * "type" constants for xine_port_send_gui_data(...)
+ */
+
+#ifndef XINE_DISABLE_DEPRECATED_FEATURES
 /* xevent *data */
-#define XINE_GUI_SEND_COMPLETION_EVENT       1
+#define XINE_GUI_SEND_COMPLETION_EVENT       1 /* DEPRECATED */
+#endif
 
 /* Drawable data */
 #define XINE_GUI_SEND_DRAWABLE_CHANGED       2
@@ -949,6 +1226,9 @@ typedef struct {
 /* XVisualInfo **data */
 #define XINE_GUI_SEND_SELECT_VISUAL          8
 
+/* Gui is about to destroy drawable */
+#define XINE_GUI_SEND_WILL_DESTROY_DRAWABLE  9
+
 
 /*********************************************************************
  * xine health check stuff                                           *
@@ -1010,7 +1290,11 @@ struct xine_cfg_entry_s {
   /* type string */
   char            *str_value;
   char            *str_default;
+#ifndef XINE_DISABLE_DEPRECATED_FEATURES
   char            *str_sticky;
+#else
+  void            *dummy;
+#endif
 
   /* common to range, enum, num, bool: */
   int              num_value;
@@ -1023,7 +1307,10 @@ struct xine_cfg_entry_s {
   /* type enum specific: */
   char           **enum_values;
 
-  /* help info for the user */
+  /* help info for the user (UTF-8)
+   * the help string must be word wrapped by the frontend.
+   * it might contain \n to mark paragraph breaks.
+   */
   const char      *description;
   const char      *help;
 
@@ -1033,6 +1320,11 @@ struct xine_cfg_entry_s {
 			        20 => expert */
 
   /* callback function and data for live changeable values */
+  /* some config entries will take effect immediately, although they
+   * do not have a callback registered; such values will have some
+   * non-NULL dummy value in callback_data; so if you want to check,
+   * if a config change will require restarting xine, check for
+   * callback_data == NULL */
   xine_config_cb_t callback;
   void            *callback_data;
 
@@ -1143,10 +1435,13 @@ void xine_config_reset (xine_t *self);
 #define XINE_EVENT_UI_SET_TITLE           3 /* request title display change in ui */
 #define XINE_EVENT_UI_MESSAGE             4 /* message (dialog) for the ui to display */
 #define XINE_EVENT_FRAME_FORMAT_CHANGE    5 /* e.g. aspect ratio change during dvd playback */
-#define XINE_EVENT_AUDIO_LEVEL            6 /* report current audio level (l/r) */
+#define XINE_EVENT_AUDIO_LEVEL            6 /* report current audio level (l/r/mute) */
 #define XINE_EVENT_QUIT                   7 /* last event sent when stream is disposed */
 #define XINE_EVENT_PROGRESS               8 /* index creation/network connections */
 #define XINE_EVENT_MRL_REFERENCE          9 /* demuxer->frontend: MRL reference(s) for the real stream */
+#define XINE_EVENT_UI_NUM_BUTTONS        10 /* number of buttons for interactive menus */
+#define XINE_EVENT_SPU_BUTTON            11 /* the mouse pointer enter/leave a button */
+#define XINE_EVENT_DROPPED_FRAMES        12 /* number of dropped frames is too high */
 
 /* input events coming from frontend */
 #define XINE_EVENT_INPUT_MOUSE_BUTTON   101
@@ -1180,6 +1475,52 @@ void xine_config_reset (xine_t *self);
 #define XINE_EVENT_INPUT_NUMBER_9       129
 #define XINE_EVENT_INPUT_NUMBER_10_ADD  130
 
+/* specific event types */
+#define XINE_EVENT_SET_V4L2             200
+#define XINE_EVENT_PVR_SAVE             201
+#define XINE_EVENT_PVR_REPORT_NAME      202
+#define XINE_EVENT_PVR_REALTIME         203
+#define XINE_EVENT_PVR_PAUSE            204
+#define XINE_EVENT_SET_MPEG_DATA        205
+
+/* VDR specific event types */
+#define XINE_EVENT_VDR_RED              300
+#define XINE_EVENT_VDR_GREEN            301
+#define XINE_EVENT_VDR_YELLOW           302
+#define XINE_EVENT_VDR_BLUE             303
+#define XINE_EVENT_VDR_PLAY             304
+#define XINE_EVENT_VDR_PAUSE            305
+#define XINE_EVENT_VDR_STOP             306
+#define XINE_EVENT_VDR_RECORD           307
+#define XINE_EVENT_VDR_FASTFWD          308
+#define XINE_EVENT_VDR_FASTREW          309
+#define XINE_EVENT_VDR_POWER            310
+#define XINE_EVENT_VDR_CHANNELPLUS      311
+#define XINE_EVENT_VDR_CHANNELMINUS     312
+#define XINE_EVENT_VDR_SCHEDULE         313
+#define XINE_EVENT_VDR_CHANNELS         314
+#define XINE_EVENT_VDR_TIMERS           315
+#define XINE_EVENT_VDR_RECORDINGS       316
+#define XINE_EVENT_VDR_SETUP            317
+#define XINE_EVENT_VDR_COMMANDS         318
+#define XINE_EVENT_VDR_BACK             319
+#define XINE_EVENT_VDR_USER1            320
+#define XINE_EVENT_VDR_USER2            321
+#define XINE_EVENT_VDR_USER3            322
+#define XINE_EVENT_VDR_USER4            323
+#define XINE_EVENT_VDR_USER5            324
+#define XINE_EVENT_VDR_USER6            325
+#define XINE_EVENT_VDR_USER7            326
+#define XINE_EVENT_VDR_USER8            327
+#define XINE_EVENT_VDR_USER9            328
+#define XINE_EVENT_VDR_VOLPLUS          329
+#define XINE_EVENT_VDR_VOLMINUS         330
+#define XINE_EVENT_VDR_MUTE             331
+#define XINE_EVENT_VDR_AUDIO            332
+/* some space for further keys */
+#define XINE_EVENT_VDR_SETVIDEOWINDOW   350
+#define XINE_EVENT_VDR_FRAMESIZECHANGED 351
+
 /*
  * xine event struct
  */
@@ -1212,13 +1553,48 @@ typedef struct {
   char                str[256]; /* might be longer */
 } xine_ui_data_t;
 
+/*
+ * Send messages to UI. used mostly to report errors.
+ */
+typedef struct {
+  /*
+   * old xine-ui versions expect xine_ui_data_t type.
+   * this struct is added for compatibility.
+   */
+  xine_ui_data_t      compatibility;
+
+  /* See XINE_MSG_xxx for defined types. */
+  int                 type;
+
+  /* defined types are provided with a standard explanation.
+   * note: zero means no explanation.
+   */
+  int                 explanation; /* add to struct address to get a valid (char *) */
+
+  /* parameters are zero terminated strings */
+  int                 num_parameters;
+  int                 parameters;  /* add to struct address to get a valid (char *) */
+
+  /* where messages are stored, will be longer
+   *
+   * this field begins with the message text itself (\0-terminated),
+   * followed by (optional) \0-terminated parameter strings
+   * the end marker is \0 \0
+   */
+  char                messages[1];
+} xine_ui_message_data_t;
+
+
 /*
  * notify frame format change
  */
 typedef struct {
   int                 width;
   int                 height;
+  /* these are aspect codes as defined in MPEG2, because this
+   * is only used for DVD playback, pan_scan is a boolean flag */
   int                 aspect;
+  int                 pan_scan;
 } xine_format_change_data_t;
 
 /*
@@ -1226,7 +1602,8 @@ typedef struct {
  */
 typedef struct {
   int                 left;
-  int                 right;  /* 0..255 */
+  int                 right;  /* 0..100 % */
+  int                 mute;
 } xine_audio_level_data_t;
 
 /*
@@ -1248,8 +1625,19 @@ typedef struct {
  *
  * mrl references should be played in the same order they are received, just
  * after the current stream finishes.
- * alternative playlists may be provided and should be used in case of
- * failure of the primary playlist.
+ * alternative entries may be provided and should be used in case of
+ * failure of the primary stream (the one with alternative=0).
+ *
+ * sample playlist:
+ *   1) http://something/something.ram
+ *     1a) rtsp://something/realsomething1.rm (alternative=0)
+ *     1b) pnm://something/realsomething1.rm (alternative=1)
+ *   2) http://another/another.avi
+ *
+ * 1 and 2 are the original items on this playlist. 1a and 1b were received
+ * by events (they are the mrl references enclosed in 1). 1a is played after 
+ * receiving the finished event from 1. note: 1b is usually ignored, it should
+ * only be used in case 1a fails to open.
  */
 typedef struct {
   int                 alternative; /* alternative playlist number, usually 0 */
@@ -1257,6 +1645,140 @@ typedef struct {
 } xine_mrl_reference_data_t;
 
 
+/* 
+ * configuration options for video4linux-like input plugins
+ */
+typedef struct {
+  /* input selection */
+  int input;                    /* select active input from card */ 
+  int channel;                  /* channel number */
+  int radio;                    /* ask for a radio channel */
+  uint32_t frequency;           /* frequency divided by 62.5KHz or 62.5 Hz */
+  uint32_t transmission;        /* The transmission standard. */
+
+  /* video parameters */
+  uint32_t framerate_numerator; /* framerate as numerator/denominator */
+  uint32_t framerate_denominator;
+  uint32_t framelines;          /* Total lines per frame including blanking */
+  uint64_t standard_id;         /* One of the V4L2_STD_* values */
+  uint32_t colorstandard;       /* One of the V4L2_COLOR_STD_* values */
+  uint32_t colorsubcarrier;     /* The color subcarrier frequency */
+  int frame_width;              /* scaled frame width */
+  int frame_height;             /* scaled frame height */
+
+  /* let some spare space so we can add new fields without breaking
+   * binary api compatibility. 
+   */
+  uint32_t spare[20];
+
+  /* used by pvr plugin */
+  int32_t session_id;           /* -1 stops pvr recording */
+     
+} xine_set_v4l2_data_t;
+
+/*
+ * configuration options for plugins that can do a kind of mpeg encoding
+ * note: highly experimental api :)
+ */
+typedef struct {
+
+  /* mpeg2 parameters */
+  int bitrate_vbr;              /* 1 = vbr, 0 = cbr */
+  int bitrate_mean;             /* mean (target) bitrate in kbps*/
+  int bitrate_peak;             /* peak (max) bitrate in kbps */
+  int gop_size;                 /* GOP size in frames */
+  int gop_closure;              /* open/closed GOP */
+  int b_frames;                 /* number of B frames to use */
+  int aspect_ratio;             /* XINE_VO_ASPECT_xxx */
+  
+  /* let some spare space so we can add new fields without breaking
+   * binary api compatibility.
+   */
+  uint32_t spare[20];
+
+} xine_set_mpeg_data_t;
+
+typedef struct {
+  int                 direction; /* 0 leave, 1 enter */
+  int32_t             button; /* button number */
+} xine_spu_button_t;
+
+#ifdef XINE_ENABLE_EXPERIMENTAL_FEATURES
+
+/* 
+ * ask pvr to save (ie. do not discard) the current session
+ * see comments on input_pvr.c to understand how it works.
+ */
+typedef struct {
+  /* mode values:
+   * -1 = do nothing, just set the name
+   * 0 = truncate current session and save from now on
+   * 1 = save from last sync point
+   * 2 = save everything on current session
+   */
+  int mode; 
+  int id;
+  char name[256]; /* name for saving, might be longer */
+} xine_pvr_save_data_t;
+
+typedef struct {
+  /* mode values:
+   * 0 = non realtime
+   * 1 = realtime
+   */
+  int mode; 
+} xine_pvr_realtime_t;
+
+typedef struct {
+  /* mode values:
+   * 0 = playing
+   * 1 = paused
+   */
+  int mode;
+} xine_pvr_pause_t;
+
+#endif
+
+/* event XINE_EVENT_DROPPED_FRAMES is generated if libxine detects a
+ * high number of dropped frames (above configured thresholds). it can
+ * be used by the front end to warn about performance problems.
+ */
+typedef struct {
+  /* these values are given for 1000 frames delivered */
+  /* (that is, divide by 10 to get percentages) */
+  int skipped_frames;
+  int skipped_threshold;
+  int discarded_frames;
+  int discarded_threshold;
+} xine_dropped_frames_t;
+
+
+/*
+ * Defined message types for XINE_EVENT_UI_MESSAGE
+ * This is the mechanism to report async errors from engine.
+ *
+ * If frontend knows about the XINE_MSG_xxx type it may safely 
+ * ignore the 'explanation' field and provide it's own custom
+ * dialog to the 'parameters'.
+ *
+ * right column specifies the usual parameters.
+ */
+
+#define XINE_MSG_NO_ERROR		0  /* (messages to UI)   */
+#define XINE_MSG_GENERAL_WARNING	1  /* (warning message)  */
+#define XINE_MSG_UNKNOWN_HOST		2  /* (host name)        */
+#define XINE_MSG_UNKNOWN_DEVICE		3  /* (device name)      */
+#define XINE_MSG_NETWORK_UNREACHABLE	4  /* none               */
+#define XINE_MSG_CONNECTION_REFUSED	5  /* (host name)        */
+#define XINE_MSG_FILE_NOT_FOUND		6  /* (file name or mrl) */
+#define XINE_MSG_READ_ERROR		7  /* (device/file/mrl)  */
+#define XINE_MSG_LIBRARY_LOAD_ERROR	8  /* (library/decoder)  */
+#define XINE_MSG_ENCRYPTED_SOURCE	9  /* none               */
+#define XINE_MSG_SECURITY		10 /* (security message) */
+#define XINE_MSG_AUDIO_OUT_UNAVAILABLE	11 /* none               */
+#define XINE_MSG_PERMISSION_ERROR       12 /* (file name or mrl) */
+#define XINE_MSG_FILE_EMPTY             13 /* file is empty      */
+
 /* opaque xine_event_queue_t */
 typedef struct xine_event_queue_s xine_event_queue_t;
 
@@ -1329,10 +1851,14 @@ void xine_event_send (xine_stream_t *stream, const xine_event_t *event);
 /* yellow text, black border, transparent background */
 #define XINE_TEXTPALETTE_YELLOW_BLACK_TRANSPARENT   3
 
+#define XINE_OSD_CAP_FREETYPE2 0x0001 /* freetype2 support compiled in     */
+#define XINE_OSD_CAP_UNSCALED  0x0002 /* unscaled overlays supp. by vo drv */
+
 typedef struct xine_osd_s xine_osd_t;
 
 xine_osd_t *xine_osd_new           (xine_stream_t *self, int x, int y,
 				    int width, int height);
+uint32_t    xine_osd_get_capabilities (xine_osd_t *self);
 void        xine_osd_draw_point    (xine_osd_t *self, int x, int y, int color);
 
 void        xine_osd_draw_line     (xine_osd_t *self, int x1, int y1,
@@ -1340,27 +1866,33 @@ void        xine_osd_draw_line     (xine_osd_t *self, int x1, int y1,
 void        xine_osd_draw_rect     (xine_osd_t *self, int x1, int y1,
 				    int x2, int y2,
 				    int color, int filled );
+/*  x1 and y1 specifies the upper left corner of the text to be rendered */
 void        xine_osd_draw_text     (xine_osd_t *self, int x1, int y1,
 				    const char *text, int color_base);
+void        xine_osd_draw_bitmap   (xine_osd_t *self, uint8_t *bitmap,
+				    int x1, int y1, int width, int height,
+				    uint8_t *palette_map);
+/* for freetype2 fonts the height is the maximum height for the whole font and not
+ * only for the specified text */
 void        xine_osd_get_text_size (xine_osd_t *self, const char *text,
 				    int *width, int *height);
-void        xine_osd_set_font      (xine_osd_t *self, const char *fontname,
+/* with freetype2 support compiled in, you can also specify a font file
+   as 'fontname' here */
+int         xine_osd_set_font      (xine_osd_t *self, const char *fontname,
 				    int size);
+/* 
+ * specifying encoding of texts
+ *   ""   ... means current locale encoding (default)
+ *   NULL ... means latin1
+ */
+void        xine_osd_set_encoding(xine_osd_t *self, const char *encoding);
 /* set position were overlay will be blended */
 void        xine_osd_set_position  (xine_osd_t *self, int x, int y);
 void        xine_osd_show          (xine_osd_t *self, int64_t vpts);
+void        xine_osd_show_unscaled (xine_osd_t *self, int64_t vpts);
 void        xine_osd_hide          (xine_osd_t *self, int64_t vpts);
 /* empty drawing area */
 void        xine_osd_clear         (xine_osd_t *self);
-/*
- * close osd rendering engine
- * loaded fonts are unloaded
- * osd objects are closed
- */
-void        xine_osd_free          (xine_osd_t *self);
-void        xine_osd_set_palette   (xine_osd_t *self,
-				    const uint32_t *const color,
-				    const uint8_t *const trans );
 /*
  * set on existing text palette
  * (-1 to set used specified palette)
@@ -1369,7 +1901,12 @@ void        xine_osd_set_palette   (xine_osd_t *self,
  * palette. The OSD palette is then modified starting at this
  * color index, up to the size of the text palette.
  *
- * Use OSD_TEXT1, OSD_TEXT2, ... for some preasssigned color indices.
+ * Use OSD_TEXT1, OSD_TEXT2, ... for some preassigned color indices.
+ *
+ * these palettes are not really well working the true type fonts.
+ * First thing is that these fonts can not have a border. So the best
+ * result you can get by loading a linearly blending palette from the
+ * background (at index 0) to the forground color (at index 10)
  */
 void        xine_osd_set_text_palette (xine_osd_t *self,
 				       int palette_number,
@@ -1377,14 +1914,39 @@ void        xine_osd_set_text_palette (xine_osd_t *self,
 /* get palette (color and transparency) */
 void        xine_osd_get_palette   (xine_osd_t *self, uint32_t *color,
 				    uint8_t *trans);
+void        xine_osd_set_palette   (xine_osd_t *self,
+				    const uint32_t *const color,
+				    const uint8_t *const trans );
+/*
+ * close osd rendering engine
+ * loaded fonts are unloaded
+ * osd objects are closed
+ */
+void        xine_osd_free          (xine_osd_t *self);
 
 
+#ifndef XINE_DISABLE_DEPRECATED_FEATURES
+
 /*********************************************************************
  * TV-mode API, to make it possible to use nvtvd to view movies      *
  *********************************************************************/
 
+/* These functions are just dummies to maintain API compatibility.
+ * You should use libnvtvsimple in your frontend instead. */
+
+typedef enum {
+	XINE_TVSYSTEM_PAL = 0,
+	XINE_TVSYSTEM_NTSC
+} xine_tvsystem;
+
 /* connect to nvtvd server and save current TV and X settings */
-void xine_tvmode_init (xine_t *self);
+int xine_tvmode_init (xine_t *self);
+
+/* Turn tvmode on/off (1/0)*/
+int xine_tvmode_use(xine_t *self, int use_tvmode);
+
+/* Set which tv system to use: XINE_TVSYSTEM_PAL or XINE_TVSYSTEM_NTSC */
+void xine_tvmode_set_tvsystem(xine_t *self, xine_tvsystem system);
 
 /* try to change TV state if enabled
  *   type select 'regular' (0) or 'TV' (1) state
@@ -1403,8 +1965,9 @@ void xine_tvmode_size (xine_t *self, int *width, int *height,
 
 /* restore old TV and X settings and close nvtvd connection */
 void xine_tvmode_exit (xine_t *self);
-       
-        
+
+#endif
+
 #ifdef __cplusplus
 }
 #endif
