@@ -14,14 +14,17 @@ if [ `expr $RPMVERSION` -lt 100 ]; then
 fi
 
 if [ `expr $RPMVERSION` -lt 400 ]; then
-  RPM_BA="rpm -ba -ta ./xine-lib-1-beta4.tar.gz"
-  RPM_BB="rpm -bb -ta ./xine-lib-1-beta4.tar.gz"
+  RPM_BA="rpm -ba -ta ./xine-lib-1.0.2.tar.gz"
+  RPM_BB="rpm -bb -ta ./xine-lib-1.0.2.tar.gz"
+elif [ `expr $RPMVERSION` -lt 420 ]; then
+  RPM_BA="rpm -ta ./xine-lib-1.0.2.tar.gz -ba"
+  RPM_BB="rpm -ta ./xine-lib-1.0.2.tar.gz -bb"
 else
-  RPM_BA="rpm -ta ./xine-lib-1-beta4.tar.gz -ba"
-  RPM_BB="rpm -ta ./xine-lib-1-beta4.tar.gz -bb"
+  RPM_BA="rpmbuild -ta ./xine-lib-1.0.2.tar.gz -ba"
+  RPM_BB="rpmbuild -ta ./xine-lib-1.0.2.tar.gz -bb"
 fi
-
-##VERSION="1.0.0"
+      
+##VERSION="1.0.2"
 
 echo "Creating tarball..."
 rm -f config.cache && ./autogen.sh && make dist
@@ -30,7 +33,7 @@ mkdir rpms
 
 echo "*****************************************************"
 echo
-echo "building rpm for xine-lib 1-beta4"
+echo "building rpm for xine-lib 1.0.2"
 echo 
 echo "current architecture:pentium"
 echo "rpms will be copied to ./rpms directory"
@@ -41,13 +44,13 @@ export XINE_BUILD=i586-pc-linux-gnu
 
 eval $RPM_BA
 
-mv /usr/src/redhat/SRPMS/libxine-1_beta4-$DATE.src.rpm ./rpms/
-mv /usr/src/redhat/RPMS/i386/$PKGNAME-1_beta4-$DATE.i386.rpm ./rpms/$PKGNAME-1_beta4-$DATE.i586.rpm
-mv /usr/src/redhat/RPMS/i386/$PKGNAME-devel-1_beta4-$DATE.i386.rpm ./rpms/$PKGNAME-devel-1_beta4-$DATE.i586.rpm
+mv /usr/src/redhat/SRPMS/libxine-1.0.2-$DATE.src.rpm ./rpms/
+mv /usr/src/redhat/RPMS/i386/$PKGNAME-1.0.2-$DATE.i386.rpm ./rpms/$PKGNAME-1.0.2-$DATE.i586.rpm
+mv /usr/src/redhat/RPMS/i386/$PKGNAME-devel-1.0.2-$DATE.i386.rpm ./rpms/$PKGNAME-devel-1.0.2-$DATE.i586.rpm
 
 echo "*****************************************************"
 echo
-echo "building rpm for xine-lib 1-beta4"
+echo "building rpm for xine-lib 1.0.2"
 echo 
 echo "current architecture:pentiumpro"
 echo "rpms will be copied to ./rpms directory"
@@ -58,11 +61,11 @@ export XINE_BUILD=i686-pc-linux-gnu
 
 eval $RPM_BB
 
-mv /usr/src/redhat/RPMS/i386/$PKGNAME-1_beta4-$DATE.i386.rpm ./rpms/$PKGNAME-1_beta4-$DATE.i686.rpm
+mv /usr/src/redhat/RPMS/i386/$PKGNAME-1.0.2-$DATE.i386.rpm ./rpms/$PKGNAME-1.0.2-$DATE.i686.rpm
 
 echo "*****************************************************"
 echo
-echo "building rpm for xine-lib 1-beta4"
+echo "building rpm for xine-lib 1.0.2"
 echo 
 echo "current architecture:k6"
 echo "rpms will be copied to ./rpms directory"
@@ -73,11 +76,11 @@ export XINE_BUILD=k6-pc-linux-gnu
 
 eval $RPM_BB
 
-mv /usr/src/redhat/RPMS/i386/$PKGNAME-1_beta4-$DATE.i386.rpm ./rpms/$PKGNAME-1_beta4-$DATE.k6.rpm
+mv /usr/src/redhat/RPMS/i386/$PKGNAME-1.0.2-$DATE.i386.rpm ./rpms/$PKGNAME-1.0.2-$DATE.k6.rpm
 
 echo "*****************************************************"
 echo
-echo "building rpm for xine-lib 1-beta4"
+echo "building rpm for xine-lib 1.0.2"
 echo 
 echo "current architecture:k7"
 echo "rpms will be copied to ./rpms directory"
@@ -88,6 +91,6 @@ export XINE_BUILD=athlon-pc-linux-gnu
 
 eval $RPM_BB
 
-mv /usr/src/redhat/RPMS/i386/$PKGNAME-1_beta4-$DATE.i386.rpm ./rpms/$PKGNAME-1_beta4-$DATE.k7.rpm
+mv /usr/src/redhat/RPMS/i386/$PKGNAME-1.0.2-$DATE.i386.rpm ./rpms/$PKGNAME-1.0.2-$DATE.k7.rpm
 
 echo "Done."
