@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2003 the xine project
+ * Copyright (C) 2001-2005 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -16,13 +16,14 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
+ */
+
+/*
  * AC3 File Demuxer by Mike Melanson (melanson@pcisys.net)
  * This demuxer detects raw AC3 data in a file and shovels AC3 data
  * directly to the AC3 decoder.
  *
- * $Id: demux_ac3.c,v 1.3 2003/01/29 03:36:04 tmmm Exp $
- *
+ * $Id: demux_ac3.c,v 1.16.2.1 2005/07/17 21:49:40 miguelfreitas Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -36,50 +37,48 @@
 #include <stdlib.h>
 #include <ctype.h>
 
+#define LOG_MODULE "demux_ac3"
+#define LOG_VERBOSE
+/*
+#define LOG
+*/
+
 #include "xine_internal.h"
 #include "xineutils.h"
 #include "demux.h"
 #include "buffer.h"
 #include "bswap.h"
+#include "group_audio.h"
 
-#define AC3_PREAMBLE_BYTES 5
+#define DATA_TAG 0x61746164
+#define PEAK_SIZE 7056        /* 3 raw cd frames */
 
 typedef struct {
-
   demux_plugin_t       demux_plugin;
 
   xine_stream_t       *stream;
-
-  config_values_t     *config;
-
   fifo_buffer_t       *video_fifo;
   fifo_buffer_t       *audio_fifo;
-
   input_plugin_t      *input;
-
   int                  status;
-  int                  seek_flag;
 
-  int                  sample_rate;  /* 0 = 48KHz, 1 = 44.1KHz, 2 = 32KHz */
+  int                  seek_flag;
+  int                  sample_rate;
   int                  frame_size;
   int                  running_time;
 
-  char                 last_mrl[1024];
+  off_t                data_start;
+
+  uint32_t             buf_type;
+
 } demux_ac3_t;
 
 typedef struct {
-
   demux_class_t     demux_class;
-
-  /* class-wide, global variables here */
-
-  xine_t           *xine;
-  config_values_t  *config;
 } demux_ac3_class_t;
 
 /* borrow some knowledge from the AC3 decoder */
-struct frmsize_s
-{
+struct frmsize_s {
   uint16_t bit_rate;
   uint16_t frm_size[3];
 };
@@ -128,87 +127,208 @@ static const struct frmsize_s frmsizecod_tbl[64] =
 
 /* returns 1 if the AC3 file was opened successfully, 0 otherwise */
 static int open_ac3_file(demux_ac3_t *this) {
+  int i;
+  int offset = 0;
+  int peak_size = 0;
+  int spdif_mode = 0;
+  uint32_t syncword = 0;
+  uint32_t blocksize;
+  uint8_t peak[PEAK_SIZE];
 
-  unsigned char preamble[AC3_PREAMBLE_BYTES];
+  lprintf("open_ac3_file\n");
 
-  /* check if the sync mark matches up */
-  this->input->seek(this->input, 0, SEEK_SET);
-  if (this->input->read(this->input, preamble, AC3_PREAMBLE_BYTES) != 
-    AC3_PREAMBLE_BYTES)
+  /* block based demuxer (i.e. cdda) will only allow reads in block
+  * sized pieces */
+  blocksize = this->input->get_blocksize(this->input);
+  if (blocksize && INPUT_IS_SEEKABLE(this->input)) {
+    int read;
+
+    this->input->seek(this->input, 0, SEEK_SET);
+    while (peak_size < PEAK_SIZE) {
+      read = this->input->read(this->input, &peak[peak_size], blocksize);
+      if (read)
+        peak_size += read;
+      else
+        break;
+    }
+    this->input->seek(this->input, 0, SEEK_SET);
+  } else {
+    peak_size = MAX_PREVIEW_SIZE;
+
+    if (_x_demux_read_header(this->input, peak, peak_size) != peak_size)
+      return 0;
+  }
+
+  lprintf("peak size: %d\n", peak_size);
+
+  /* Check for wav header, as we'll handle AC3 with a wav header shoved
+  * on the front for CD burning */
+  if ((peak[0]  == 'R') && (peak[1]  == 'I') && (peak[2]  == 'F') && 
+       (peak[3]  == 'F') && (peak[8]  == 'W') && (peak[9]  == 'A') &&
+       (peak[10] == 'V') && (peak[11] == 'E') && (peak[12] == 'f') &&
+       (peak[13] == 'm') && (peak[14] == 't') && (peak[15] == ' ')) {
+    /* Check this looks like a cd audio wav */
+    unsigned int audio_type;
+    xine_waveformatex *wave = (xine_waveformatex *) &peak[20];
+
+    _x_waveformatex_le2me(wave);
+    audio_type = _x_formattag_to_buf_audio(wave->wFormatTag);
+
+    if ((audio_type != BUF_AUDIO_LPCM_LE) || (wave->nChannels != 2) ||
+         (wave->nSamplesPerSec != 44100) || (wave->wBitsPerSample != 16))
+      return 0;
+
+    lprintf("looks like a cd audio wav file\n");
+
+    /* Find the data chunk */
+    offset = 20 + LE_32(&peak[16]);
+    while (offset < peak_size-8) {
+      unsigned int chunk_tag = LE_32(&peak[offset]);
+      unsigned int chunk_size = LE_32(&peak[offset+4]);
+
+      if (chunk_tag == DATA_TAG) {
+        offset += 8;
+        lprintf("found the start of the data at offset %d\n", offset);
+        break;
+      } else
+        offset += chunk_size;
+    }
+  }
+
+  /* Look for a valid AC3 sync word */
+  for (i=offset; i<peak_size; i++) {
+    if ((syncword & 0xffff) == 0x0b77) {
+      this->data_start = i-2;
+      lprintf("found AC3 syncword at offset %d\n", i-2);
+      break;
+    }
+
+    if ((syncword == 0x72f81f4e) && (peak[i] == 0x01)) {
+      spdif_mode = 1;
+      this->data_start = i+4;
+      lprintf("found AC3 SPDIF header at offset %d\n", i-4);
+      break;
+    }
+
+    syncword = (syncword << 8) | peak[i];
+  }
+
+  if (i >= peak_size-2)
     return 0;
 
-  if ((preamble[0] != 0x0B) ||
-      (preamble[1] != 0x77))
-    return 0;
-
-  this->sample_rate = preamble[4] >> 6;
-  if (this->sample_rate > 2)
-    return 0;
-
-  this->frame_size = 
-    frmsizecod_tbl[preamble[4] & 0x3F].frm_size[this->sample_rate];
-
-  /* convert the sample rate to a more useful number */
-  if (this->sample_rate == 0)
-    this->sample_rate = 48000;
-  else if (this->sample_rate == 1)
+  if (spdif_mode) {
     this->sample_rate = 44100;
-  else
-    this->sample_rate = 32000;
+    this->frame_size = 256*6*4;
+    this->buf_type = BUF_AUDIO_DNET;
+  } else {
+    int fscod, frmsizecod;
 
-  this->running_time = this->input->get_length(this->input);
+    fscod = peak[this->data_start+4] >> 6;
+    frmsizecod = peak[this->data_start+4] & 0x3F;
+
+    if ((fscod > 2) || (frmsizecod > 37))
+      return 0;
+
+    this->frame_size = frmsizecod_tbl[frmsizecod].frm_size[fscod] * 2;
+
+    /* convert the sample rate to a more useful number */
+    switch (fscod) {
+      case 0:
+        this->sample_rate = 48000;
+        break;
+      case 1:
+        this->sample_rate = 44100;
+        break;
+      default:
+        this->sample_rate = 32000;
+        break;
+    }
+
+    /* Look for a second sync word */
+    if ((this->data_start+this->frame_size+1 >= peak_size) ||
+        (peak[this->data_start+this->frame_size] != 0x0b) ||
+        (peak[this->data_start+this->frame_size + 1] != 0x77)) {
+      return 0;
+    }
+
+    lprintf("found second AC3 sync word\n");
+
+    this->buf_type = BUF_AUDIO_A52;
+  }
+
+  this->running_time = this->input->get_length(this->input) -
+                       this->data_start;
   this->running_time /= this->frame_size;
-  this->running_time *= (90000 / 1000) * (256 * 3);
+  this->running_time *= (90000 / 1000) * (256 * 6);
   this->running_time /= this->sample_rate;
 
+  lprintf("sample rate: %d\n", this->sample_rate);
+  lprintf("frame size: %d\n", this->frame_size);
+  lprintf("running time: %d\n", this->running_time);
+
   return 1;
 }
 
 static int demux_ac3_send_chunk (demux_plugin_t *this_gen) {
-
   demux_ac3_t *this = (demux_ac3_t *) this_gen;
+
   buf_element_t *buf = NULL;
-  off_t current_file_pos;
+  off_t current_stream_pos;
   int64_t audio_pts;
-  int bytes_read;
   int frame_number;
+  uint32_t blocksize;
 
-  current_file_pos = this->input->get_current_pos(this->input);
-  frame_number = current_file_pos / this->frame_size;
+  current_stream_pos = this->input->get_current_pos(this->input);
+  frame_number = current_stream_pos / this->frame_size;
 
-  /* 
-   * Each frame represents 256 new audio samples according to the a52 spec.
+  /*
+   * Each frame represents 256*6 new audio samples according to the a52 spec.
    * Thus, the pts computation should work something like:
    *
-   *   pts = frame #  *  256 samples        1 sec        90000 pts
-   *                     -----------  *  -----------  *  ---------
-   *                       1 frame       sample rate       1 sec
-   *
-   * For some reason, the computation only works out correctly
-   * assuming 256 * 3 = 768 samples/frame.
+   *   pts = frame #  *  256*6 samples        1 sec        90000 pts
+   *                     -------------  *  -----------  *  ---------
+   *                        1 frame        sample rate       1 sec
    */
   audio_pts = frame_number;
   audio_pts *= 90000;
-  audio_pts *= 256 * 3;
+  audio_pts *= 256 * 6;
   audio_pts /= this->sample_rate;
 
   if (this->seek_flag) {
-    xine_demux_control_newpts(this->stream, audio_pts, 0);
+    _x_demux_control_newpts(this->stream, audio_pts, BUF_FLAG_SEEK);
     this->seek_flag = 0;
   }
 
-  buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-  buf->type = BUF_AUDIO_A52;
-  bytes_read = this->input->read(this->input, buf->content, this->frame_size);
-  if (bytes_read == 0) {
+  blocksize = this->input->get_blocksize(this->input);
+  if (blocksize) {
+    buf = this->input->read_block(this->input, this->audio_fifo,
+                                  blocksize);
+    if (!buf) {
+      this->status = DEMUX_FINISHED;
+      return this->status;
+    }
+  } else {
+    buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
+    buf->size = this->input->read(this->input, buf->content, 
+                                  this->frame_size);
+  }
+
+  if (buf->size == 0) {
     buf->free_buffer(buf);
     this->status = DEMUX_FINISHED;
     return this->status;
   }
-  buf->size = bytes_read;
 
-  buf->extra_info->input_pos = current_file_pos;
-  buf->extra_info->input_length = this->input->get_length(this->input);
+  if (buf->size == 0) {
+    buf->free_buffer(buf);
+    this->status = DEMUX_FINISHED;
+    return this->status;
+  }
+
+  buf->type = this->buf_type;
+  if( this->input->get_length (this->input) )
+    buf->extra_info->input_normpos = (int)( (double) current_stream_pos * 
+                                     65535 / this->input->get_length (this->input) );
   buf->extra_info->input_time = audio_pts / 90;
   buf->pts = audio_pts;
   buf->decoder_flags |= BUF_FLAG_FRAME_END;
@@ -219,7 +339,6 @@ static int demux_ac3_send_chunk (demux_plugin_t *this_gen) {
 }
 
 static void demux_ac3_send_headers(demux_plugin_t *this_gen) {
-
   demux_ac3_t *this = (demux_ac3_t *) this_gen;
   buf_element_t *buf;
 
@@ -229,26 +348,37 @@ static void demux_ac3_send_headers(demux_plugin_t *this_gen) {
   this->status = DEMUX_OK;
 
   /* load stream information */
-  this->stream->stream_info[XINE_STREAM_INFO_HAS_VIDEO] = 0;
-  this->stream->stream_info[XINE_STREAM_INFO_HAS_AUDIO] = 1;
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_HAS_VIDEO, 0);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_HAS_AUDIO, 1);
 
   /* send start buffers */
-  xine_demux_control_start(this->stream);
+  _x_demux_control_start(this->stream);
 
   /* send init info to decoders */
   if (this->audio_fifo) {
     buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-    buf->type = BUF_AUDIO_A52;
-    buf->decoder_flags = BUF_FLAG_HEADER;
+    buf->type = this->buf_type;
+    buf->decoder_flags = BUF_FLAG_HEADER|BUF_FLAG_FRAME_END;
     buf->size = 0;
     this->audio_fifo->put (this->audio_fifo, buf);
   }
 }
 
 static int demux_ac3_seek (demux_plugin_t *this_gen,
-                           off_t start_pos, int start_time) {
+                           off_t start_pos, int start_time, int playing) {
 
   demux_ac3_t *this = (demux_ac3_t *) this_gen;
+  start_pos = (off_t) ( (double) start_pos / 65535 *
+              this->input->get_length (this->input) );
+
+  this->seek_flag = 1;
+  this->status = DEMUX_OK;
+  _x_demux_flush_engine (this->stream);
+
+  /* if input is non-seekable, do not proceed with the rest of this
+   * seek function */
+  if (!INPUT_IS_SEEKABLE(this->input))
+    return this->status;
 
   /* divide the requested offset integer-wise by the frame alignment and
    * multiply by the frame alignment to determine the new starting block */
@@ -256,10 +386,6 @@ static int demux_ac3_seek (demux_plugin_t *this_gen,
   start_pos *= this->frame_size;
   this->input->seek(this->input, start_pos, SEEK_SET);
 
-  this->seek_flag = 1;
-  this->status = DEMUX_OK;
-  xine_demux_flush_engine (this->stream);
-
   return this->status;
 }
 
@@ -277,7 +403,6 @@ static int demux_ac3_get_status (demux_plugin_t *this_gen) {
 
 /* return the approximate length in milliseconds */
 static int demux_ac3_get_stream_length (demux_plugin_t *this_gen) {
-
   demux_ac3_t *this = (demux_ac3_t *) this_gen;
 
   return this->running_time;
@@ -293,16 +418,10 @@ static int demux_ac3_get_optional_data(demux_plugin_t *this_gen,
 }
 
 static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *stream,
-                                   input_plugin_t *input_gen) {
+                                   input_plugin_t *input) {
 
-  input_plugin_t *input = (input_plugin_t *) input_gen;
   demux_ac3_t   *this;
 
-  if (! (input->get_capabilities(input) & INPUT_CAP_SEEKABLE)) {
-    printf(_("demux_ac3.c: input not seekable, can not handle!\n"));
-    return NULL;
-  }
-
   this         = xine_xmalloc (sizeof (demux_ac3_t));
   this->stream = stream;
   this->input  = input;
@@ -313,8 +432,6 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   this->demux_plugin.dispose           = demux_ac3_dispose;
   this->demux_plugin.get_status        = demux_ac3_get_status;
   this->demux_plugin.get_stream_length = demux_ac3_get_stream_length;
-  this->demux_plugin.get_video_frame   = NULL;
-  this->demux_plugin.got_video_frame_cb= NULL;
   this->demux_plugin.get_capabilities  = demux_ac3_get_capabilities;
   this->demux_plugin.get_optional_data = demux_ac3_get_optional_data;
   this->demux_plugin.demux_class       = class_gen;
@@ -323,6 +440,19 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   switch (stream->content_detection_method) {
 
+  case METHOD_BY_EXTENSION: {
+    char *extensions, *mrl;
+
+    mrl = input->get_mrl (input);
+    extensions = class_gen->get_extensions (class_gen);
+
+    if (!_x_demux_check_extension (mrl, extensions)) {
+      free (this);
+      return NULL;
+    }
+  }
+  /* falling through is intended */
+
   case METHOD_BY_CONTENT:
   case METHOD_EXPLICIT:
 
@@ -333,38 +463,16 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   break;
 
-  case METHOD_BY_EXTENSION: {
-    char *ending, *mrl;
-
-    mrl = input->get_mrl (input);
-
-    ending = strrchr(mrl, '.');
-
-    if (!ending) {
-      free (this);
-      return NULL;
-    }
-
-    if (strncasecmp (ending, ".ac3", 4)) {
-      free (this);
-      return NULL;
-    }
-
-  }
-  break;
-
   default:
     free (this);
     return NULL;
   }
 
-  strncpy (this->last_mrl, input->get_mrl (input), 1024);
-
   return &this->demux_plugin;
 }
 
 static char *get_description (demux_class_t *this_gen) {
-  return "Raw AC3 file demux plugin";
+  return "Raw AC3 demux plugin";
 }
 
 static char *get_identifier (demux_class_t *this_gen) {
@@ -380,19 +488,15 @@ static char *get_mimetypes (demux_class_t *this_gen) {
 }
 
 static void class_dispose (demux_class_t *this_gen) {
-
   demux_ac3_class_t *this = (demux_ac3_class_t *) this_gen;
 
   free (this);
 }
 
 void *demux_ac3_init_plugin (xine_t *xine, void *data) {
-
   demux_ac3_class_t     *this;
 
-  this         = xine_xmalloc (sizeof (demux_ac3_class_t));
-  this->config = xine->config;
-  this->xine   = xine;
+  this = xine_xmalloc (sizeof (demux_ac3_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
   this->demux_class.get_description = get_description;
