@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: demux_asf.c,v 1.167 2004/12/24 17:43:38 hadess Exp $
+ * $Id: demux_asf.c,v 1.167.2.3 2005/07/17 21:49:40 miguelfreitas Exp $
  *
  * demultiplexer for asf streams
  *
@@ -44,7 +44,6 @@
 /*
 #define LOG
 */
-
 #include "xine_internal.h"
 #include "demux.h"
 #include "xineutils.h"
@@ -98,6 +97,10 @@ typedef struct {
 
   int                 payload_size;
 
+  /* palette handling */
+  int               palette_count;
+  palette_entry_t   palette[256];
+  
 } asf_stream_t;
 
 typedef struct demux_asf_s {
@@ -246,11 +249,11 @@ static uint64_t get_le64 (demux_asf_t *this) {
   return LE_64(buf);
 }
 
-static int get_guid_id (demux_asf_t *this, GUID g) {
+static int get_guid_id (demux_asf_t *this, GUID *g) {
   int i;
 
   for (i = 1; i < GUID_END; i++) {
-    if (!memcmp(&g, &guids[i].guid, sizeof(GUID))) {
+    if (!memcmp(g, &guids[i].guid, sizeof(GUID))) {
       lprintf ("GUID: %s\n", guids[i].name);
       return i;
     }
@@ -259,8 +262,8 @@ static int get_guid_id (demux_asf_t *this, GUID g) {
   xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
 	  "demux_asf: unknown GUID: 0x%" PRIx32 ", 0x%" PRIx16 ", 0x%" PRIx16 ", "
 	  "{ 0x%" PRIx8 ", 0x%" PRIx8 ", 0x%" PRIx8 ", 0x%" PRIx8 ", 0x%" PRIx8 ", 0x%" PRIx8 ", 0x%" PRIx8 ", 0x%" PRIx8 " }\n",
-	  g.Data1, g.Data2, g.Data3,
-	  g.Data4[0], g.Data4[1], g.Data4[2], g.Data4[3], g.Data4[4], g.Data4[5], g.Data4[6], g.Data4[7]);
+	  g->Data1, g->Data2, g->Data3,
+	  g->Data4[0], g->Data4[1], g->Data4[2], g->Data4[3], g->Data4[4], g->Data4[5], g->Data4[6], g->Data4[7]);
 
   return GUID_ERROR;
 }
@@ -277,7 +280,7 @@ static int get_guid (demux_asf_t *this) {
     g.Data4[i] = get_byte(this);
   }
    
-  return get_guid_id(this, g);
+  return get_guid_id(this, &g);
 }
 
 static void get_str16_nolen(demux_asf_t *this, int len,
@@ -361,10 +364,24 @@ static void asf_send_video_header (demux_asf_t *this, int stream) {
   buf->decoder_info[0] = 0;
   buf->size = asf_stream->bih_size;
   memcpy (buf->content, bih, buf->size);
-  buf->type = this->streams[stream].buf_type ;
+  buf->type = this->streams[stream].buf_type;
 
   this->video_fifo->put (this->video_fifo, buf);
 
+
+  /* send off the palette, if there is one */
+  if (asf_stream->palette_count) {
+    xprintf(this->stream->xine, XINE_VERBOSITY_LOG, 
+	    "demux_asf: stream %d, palette : %d entries\n", stream, asf_stream->palette_count);
+    buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
+    buf->decoder_flags = BUF_FLAG_SPECIAL|BUF_FLAG_HEADER;
+    buf->decoder_info[1] = BUF_SPECIAL_PALETTE;
+    buf->decoder_info[2] = asf_stream->palette_count;
+    buf->decoder_info_ptr[2] = &asf_stream->palette;
+    buf->size = 0;
+    buf->type = this->streams[stream].buf_type;
+    this->video_fifo->put (this->video_fifo, buf);
+  }
 }
 
 static int asf_read_header (demux_asf_t *this) {
@@ -396,7 +413,7 @@ static int asf_read_header (demux_asf_t *this) {
 
           guid = get_guid(this);
           file_size = get_le64(this); /* file size */
-
+        
           get_le64(this); /* creation time */
           this->packet_count = get_le64(this); /* nb packets */
   
@@ -450,7 +467,10 @@ static int asf_read_header (demux_asf_t *this) {
               break;
 
             default:
-              goto fail;
+	      lprintf("unexpected GUID\n");
+	      pos2 = this->input->get_current_pos (this->input);
+	      this->input->seek (this->input, gsize - (pos2 - pos1 + 24), SEEK_CUR);
+              continue;
           }
 
           guid = get_guid(this);
@@ -532,6 +552,7 @@ static int asf_read_header (demux_asf_t *this) {
             uint16_t i;
             uint32_t width;
             uint32_t height;
+	    int j;
 
             width = get_le32(this); /* width */
             height = get_le32(this); /* height */
@@ -563,6 +584,31 @@ static int asf_read_header (demux_asf_t *this) {
               this->streams[this->num_streams].frag_offset  = 0;
               this->streams[this->num_streams].defrag       = 0;
 
+	      /* load the palette, if there is one */
+	      asf_stream->palette_count = asf_stream->bih->biClrUsed;
+
+	      lprintf ("palette_count: %d\n", asf_stream->palette_count);
+	      if (asf_stream->palette_count > 256) {
+		lprintf ("number of colors exceeded 256 (%d)", asf_stream->palette_count);
+		asf_stream->palette_count = 256;
+	      }
+	      if ((asf_stream->bih_size - sizeof(xine_bmiheader)) >= (asf_stream->palette_count * 4)) {
+		/* load the palette from the end of the strf chunk */
+		for (j = 0; j < asf_stream->palette_count; j++) {
+		  asf_stream->palette[j].b = *((uint8_t *)asf_stream->bih + sizeof(xine_bmiheader) + j * 4 + 0);
+		  asf_stream->palette[j].g = *((uint8_t *)asf_stream->bih + sizeof(xine_bmiheader) + j * 4 + 1);
+		  asf_stream->palette[j].r = *((uint8_t *)asf_stream->bih + sizeof(xine_bmiheader) + j * 4 + 2);
+		}
+	      } else {
+		/* generate a greyscale palette */
+		asf_stream->palette_count = 256;
+		for (j = 0; j < asf_stream->palette_count; j++) {
+		  asf_stream->palette[j].r = j;
+		  asf_stream->palette[j].g = j;
+		  asf_stream->palette[j].b = j;
+		}
+	      }
+
             } else
               xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
 		       "demux_asf: invalid bih_size received (%d), v_stream ignored.\n", i );
@@ -978,6 +1024,7 @@ static int asf_parse_packet_align(demux_asf_t *this) {
   mod = (current_pos - this->first_packet_pos) % this->packet_size;
   this->packet_size_left = mod ? this->packet_size - mod : 0;
   packet_pos = current_pos + this->packet_size_left;
+
   if (this->packet_size_left) {
     lprintf("last packet is not finished, %d bytes\n", this->packet_size_left);
     current_pos = this->input->seek (this->input, packet_pos, SEEK_SET);
@@ -988,9 +1035,9 @@ static int asf_parse_packet_align(demux_asf_t *this) {
   this->packet_size_left = 0;
   
   /* check packet_count */
-  packet_num = (current_pos - this->first_packet_pos) / this->packet_size;
+  packet_num = (packet_pos - this->first_packet_pos) / this->packet_size;
   lprintf("packet_num=%lld, packet_count=%lld\n", packet_num, this->packet_count);
-  if (packet_num == this->packet_count) {
+  if (packet_num >= this->packet_count) {
     /* end of payload data */
     current_pos = this->input->get_current_pos (this->input);
     lprintf("end of payload data, current_pos=%lld\n", current_pos);
@@ -1046,10 +1093,29 @@ static int asf_parse_packet_ecd(demux_asf_t *this, uint32_t  *p_hdr_size) {
         *p_hdr_size += read_size;
 
       } else {
-
-        /* skip invalid packet */
-        xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "demux_asf: skip invalid packet: %2X\n", ecd_flags);
-        this->input->seek (this->input, this->packet_size - *p_hdr_size, SEEK_CUR);
+        GUID *guid = (GUID *)buf;
+      
+        /* check if it's a new stream */
+        buf[0] = ecd_flags;
+        if (this->input->read (this->input, buf + 1, 15) != 15) {
+          this->status = DEMUX_FINISHED;
+          return 1;
+        }
+        *p_hdr_size += 15;
+        guid->Data1 = LE_32(buf);
+        guid->Data2 = LE_16(buf + 4);
+        guid->Data3 = LE_16(buf + 6);
+        if (get_guid_id(this, guid) == GUID_ASF_HEADER) {
+          lprintf("new asf header detected\n");
+          _x_demux_control_end(this->stream, 0);
+          if (demux_asf_send_headers_common(this))
+            return 1;
+        } else {
+      
+          /* skip invalid packet */
+          xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "demux_asf: skip invalid packet: %2X\n", ecd_flags);
+          this->input->seek (this->input, this->packet_size - *p_hdr_size, SEEK_CUR);
+        }
         invalid_packet = 1;
       }
     }
@@ -1282,7 +1348,7 @@ static int asf_parse_packet_compressed_payload(demux_asf_t *this,
 
   *timestamp = frag_offset;
   if (*timestamp)
-  	*timestamp -= this->preroll;
+    *timestamp -= this->preroll;
 
   frag_offset = 0;
   get_byte (this); s_hdr_size += 1;
@@ -2235,6 +2301,6 @@ demuxer_info_t demux_info_asf = {
  
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_DEMUX, 25, "asf", XINE_VERSION_CODE, &demux_info_asf, init_class },
+  { PLUGIN_DEMUX, 26, "asf", XINE_VERSION_CODE, &demux_info_asf, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
