@@ -19,7 +19,7 @@
  */
 
 /*
- * $Id: demux_avi.c,v 1.214 2004/12/25 17:52:10 f1rmb Exp $
+ * $Id: demux_avi.c,v 1.214.2.3 2005/04/20 17:21:02 mroi Exp $
  *
  * demultiplexer for avi streams
  *
@@ -485,7 +485,7 @@ static int start_pos_stopper(demux_avi_t *this, void *data) {
     maxframe--;
   }
   return -1;
-}
+} 
 
 /* Use this one to ensure that a video frame with the given timestamp
  * is in the index. */
@@ -620,7 +620,7 @@ static int idx_grow(demux_avi_t *this, int (*stopper)(demux_avi_t *, void *),
         if ((data[0] == audio->audio_tag[0]) &&
             (data[1] == audio->audio_tag[1])) {
           off_t pos = chunk_pos + AVI_HEADER_SIZE;
-  
+
           valid_chunk = 1;
           /* VBR streams (hack from mplayer) */
           if (audio->wavex && audio->wavex->nBlockAlign) {
@@ -639,7 +639,7 @@ static int idx_grow(demux_avi_t *this, int (*stopper)(demux_avi_t *, void *),
       }
     }
     if (!valid_chunk) {
-      xine_log(this->stream->xine, XINE_LOG_MSG, _("demux_avi: invalid avi chunk \"%c%c%c%c\" at pos %" PRIiMAX "\n"), data[0], data[1], data[2], data[3], (intmax_t)chunk_pos);
+      xine_log(this->stream->xine, XINE_LOG_MSG, _("demux_avi: invalid avi chunk \"%c%c%c%c\" at pos %" PRIdMAX "\n"), data[0], data[1], data[2], data[3], (intmax_t)chunk_pos);
     }
     chunk_pos = this->input->seek(this->input, this->idx_grow.nexttagoffset, SEEK_SET);
     if (chunk_pos != this->idx_grow.nexttagoffset) {
@@ -687,6 +687,7 @@ static video_index_entry_t *video_cur_index_entry(demux_avi_t *this) {
 static audio_index_entry_t *audio_cur_index_entry(demux_avi_t *this,
     avi_audio_t *AVI_A) {
 
+  lprintf("posc: %d, chunks: %d\n", AVI_A->audio_posc, AVI_A->audio_idx.audio_chunks);
   if (AVI_A->audio_posc >= AVI_A->audio_idx.audio_chunks) {
     /* We don't have enough chunks; see if the file's bigger yet. */
     if (idx_grow(this, audio_pos_stopper, AVI_A) < 0) {
@@ -782,7 +783,7 @@ static avi_t *AVI_init(demux_avi_t *this) {
     this->idx_grow.nexttagoffset = this->input->get_current_pos(this->input);
 
     if (this->input->read(this->input, data,8) != 8 ) {
-      xprintf(this->stream->xine, XINE_VERBOSITY_LOG, "failed to read 8 bytes at pos %" PRIiMAX "\n", (intmax_t)this->idx_grow.nexttagoffset);
+      xprintf(this->stream->xine, XINE_VERBOSITY_LOG, "failed to read 8 bytes at pos %" PRIdMAX "\n", (intmax_t)this->idx_grow.nexttagoffset);
       break; /* We assume it's EOF */
     }
 
@@ -846,7 +847,7 @@ static avi_t *AVI_init(demux_avi_t *this) {
       }
     }
     if (next_chunk != this->input->seek(this->input, next_chunk, SEEK_SET)) {
-      xine_log (this->stream->xine, XINE_LOG_MSG, _("demux_avi: failed to seek to the next chunk (pos %" PRIiMAX ")\n"), (intmax_t)next_chunk);
+      xine_log (this->stream->xine, XINE_LOG_MSG, _("demux_avi: failed to seek to the next chunk (pos %" PRIdMAX ")\n"), (intmax_t)next_chunk);
       break;  /* probably slow seek */
     }
   }
@@ -1056,6 +1057,7 @@ static avi_t *AVI_init(demux_avi_t *this) {
 #endif
       }
 
+      this->has_index = 1;
       if (lasttag == 1) {
          /* V I D E O */
          AVI->video_superindex = superindex;
@@ -1192,6 +1194,7 @@ static avi_t *AVI_init(demux_avi_t *this) {
         off_t pos = LE_32(AVI->idx[i] + 8) + ioff;
         uint32_t len = LE_32(AVI->idx[i] + 12);
         uint32_t flags = LE_32(AVI->idx[i] + 4);
+
         if (video_index_append(AVI, pos, len, flags) == -1) {
           ERR_EXIT(AVI_ERR_NO_MEM) ;
         }
@@ -1229,12 +1232,11 @@ static avi_t *AVI_init(demux_avi_t *this) {
       int k = 0, audtr = 0;
       uint32_t nrEntries = 0;
       int nvi, nai[MAX_AUDIO_STREAMS];
-      uint64_t totb, tot[MAX_AUDIO_STREAMS];
 
       xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
                "demux_avi: This is an OpenDML stream\n");
       nvi = 0;
-      for(audtr=0; audtr<AVI->n_audio; ++audtr) nai[audtr] = tot[audtr] = 0; 
+      for(audtr=0; audtr<AVI->n_audio; ++audtr) nai[audtr] = 0; 
 
       /* ************************ */
       /* VIDEO */
@@ -1305,27 +1307,28 @@ static avi_t *AVI_init(demux_avi_t *this) {
       /* ************************ */
       lprintf("audio tracks\n");
       for(audtr=0; audtr<AVI->n_audio; ++audtr) {
+        avi_audio_t *audio = AVI->audio[audtr];
 
         k = 0;
-        if (!AVI->audio[audtr]->audio_superindex) {
+        if (!audio->audio_superindex) {
           xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
                    "demux_avi: Warning: cannot read audio index for track %d\n", audtr);
           continue;
         }
-        for (j=0; j<AVI->audio[audtr]->audio_superindex->nEntriesInUse; j++) {
+        for (j=0; j<audio->audio_superindex->nEntriesInUse; j++) {
 
           /* read from file */
-          chunk_start = en = malloc (AVI->audio[audtr]->audio_superindex->aIndex[j].dwSize+hdrl_len);
+          chunk_start = en = malloc (audio->audio_superindex->aIndex[j].dwSize+hdrl_len);
 
-          if (this->input->seek(this->input, AVI->audio[audtr]->audio_superindex->aIndex[j].qwOffset, SEEK_SET) == (off_t)-1) {
-            lprintf("cannot seek to 0x%" PRIx64 "\n", AVI->audio[audtr]->audio_superindex->aIndex[j].qwOffset);
+          if (this->input->seek(this->input, audio->audio_superindex->aIndex[j].qwOffset, SEEK_SET) == (off_t)-1) {
+            lprintf("cannot seek to 0x%" PRIx64 "\n", audio->audio_superindex->aIndex[j].qwOffset);
             free(chunk_start);
             continue;
           }
 
-          if (this->input->read(this->input, en, AVI->audio[audtr]->audio_superindex->aIndex[j].dwSize+hdrl_len) <= 0) {
+          if (this->input->read(this->input, en, audio->audio_superindex->aIndex[j].dwSize+hdrl_len) <= 0) {
             lprintf("cannot read from offset 0x%" PRIx64 "; broken (incomplete) file?\n",
-              AVI->audio[audtr]->audio_superindex->aIndex[j].qwOffset);
+              audio->audio_superindex->aIndex[j].qwOffset);
             free(chunk_start);
             continue;
           }
@@ -1347,19 +1350,25 @@ static avi_t *AVI_init(demux_avi_t *this) {
 
             pos = offset + LE_32(en); en += 4;
             len = odml_len(en); en += 4;
-            totb = tot[audtr];
-            tot[audtr] += len;
-            audio_index_append(AVI, audtr, pos, len, totb, k);
+          
+            /* VBR streams (hack from mplayer) */
+            if (audio->wavex && audio->wavex->nBlockAlign) {
+              audio->block_no += (len + audio->wavex->nBlockAlign - 1) /
+                                 audio->wavex->nBlockAlign;
+            } else {
+              audio->block_no += 1;
+            }
 
+            audio_index_append(AVI, audtr, pos, len, audio->audio_tot, audio->block_no);
 
 #ifdef DEBUG_ODML
             /*
             printf("[%d:%d] POS 0x%llX len=%d offset (%llx) (%ld)\n", k, audtr,
             pos, (int)len,
-            offset, (long)AVI->audio[audtr]->audio_superindex->aIndex[j].dwSize);
+            offset, (long)audio->audio_superindex->aIndex[j].dwSize);
             */
 #endif
-
+            audio->audio_tot += len;
             ++k;
           }
           free(chunk_start);
@@ -1422,24 +1431,9 @@ static int AVI_read_audio(demux_avi_t *this, avi_audio_t *AVI_A, char *audbuf,
 
   /* lprintf ("avi audio package len: %d\n", AVI_A->audio_index[AVI_A->audio_posc].len); */
 
-
-  while(bytes > 0) {
-    left = aie->len - AVI_A->audio_posb;
-    if(left == 0) {
-      AVI_A->audio_posc++;
-      AVI_A->audio_posb = 0;
-      aie = audio_cur_index_entry(this, AVI_A);
-      if (!aie) {
-        this->AVI_errno = AVI_ERR_NO_IDX;
-        return -1;
-      }
-      if (nr > 0) {
-        *buf_flags = BUF_FLAG_FRAME_END;
-        return nr;
-      }
-      left = aie->len - AVI_A->audio_posb;
-    }
-    if(bytes < left)
+  left = aie->len - AVI_A->audio_posb;
+  while ((bytes > 0) && (left > 0)) {
+    if (bytes < left)
       todo = bytes;
     else
       todo = left;
@@ -1455,13 +1449,16 @@ static int AVI_read_audio(demux_avi_t *this, avi_audio_t *AVI_A, char *audbuf,
     bytes -= todo;
     nr    += todo;
     AVI_A->audio_posb += todo;
+    left = aie->len - AVI_A->audio_posb;
   }
 
-  left = aie->len - AVI_A->audio_posb;
-  if (left == 0)
+  if (left == 0) {
+    AVI_A->audio_posc++;
+    AVI_A->audio_posb = 0;
     *buf_flags = BUF_FLAG_FRAME_END;
-  else
+  } else {
     *buf_flags = 0;
+  }
 
   return nr;
 }
@@ -1480,25 +1477,10 @@ static int AVI_read_video(demux_avi_t *this, avi_t *AVI, char *vidbuf,
 
   nr = 0; /* total number of bytes read */
 
-  while(bytes>0) {
-
-    left = vie->len - AVI->video_posb;
-
-    if(left==0) {
-      AVI->video_posf++;
-      AVI->video_posb = 0;
-      vie = video_cur_index_entry(this);
-      if (!vie) {
-        this->AVI_errno = AVI_ERR_NO_IDX;
-        return -1;
-      }
-      if (nr>0) {
-        *buf_flags = BUF_FLAG_FRAME_END;
-        return nr;
-      }
-      left = vie->len - AVI->video_posb;
-    }
-    if(bytes<left)
+  left = vie->len - AVI->video_posb;
+  
+  while ((bytes > 0) && (left > 0)) {
+    if (bytes < left)
       todo = bytes;
     else
       todo = left;
@@ -1514,14 +1496,16 @@ static int AVI_read_video(demux_avi_t *this, avi_t *AVI, char *vidbuf,
     bytes -= todo;
     nr    += todo;
     AVI->video_posb += todo;
+    left = vie->len - AVI->video_posb;
   }
 
-  left = vie->len - AVI->video_posb;
-  if (left==0)
+  if (left == 0) {
+    AVI->video_posf++;
+    AVI->video_posb = 0;
     *buf_flags = BUF_FLAG_FRAME_END;
-  else
+  } else {
     *buf_flags = 0;
-
+  }
   return nr;
 }
 
@@ -1532,6 +1516,8 @@ static int demux_avi_next (demux_avi_t *this, int decoder_flags) {
   buf_element_t *buf = NULL;
   int64_t        audio_pts, video_pts;
   int            do_read_video = (this->avi->n_audio == 0);
+  int            video_sent = 0;
+  int            audio_sent = 0;
 
   lprintf("begin\n");
   
@@ -1540,23 +1526,21 @@ static int demux_avi_next (demux_avi_t *this, int decoder_flags) {
    * the end of the stream. */
   if (this->avi->video_idx.video_frames <= this->avi->video_posf) {
     if (idx_grow(this, video_pos_stopper, NULL) < 0) {
-      lprintf("end: idx_grow video_pos_stopper\n");
-      return 0;
+      lprintf("end of stream\n");
     }
   }
-
+ 
   for (i = 0; i < this->avi->n_audio; i++) {
     avi_audio_t *audio = this->avi->audio[i];
 
     if (!this->no_audio &&
         (audio->audio_idx.audio_chunks <= audio->audio_posc)) {
       if (idx_grow(this, audio_pos_stopper, this->avi->audio[i]) < 0) {
-        lprintf("end: idx_grow audio_pos_stopper\n");
-        return 0;
+        lprintf("end of stream\n");
       }
     }
   }
-
+  
   video_pts = get_video_pts (this, this->avi->video_posf);
 
   for (i=0; i < this->avi->n_audio; i++) {
@@ -1566,8 +1550,8 @@ static int demux_avi_next (demux_avi_t *this, int decoder_flags) {
     /* The tests above mean aie should never be NULL, but just to be
      * safe. */
     if (!aie) {
-      lprintf("end: aie == NULL\n");
-      return 0;
+      lprintf("aie == NULL\n");
+      continue;
     }
 
     audio_pts =
@@ -1588,22 +1572,28 @@ static int demux_avi_next (demux_avi_t *this, int decoder_flags) {
 
       if (buf->size < 0) {
         buf->free_buffer (buf);
-        lprintf("end: audio buf->size < 0\n");
-        return 0;
+        lprintf("audio buf->size < 0\n");
+      } else {
+
+        buf->type = audio->audio_type | i;
+        buf->extra_info->input_time = audio_pts / 90;
+        if( this->input->get_length (this->input) )
+          buf->extra_info->input_normpos = (int)( (double) this->input->get_current_pos (this->input) * 
+                                           65535 / this->input->get_length (this->input) );
+  
+        check_newpts (this, buf->pts, PTS_AUDIO);
+        this->audio_fifo->put (this->audio_fifo, buf);
+        
+        audio_sent++;
       }
-
-      buf->type = audio->audio_type | i;
-      buf->extra_info->input_time = audio_pts / 90;
-      if( this->input->get_length (this->input) )
-        buf->extra_info->input_normpos = (int)( (double) this->input->get_current_pos (this->input) * 
-                                         65535 / this->input->get_length (this->input) );
-
-      check_newpts (this, buf->pts, PTS_AUDIO);
-      this->audio_fifo->put (this->audio_fifo, buf);
     } else
       do_read_video = 1;
   }
 
+  if (audio_sent == 0) {
+    do_read_video = 1;
+  }
+  
   if (do_read_video) {
 
     buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
@@ -1632,19 +1622,25 @@ static int demux_avi_next (demux_avi_t *this, int decoder_flags) {
 
     if (buf->size < 0) {
       buf->free_buffer (buf);
-      lprintf("end: video buf->size < 0\n");
-      return 0;
+      lprintf("video buf->size < 0\n");
+    } else {
+
+      /*
+        lprintf ("adding buf %d to video fifo, decoder_info[0]: %d\n",
+        buf, buf->decoder_info[0]);
+      */
+  
+      check_newpts (this, buf->pts, PTS_VIDEO);
+      this->video_fifo->put (this->video_fifo, buf);
+      video_sent++;
     }
-
-    /*
-      lprintf ("adding buf %d to video fifo, decoder_info[0]: %d\n",
-      buf, buf->decoder_info[0]);
-    */
-
-     check_newpts (this, buf->pts, PTS_VIDEO);
-     this->video_fifo->put (this->video_fifo, buf);
   }
 
+  if (!audio_sent && !video_sent) {
+    xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
+             "demux_avi: video and audio streams are ended\n");
+    return 0;
+  }
   return 1;
 }
 
@@ -1696,7 +1692,7 @@ static int get_chunk_header(demux_avi_t *this, uint32_t *len, int *audio_stream)
         return AVI_HEADER_AUDIO;
       }
     }
-    xine_log (this->stream->xine, XINE_LOG_MSG, _("demux_avi: invalid avi chunk \"%c%c%c%c\" at pos %" PRIiMAX "\n"), data[0], data[1], data[2], data[3], (intmax_t)this->input->get_current_pos(this->input));
+    xine_log (this->stream->xine, XINE_LOG_MSG, _("demux_avi: invalid avi chunk \"%c%c%c%c\" at pos %" PRIdMAX "\n"), data[0], data[1], data[2], data[3], (intmax_t)this->input->get_current_pos(this->input));
     return AVI_HEADER_UNKNOWN;
   }
   /* unreachable code */
@@ -2088,7 +2084,7 @@ static int demux_avi_seek_internal (demux_avi_t *this) {
    */
 
   xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
-          "start pos is %" PRIiMAX ", start time is %d\n", (intmax_t)start_pos,
+          "start pos is %" PRIdMAX ", start time is %d\n", (intmax_t)start_pos,
           start_time);
 
   /* Seek video.  We do a single idx_grow at the beginning rather than
@@ -2371,6 +2367,6 @@ demuxer_info_t demux_info_avi = {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_DEMUX, 25, "avi", XINE_VERSION_CODE, &demux_info_avi, init_class },
+  { PLUGIN_DEMUX, 26, "avi", XINE_VERSION_CODE, &demux_info_avi, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
