@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2004 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -16,13 +16,15 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
+ */
+
+/*
  * FLI File Demuxer by Mike Melanson (melanson@pcisys.net)
  * For information on the FLI format, as well as various traps to
  * avoid while programming a FLI decoder, visit:
  *   http://www.pcisys.net/~melanson/codecs/
  *
- * $Id: demux_fli.c,v 1.37 2003/01/23 16:12:02 miguelfreitas Exp $
+ * $Id: demux_fli.c,v 1.55.2.1 2005/04/20 17:21:03 mroi Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -42,30 +44,25 @@
 #include "bswap.h"
 
 #define FLI_HEADER_SIZE 128
+#define FLI_HEADER_SIZE_MC 12  /* header size for Magic Carpet game FLIs */
 #define FLI_FILE_MAGIC_1 0xAF11
 #define FLI_FILE_MAGIC_2 0xAF12
+#define FLI_FILE_MAGIC_3 0xAF13  /* for internal use only */
 #define FLI_CHUNK_MAGIC_1 0xF1FA
 #define FLI_CHUNK_MAGIC_2 0xF5FA
+#define FLI_MC_PTS_INC 6000  /* pts increment for Magic Carpet game FLIs */
 
 typedef struct {
-
   demux_plugin_t       demux_plugin;
 
   xine_stream_t       *stream;
-
-  config_values_t     *config;
-
   fifo_buffer_t       *video_fifo;
   fifo_buffer_t       *audio_fifo;
-
   input_plugin_t      *input;
-
-  off_t                start;
   int                  status;
 
   /* video information */
-  unsigned int         width;
-  unsigned int         height;
+  xine_bmiheader       bih;
   unsigned char        fli_header[FLI_HEADER_SIZE];
 
   /* playback info */
@@ -73,29 +70,19 @@ typedef struct {
   unsigned int         speed;
   unsigned int         frame_pts_inc;
   unsigned int         frame_count;
+  int64_t              pts_counter;
 
-  char                 last_mrl[1024];
-
-  off_t stream_len;
+  off_t                stream_len;
 } demux_fli_t;
 
 typedef struct {
-
   demux_class_t     demux_class;
-
-  /* class-wide, global variables here */
-
-  xine_t           *xine;
-  config_values_t  *config;
 } demux_fli_class_t;
 
 /* returns 1 if the FLI file was opened successfully, 0 otherwise */
 static int open_fli_file(demux_fli_t *this) {
 
-  /* read the whole header */
-  this->input->seek(this->input, 0, SEEK_SET);
-  if (this->input->read(this->input, this->fli_header, FLI_HEADER_SIZE) !=
-    FLI_HEADER_SIZE)
+  if (_x_demux_read_header(this->input, this->fli_header, FLI_HEADER_SIZE) != FLI_HEADER_SIZE)
     return 0;
 
   /* validate the file */
@@ -104,11 +91,29 @@ static int open_fli_file(demux_fli_t *this) {
       (this->magic_number != FLI_FILE_MAGIC_2))
     return 0;
 
+  /* file is qualified; skip over the signature bytes in the stream */
+  this->input->seek(this->input, FLI_HEADER_SIZE, SEEK_SET);
+
+  /* check if this is a special FLI file from Magic Carpet game */
+  if (LE_16(&this->fli_header[16]) == FLI_CHUNK_MAGIC_1) {
+    /* if the input is non-seekable, do not bother with playing the
+     * special file type */
+    if (INPUT_IS_SEEKABLE(this->input)) {
+      this->input->seek(this->input, FLI_HEADER_SIZE_MC, SEEK_SET);
+    } else {
+      return 0;
+    }
+
+    /* use a contrived internal FLI type, 0xAF13 */
+    this->magic_number = FLI_FILE_MAGIC_3;
+  }
+
   this->frame_count = LE_16(&this->fli_header[6]);
-  this->width = LE_16(&this->fli_header[8]);
-  this->height = LE_16(&this->fli_header[10]);
+  this->bih.biWidth = LE_16(&this->fli_header[8]);
+  this->bih.biHeight = LE_16(&this->fli_header[10]);
+
   this->speed = LE_32(&this->fli_header[16]);
-  if (this->magic_number == 0xAF11) {
+  if (this->magic_number == FLI_FILE_MAGIC_1) {
     /* 
      * in this case, the speed (n) is number of 1/70s ticks between frames:
      *
@@ -119,7 +124,7 @@ static int open_fli_file(demux_fli_t *this) {
      *  therefore, the frame pts increment = n * 1285.7
      */
      this->frame_pts_inc = this->speed * 1285.7;
-  } else {
+  } else if (this->magic_number == FLI_FILE_MAGIC_2) {
     /* 
      * in this case, the speed (n) is number of milliseconds between frames:
      *
@@ -130,28 +135,35 @@ static int open_fli_file(demux_fli_t *this) {
      *  therefore, the frame pts increment = n * 90
      */
      this->frame_pts_inc = this->speed * 90;
+  } else {
+    /* special case for Magic Carpet FLIs which don't carry speed info */
+    this->frame_pts_inc = FLI_MC_PTS_INC;
   }
 
   /* sanity check: the FLI file must have non-zero values for width, height,
    * and frame count */
-  if ((!this->width) || (!this->height) || (!this->frame_count))
+  if ((!this->bih.biWidth) || (!this->bih.biHeight) || (!this->frame_count))
     return 0;
 
+  if (this->magic_number == FLI_FILE_MAGIC_3)
+      this->bih.biSize = sizeof(xine_bmiheader) + FLI_HEADER_SIZE_MC;
+  else
+      this->bih.biSize = sizeof(xine_bmiheader) + FLI_HEADER_SIZE;
+
   return 1;
 }
 
 static int demux_fli_send_chunk(demux_plugin_t *this_gen) {
-
   demux_fli_t *this = (demux_fli_t *) this_gen;
+
   buf_element_t *buf = NULL;
   unsigned char fli_buf[6];
   unsigned int chunk_size;
   unsigned int chunk_magic;
-  int64_t pts_counter = 0;
   off_t current_file_pos;
 
   current_file_pos = this->input->get_current_pos(this->input);
-  
+
   /* get the chunk size nd magic number */
   if (this->input->read(this->input, fli_buf, 6) != 6) {
     this->status = DEMUX_FINISHED;
@@ -159,46 +171,56 @@ static int demux_fli_send_chunk(demux_plugin_t *this_gen) {
   }
   chunk_size = LE_32(&fli_buf[0]);
   chunk_magic = LE_16(&fli_buf[4]);
-  
-  /* rewind over the size and packetize the chunk */
-  this->input->seek(this->input, -6, SEEK_CUR);
-  
-  if ((chunk_magic == FLI_CHUNK_MAGIC_1) || 
+
+  if ((chunk_magic == FLI_CHUNK_MAGIC_1) ||
       (chunk_magic == FLI_CHUNK_MAGIC_2)) {
+
+    /* send a buffer with only the chunk header */
+    buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
+    buf->type = BUF_VIDEO_FLI;
+    if( this->stream_len )
+      buf->extra_info->input_normpos = (int)( (double) current_file_pos * 65535 / this->stream_len);
+    buf->extra_info->input_time = this->pts_counter / 90;
+    buf->pts = this->pts_counter;
+    buf->size = 6;
+    memcpy(buf->content, fli_buf, 6);
+    this->video_fifo->put(this->video_fifo, buf);
+
+    chunk_size -= 6;
+
     while (chunk_size) {
       buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
       buf->type = BUF_VIDEO_FLI;
-      buf->extra_info->input_pos = current_file_pos;
-      buf->extra_info->input_time = pts_counter / 90;
-      buf->extra_info->input_length = this->stream_len;
-      buf->pts = pts_counter;
-  
+      if( this->stream_len )
+        buf->extra_info->input_normpos = (int)( (double) current_file_pos * 65535 / this->stream_len);
+      buf->extra_info->input_time = this->pts_counter / 90;
+      buf->pts = this->pts_counter;
+
       if (chunk_size > buf->max_size)
         buf->size = buf->max_size;
       else
         buf->size = chunk_size;
       chunk_size -= buf->size;
-  
+
       if (this->input->read(this->input, buf->content, buf->size) !=
         buf->size) {
         buf->free_buffer(buf);
         this->status = DEMUX_FINISHED;
         break;
       }
-  
+
       if (!chunk_size)
         buf->decoder_flags |= BUF_FLAG_FRAME_END;
       this->video_fifo->put(this->video_fifo, buf);
     }
-    pts_counter += this->frame_pts_inc;
+    this->pts_counter += this->frame_pts_inc;
   } else
     this->input->seek(this->input, chunk_size, SEEK_CUR);
-  
+
   return this->status;
 }
 
 static void demux_fli_send_headers(demux_plugin_t *this_gen) {
-
   demux_fli_t *this = (demux_fli_t *) this_gen;
   buf_element_t *buf;
 
@@ -208,44 +230,41 @@ static void demux_fli_send_headers(demux_plugin_t *this_gen) {
   this->status = DEMUX_OK;
 
   /* load stream information */
-  this->stream->stream_info[XINE_STREAM_INFO_HAS_VIDEO] = 1;
-  this->stream->stream_info[XINE_STREAM_INFO_HAS_AUDIO] = 0;
-  this->stream->stream_info[XINE_STREAM_INFO_VIDEO_WIDTH] = this->width;
-  this->stream->stream_info[XINE_STREAM_INFO_VIDEO_HEIGHT] = this->height;
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_HAS_VIDEO, 1);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_HAS_AUDIO, 0);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_VIDEO_WIDTH, this->bih.biWidth);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_VIDEO_HEIGHT, this->bih.biHeight);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_FRAME_DURATION,
+    this->frame_pts_inc);
 
   /* send start buffers */
-  xine_demux_control_start(this->stream);
+  _x_demux_control_start(this->stream);
 
   /* send init info to FLI decoder */
   buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-  buf->decoder_flags = BUF_FLAG_HEADER;
-  buf->decoder_info[0] = 0;
-  buf->decoder_info[1] = this->frame_pts_inc;  /* initial video_step */
-  /* be a rebel and send the FLI header instead of the bih */
-  memcpy(buf->content, this->fli_header, FLI_HEADER_SIZE);
-  buf->size = FLI_HEADER_SIZE;
+  buf->decoder_flags = BUF_FLAG_HEADER|BUF_FLAG_STDHEADER|BUF_FLAG_FRAMERATE|
+                       BUF_FLAG_FRAME_END;
+  buf->decoder_info[0] = this->frame_pts_inc;  /* initial video_step */
+  buf->size = this->bih.biSize;
+  memcpy(buf->content, &this->bih, sizeof(xine_bmiheader) + this->bih.biSize);
   buf->type = BUF_VIDEO_FLI;
   this->video_fifo->put (this->video_fifo, buf);
 }
 
-static int demux_fli_seek (demux_plugin_t *this_gen,
-                           off_t start_pos, int start_time) {
-
+static int demux_fli_seek (demux_plugin_t *this_gen, off_t start_pos, int start_time, int playing) {
   demux_fli_t *this = (demux_fli_t *) this_gen;
 
   /* if thread is not running, initialize demuxer */
-  if( !this->stream->demux_thread_running ) {
+  if( !playing ) {
 
     /* send new pts */
-    xine_demux_control_newpts(this->stream, 0, 0);
+    _x_demux_control_newpts(this->stream, 0, 0);
 
-    this->status = DEMUX_OK;
-  
-    /* make sure to start just after the header */
-    this->input->seek(this->input, FLI_HEADER_SIZE, SEEK_SET);
-    this->stream_len = this->input->get_length(this->input);
+    this->status      = DEMUX_OK;
+    this->stream_len  = this->input->get_length(this->input);
+    this->pts_counter = 0;
   }
-  
+
   return this->status;
 }
 
@@ -262,7 +281,14 @@ static int demux_fli_get_status (demux_plugin_t *this_gen) {
 
 static int demux_fli_get_stream_length (demux_plugin_t *this_gen) {
 
-  return 0;
+  demux_fli_t *this = (demux_fli_t *) this_gen;
+  int64_t end_pts;
+
+  end_pts = this->frame_count;
+  end_pts *= this->frame_pts_inc;
+  end_pts /= 90;
+
+  return (int)end_pts;
 }
 
 static uint32_t demux_fli_get_capabilities(demux_plugin_t *this_gen) {
@@ -275,16 +301,10 @@ static int demux_fli_get_optional_data(demux_plugin_t *this_gen,
 }
 
 static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *stream,
-                                    input_plugin_t *input_gen) {
+                                    input_plugin_t *input) {
 
-  input_plugin_t *input = (input_plugin_t *) input_gen;
   demux_fli_t    *this;
 
-  if (! (input->get_capabilities(input) & INPUT_CAP_SEEKABLE)) {
-    printf(_("demux_fli.c: input not seekable, can not handle!\n"));
-    return NULL;
-  }
-
   this         = xine_xmalloc (sizeof (demux_fli_t));
   this->stream = stream;
   this->input  = input;
@@ -295,8 +315,6 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   this->demux_plugin.dispose           = demux_fli_dispose;
   this->demux_plugin.get_status        = demux_fli_get_status;
   this->demux_plugin.get_stream_length = demux_fli_get_stream_length;
-  this->demux_plugin.get_video_frame   = NULL;
-  this->demux_plugin.got_video_frame_cb= NULL;
   this->demux_plugin.get_capabilities  = demux_fli_get_capabilities;
   this->demux_plugin.get_optional_data = demux_fli_get_optional_data;
   this->demux_plugin.demux_class       = class_gen;
@@ -305,6 +323,19 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   switch (stream->content_detection_method) {
 
+  case METHOD_BY_EXTENSION: {
+    char *extensions, *mrl;
+
+    mrl = input->get_mrl (input);
+    extensions = class_gen->get_extensions (class_gen);
+
+    if (!_x_demux_check_extension (mrl, extensions)) {
+      free (this);
+      return NULL;
+    }
+  }
+  /* falling through is intended */
+
   case METHOD_BY_CONTENT:
   case METHOD_EXPLICIT:
 
@@ -315,40 +346,11 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   break;
 
-  case METHOD_BY_EXTENSION: {
-    char *ending, *mrl;
-
-    mrl = input->get_mrl (input);
-
-    ending = strrchr(mrl, '.');
-
-    if (!ending) {
-      free (this);
-      return NULL;
-    }
-
-    if (strncasecmp (ending, ".fli", 4) &&
-        strncasecmp (ending, ".flc", 4)) {
-      free (this);
-      return NULL;
-    }
-
-    if (!open_fli_file(this)) {
-      free (this);
-      return NULL;
-    }
-
-  }
-
-  break;
-
   default:
     free (this);
     return NULL;
   }
 
-  strncpy (this->last_mrl, input->get_mrl (input), 1024);
-
   return &this->demux_plugin;
 }
 
@@ -369,19 +371,15 @@ static char *get_mimetypes (demux_class_t *this_gen) {
 }
 
 static void class_dispose (demux_class_t *this_gen) {
-
   demux_fli_class_t *this = (demux_fli_class_t *) this_gen;
 
   free (this);
 }
 
 static void *init_plugin (xine_t *xine, void *data) {
-
   demux_fli_class_t     *this;
 
-  this         = xine_xmalloc (sizeof (demux_fli_class_t));
-  this->config = xine->config;
-  this->xine   = xine;
+  this = xine_xmalloc (sizeof (demux_fli_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
   this->demux_class.get_description = get_description;
@@ -396,9 +394,12 @@ static void *init_plugin (xine_t *xine, void *data) {
 /*
  * exported plugin catalog entry
  */
+demuxer_info_t demux_info_fli = {
+  10                       /* priority */
+};
 
 plugin_info_t xine_plugin_info[] = {
-  /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_DEMUX, 20, "fli", XINE_VERSION_CODE, NULL, init_plugin },
+  /* type, API, "name", version, special_info, init_function */
+  { PLUGIN_DEMUX, 26, "fli", XINE_VERSION_CODE, &demux_info_fli, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
