@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -16,7 +16,9 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
+ */
+
+/*
  * CIN File Demuxer by Mike Melanson (melanson@pcisys.net)
  * For more information regarding the Id CIN file format, visit:
  *   http://www.csse.monash.edu.au/~timf/
@@ -63,7 +65,7 @@
  *     - if any bytes exceed 63, do not shift the bytes at all before
  *       transmitting them to the video decoder
  *
- * $Id: demux_idcin.c,v 1.37 2003/01/25 03:04:15 tmmm Exp $
+ * $Id: demux_idcin.c,v 1.52 2004/06/13 21:28:53 miguelfreitas Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -76,11 +78,19 @@
 #include <string.h>
 #include <stdlib.h>
 
+#define LOG_MODULE "demux_idcin"
+#define LOG_VERBOSE
+
+/* define LOG to output information about the A/V chunks that the
+ * demuxer is dispatching to the engine */
+/* #define LOG */
+
 #include "xine_internal.h"
 #include "xineutils.h"
 #include "compat.h"
 #include "demux.h"
 #include "bswap.h"
+#include "group_games.h"
 
 #define IDCIN_HEADER_SIZE 20
 #define HUFFMAN_TABLE_SIZE 65536
@@ -88,61 +98,32 @@
 #define PALETTE_SIZE 256
 
 typedef struct {
-
   demux_plugin_t       demux_plugin;
 
   xine_stream_t       *stream;
-
-  config_values_t     *config;
-
   fifo_buffer_t       *video_fifo;
   fifo_buffer_t       *audio_fifo;
-
   input_plugin_t      *input;
-
-  off_t                start;
-  off_t                filesize;
   int                  status;
 
-  unsigned int         video_width;
-  unsigned int         video_height;
-  unsigned int         audio_sample_rate;
-  unsigned int         audio_bytes_per_sample;
-  unsigned int         audio_channels;
+  off_t                filesize;
+  unsigned char        bih[sizeof(xine_bmiheader) + HUFFMAN_TABLE_SIZE];
+  xine_waveformatex    wave;
 
   int                  audio_chunk_size1;
   int                  audio_chunk_size2;
   int                  current_audio_chunk;
 
-  unsigned char        huffman_table[HUFFMAN_TABLE_SIZE];
   uint64_t             pts_counter;
-
-  char                 last_mrl[1024];
 } demux_idcin_t;
 
 typedef struct {
-
   demux_class_t     demux_class;
-
-  /* class-wide, global variables here */
-
-  xine_t           *xine;
-  config_values_t  *config;
 } demux_idcin_class_t;
 
-/* set DEBUG_IDCIN to output information about the A/V chunks that the
- * demuxer is dispatching to the engine */
-#define DEBUG_IDCIN 0
-
-#if DEBUG_IDCIN
-#define debug_idcin printf
-#else
-static inline void debug_idcin(const char *format, ...) { }
-#endif
-
 static int demux_idcin_send_chunk(demux_plugin_t *this_gen) {
-
   demux_idcin_t *this = (demux_idcin_t *) this_gen;
+
   buf_element_t *buf = NULL;
   unsigned int command;
   unsigned char preamble[8];
@@ -159,14 +140,14 @@ static int demux_idcin_send_chunk(demux_plugin_t *this_gen) {
   }
 
   command = le2me_32(command);
-  debug_idcin("  demux_idcin: command %d: ", command);
+  lprintf("command %d: ", command);
   if (command == 2) {
-    debug_idcin("demux finished\n");
+    lprintf("demux finished\n");
     this->status = DEMUX_FINISHED;
     return this->status;
   } else {
     if (command == 1) {
-      debug_idcin("load palette\n");
+      lprintf("load palette\n");
 
       /* load a 768-byte palette and pass it to the demuxer */
       if (this->input->read(this->input, disk_palette, PALETTE_SIZE * 3) !=
@@ -194,7 +175,7 @@ static int demux_idcin_send_chunk(demux_plugin_t *this_gen) {
       }
 
       buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-      buf->decoder_flags = BUF_FLAG_SPECIAL;
+      buf->decoder_flags = BUF_FLAG_SPECIAL|BUF_FLAG_HEADER;
       buf->decoder_info[1] = BUF_SPECIAL_PALETTE;
       buf->decoder_info[2] = PALETTE_SIZE;
       buf->decoder_info_ptr[2] = &palette;
@@ -202,7 +183,7 @@ static int demux_idcin_send_chunk(demux_plugin_t *this_gen) {
       buf->type = BUF_VIDEO_IDCIN;
       this->video_fifo->put (this->video_fifo, buf);
     } else
-      debug_idcin("load video and audio\n");
+      lprintf("load video and audio\n");
   }
 
   /* load the video frame */
@@ -212,13 +193,14 @@ static int demux_idcin_send_chunk(demux_plugin_t *this_gen) {
   }
   remaining_sample_bytes = LE_32(&preamble[0]) - 4;
 
-  debug_idcin("  demux_idcin: dispatching %d video bytes\n",
-    remaining_sample_bytes);
+  lprintf("dispatching %d video bytes\n", remaining_sample_bytes);
   while (remaining_sample_bytes) {
     buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
     buf->type = BUF_VIDEO_IDCIN;
-    buf->extra_info->input_pos = this->input->get_current_pos(this->input);
-    buf->extra_info->input_length = this->filesize;
+    if( this->filesize )
+      buf->extra_info->input_normpos = (int)( (double) (this->input->get_current_pos (this->input) -
+                                                        IDCIN_HEADER_SIZE - HUFFMAN_TABLE_SIZE) *
+                                     65535 / this->filesize );
     buf->extra_info->input_time = this->pts_counter / 90;
     buf->pts = this->pts_counter;
 
@@ -240,13 +222,12 @@ static int demux_idcin_send_chunk(demux_plugin_t *this_gen) {
     if (!remaining_sample_bytes)
       buf->decoder_flags |= BUF_FLAG_FRAME_END;
 
-    debug_idcin("    sending video buf with %d bytes, %lld pts\n",
-        buf->size, buf->pts);
+    lprintf("sending video buf with %d bytes, %lld pts\n", buf->size, buf->pts);
     this->video_fifo->put(this->video_fifo, buf);
   }
 
   /* load the audio frame */
-  if (this->audio_fifo && this->audio_sample_rate) {
+  if (this->audio_fifo && this->wave.nSamplesPerSec) {
 
     if (this->current_audio_chunk == 1) {
       remaining_sample_bytes = this->audio_chunk_size1;
@@ -256,13 +237,13 @@ static int demux_idcin_send_chunk(demux_plugin_t *this_gen) {
       this->current_audio_chunk = 1;
     }
 
-    debug_idcin("  demux_idcin: dispatching %d audio bytes\n",
-      remaining_sample_bytes);
+    lprintf("dispatching %d audio bytes\n", remaining_sample_bytes);
     while (remaining_sample_bytes) {
       buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
       buf->type = BUF_AUDIO_LPCM_LE;
-      buf->extra_info->input_pos = this->input->get_current_pos(this->input);
-      buf->extra_info->input_length = this->filesize;
+      if( this->filesize )
+        buf->extra_info->input_normpos = (int)( (double) this->input->get_current_pos (this->input) *
+                                         65535 / this->filesize );
       buf->extra_info->input_time = this->pts_counter / 90;
       buf->pts = this->pts_counter;
 
@@ -282,14 +263,10 @@ static int demux_idcin_send_chunk(demux_plugin_t *this_gen) {
       if (!remaining_sample_bytes)
         buf->decoder_flags |= BUF_FLAG_FRAME_END;
 
-      debug_idcin("    sending audio buf with %d bytes, %lld pts\n",
-          buf->size, buf->pts);
+      lprintf("sending audio buf with %d bytes, %lld pts\n", buf->size, buf->pts);
       this->audio_fifo->put(this->audio_fifo, buf);
     }
   }
-
-  debug_idcin("\n");
-
   this->pts_counter += IDCIN_FRAME_PTS_INC;
 
   return this->status;
@@ -297,12 +274,11 @@ static int demux_idcin_send_chunk(demux_plugin_t *this_gen) {
 
 /* returns 1 if the CIN file was opened successfully, 0 otherwise */
 static int open_idcin_file(demux_idcin_t *this) {
-
   unsigned char header[IDCIN_HEADER_SIZE];
+  xine_bmiheader *bih = (xine_bmiheader *)this->bih;
+  unsigned char *huffman_table = this->bih + sizeof(xine_bmiheader);
 
-  this->input->seek(this->input, 0, SEEK_SET);
-  if (this->input->read(this->input, header, IDCIN_HEADER_SIZE) != 
-    IDCIN_HEADER_SIZE)
+  if (_x_demux_read_header(this->input, header, IDCIN_HEADER_SIZE) != IDCIN_HEADER_SIZE)
     return 0;
 
   /*
@@ -317,66 +293,72 @@ static int open_idcin_file(demux_idcin_t *this) {
    */
 
   /* check the width */
-  this->video_width = LE_32(&header[0]);
-  if ((this->video_width == 0) || (this->video_width > 1024))
+  bih->biWidth = LE_32(&header[0]);
+  if ((bih->biWidth == 0) || (bih->biWidth > 1024))
     return 0;
 
   /* check the height */
-  this->video_height = LE_32(&header[4]);
-  if ((this->video_height == 0) || (this->video_height > 1024))
+  bih->biHeight = LE_32(&header[4]);
+  if ((bih->biHeight == 0) || (bih->biHeight > 1024))
     return 0;
 
   /* check the audio sample rate */
-  this->audio_sample_rate = LE_32(&header[8]);
-  if ((this->audio_sample_rate != 0) && 
-     ((this->audio_sample_rate < 8000) || (this->audio_sample_rate > 48000)))
+  this->wave.nSamplesPerSec = LE_32(&header[8]);
+  if ((this->wave.nSamplesPerSec != 0) &&
+     ((this->wave.nSamplesPerSec < 8000) || (this->wave.nSamplesPerSec > 48000)))
     return 0;
 
   /* check the audio bytes/sample */
-  this->audio_bytes_per_sample = LE_32(&header[12]);
-  if (this->audio_bytes_per_sample > 2)
+  this->wave.wBitsPerSample = LE_32(&header[12]) * 8;
+  if (this->wave.wBitsPerSample > 16)
     return 0;
 
   /* check the audio channels */
-  this->audio_channels = LE_32(&header[16]);
-  if (this->audio_channels > 2)
+  this->wave.nChannels = LE_32(&header[16]);
+  if (this->wave.nChannels > 2)
     return 0;
 
-  /* if execution got this far, qualify it as a valid Id CIN file 
+  /* if execution got this far, qualify it as a valid Id CIN file
    * and continue loading */
-  debug_idcin("  demux_idcin: %dx%d video, %d Hz, %d channels, %d bits PCM audio\n",
-    this->video_width, this->video_height,
-    this->audio_sample_rate, 
-    this->audio_channels,
-    this->audio_bytes_per_sample * 8);
+  lprintf("%dx%d video, %d Hz, %d channels, %d bit PCM audio\n",
+    bih->biWidth, bih->biHeight,
+    this->wave.nSamplesPerSec,
+    this->wave.nChannels,
+    this->wave.wBitsPerSample);
+
+  /* file is qualified; skip over the signature bytes in the stream */
+  this->input->seek(this->input, IDCIN_HEADER_SIZE, SEEK_SET);
 
   /* read the Huffman table */
-  if (this->input->read(this->input, this->huffman_table,
-    HUFFMAN_TABLE_SIZE) != HUFFMAN_TABLE_SIZE)
+  if (this->input->read(this->input, huffman_table, HUFFMAN_TABLE_SIZE) != 
+    HUFFMAN_TABLE_SIZE)
     return 0;
 
   /* load stream information */
-  this->stream->stream_info[XINE_STREAM_INFO_HAS_VIDEO] = 1;
-  this->stream->stream_info[XINE_STREAM_INFO_HAS_AUDIO] = 
-    (this->audio_channels) ? 1 : 0;
-  this->stream->stream_info[XINE_STREAM_INFO_VIDEO_WIDTH] =
-    this->video_width;
-  this->stream->stream_info[XINE_STREAM_INFO_VIDEO_HEIGHT] = 
-    this->video_height;
-  this->stream->stream_info[XINE_STREAM_INFO_AUDIO_CHANNELS] =
-    this->audio_channels;
-  this->stream->stream_info[XINE_STREAM_INFO_AUDIO_SAMPLERATE] =
-    this->audio_sample_rate;
-  this->stream->stream_info[XINE_STREAM_INFO_AUDIO_BITS] =
-    this->audio_bytes_per_sample * 8;
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_HAS_VIDEO, 1);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_HAS_AUDIO,
+    (this->wave.nChannels) ? 1 : 0);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_VIDEO_WIDTH,
+    bih->biWidth);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_VIDEO_HEIGHT,
+    bih->biHeight);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_CHANNELS,
+    this->wave.nChannels);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_SAMPLERATE,
+    this->wave.nSamplesPerSec);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_BITS,
+    this->wave.wBitsPerSample);
+
+  this->filesize = this->input->get_length(this->input) - 
+    IDCIN_HEADER_SIZE - HUFFMAN_TABLE_SIZE;
 
   return 1;
 }
 
 static void demux_idcin_send_headers(demux_plugin_t *this_gen) {
-
   demux_idcin_t *this = (demux_idcin_t *) this_gen;
   buf_element_t *buf;
+  xine_bmiheader *bih = (xine_bmiheader *)this->bih;
 
   this->video_fifo  = this->stream->video_fifo;
   this->audio_fifo  = this->stream->audio_fifo;
@@ -384,75 +366,62 @@ static void demux_idcin_send_headers(demux_plugin_t *this_gen) {
   this->status = DEMUX_OK;
 
   /* send start buffers */
-  xine_demux_control_start(this->stream);
+  _x_demux_control_start(this->stream);
 
   /* send init info to decoders */
+  bih->biSize = sizeof(xine_bmiheader) + HUFFMAN_TABLE_SIZE;
   buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-  buf->decoder_flags = BUF_FLAG_HEADER;
-  buf->decoder_info[0] = 0;
-  buf->decoder_info[1] = IDCIN_FRAME_PTS_INC;  /* initial video_step */
-  /* really be a rebel: No structure at all, just put the video width
-   * and height straight into the buffer, BE_16 format */
-  buf->content[0] = (this->video_width >> 8) & 0xFF;
-  buf->content[1] = (this->video_width >> 0) & 0xFF;
-  buf->content[2] = (this->video_height >> 8) & 0xFF;
-  buf->content[3] = (this->video_height >> 0) & 0xFF;
-  buf->size = 4;
+  buf->decoder_flags = BUF_FLAG_HEADER|BUF_FLAG_STDHEADER|BUF_FLAG_FRAMERATE|
+                       BUF_FLAG_FRAME_END;
+  buf->decoder_info[0] = IDCIN_FRAME_PTS_INC;  /* initial video_step */
+  buf->size = bih->biSize;
+  memcpy(buf->content, this->bih, buf->size);
   buf->type = BUF_VIDEO_IDCIN;
   this->video_fifo->put (this->video_fifo, buf);
 
-  /* send the Huffman table */
-  buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-  buf->decoder_flags = BUF_FLAG_SPECIAL;
-  buf->decoder_info[1] = BUF_SPECIAL_IDCIN_HUFFMAN_TABLE;
-  buf->decoder_info[2] = sizeof(this->huffman_table);
-  buf->decoder_info_ptr[2] = &this->huffman_table;
-  buf->size = 0;
-  buf->type = BUF_VIDEO_IDCIN;
-  this->video_fifo->put (this->video_fifo, buf);
-
-  if (this->audio_fifo && this->audio_channels) {
+  if (this->audio_fifo && this->wave.nChannels) {
 
     /* initialize the chunk sizes */
-    if (this->audio_sample_rate % 14 != 0) {
-      this->audio_chunk_size1 = (this->audio_sample_rate / 14) *
-        this->audio_bytes_per_sample * this->audio_channels;
-      this->audio_chunk_size2 = (this->audio_sample_rate / 14 + 1) *
-        this->audio_bytes_per_sample * this->audio_channels;
+    if (this->wave.nSamplesPerSec % 14 != 0) {
+      this->audio_chunk_size1 = (this->wave.nSamplesPerSec / 14) *
+        this->wave.wBitsPerSample / 8 * this->wave.nChannels;
+      this->audio_chunk_size2 = (this->wave.nSamplesPerSec / 14 + 1) *
+        this->wave.wBitsPerSample / 8 * this->wave.nChannels;
     } else {
       this->audio_chunk_size1 = this->audio_chunk_size2 =
-        (this->audio_sample_rate / 14) * this->audio_bytes_per_sample *
-        this->audio_channels;
+        (this->wave.nSamplesPerSec / 14) * this->wave.wBitsPerSample / 8 *
+        this->wave.nChannels;
     }
-    debug_idcin("  demux_idcin: audio_chunk_size[1,2] = %d, %d\n",
-      this->audio_chunk_size1, this->audio_chunk_size2);
+    lprintf("audio_chunk_size[1,2] = %d, %d\n",
+            this->audio_chunk_size1, this->audio_chunk_size2);
 
     buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
     buf->type = BUF_AUDIO_LPCM_LE;
-    buf->decoder_flags = BUF_FLAG_HEADER;
+    buf->decoder_flags = BUF_FLAG_HEADER|BUF_FLAG_STDHEADER|BUF_FLAG_FRAME_END;
     buf->decoder_info[0] = 0;
-    buf->decoder_info[1] = this->audio_sample_rate;
-    buf->decoder_info[2] = this->audio_bytes_per_sample * 8;
-    buf->decoder_info[3] = this->audio_channels;
+    buf->decoder_info[1] = this->wave.nSamplesPerSec;
+    buf->decoder_info[2] = this->wave.wBitsPerSample;
+    buf->decoder_info[3] = this->wave.nChannels;
+    buf->size = sizeof(this->wave);
+    memcpy(buf->content, &this->wave, buf->size);
     this->audio_fifo->put (this->audio_fifo, buf);
   }
 }
 
-static int demux_idcin_seek (demux_plugin_t *this_gen,
-                              off_t start_pos, int start_time) {
-
+static int demux_idcin_seek (demux_plugin_t *this_gen, off_t start_pos, int start_time, int playing) {
   demux_idcin_t *this = (demux_idcin_t *) this_gen;
 
   /* if thread is not running, initialize demuxer */
-  if( !this->stream->demux_thread_running ) {
+  if( !playing ) {
 
     /* send new pts */
-    xine_demux_control_newpts(this->stream, 0, 0);
+    _x_demux_control_newpts(this->stream, 0, 0);
 
     this->status = DEMUX_OK;
-  
+
     /* reposition stream past the Huffman tables */
-    this->input->seek(this->input, 0x14 + 0x10000, SEEK_SET);
+    this->input->seek(this->input, IDCIN_HEADER_SIZE + HUFFMAN_TABLE_SIZE, 
+      SEEK_SET);
 
     this->pts_counter = 0;
     this->current_audio_chunk = 1;
@@ -473,7 +442,6 @@ static int demux_idcin_get_status (demux_plugin_t *this_gen) {
 }
 
 static int demux_idcin_get_stream_length (demux_plugin_t *this_gen) {
-
   return 0;
 }
 
@@ -487,16 +455,10 @@ static int demux_idcin_get_optional_data(demux_plugin_t *this_gen,
 }
 
 static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *stream,
-                                    input_plugin_t *input_gen) {
+                                    input_plugin_t *input) {
 
-  input_plugin_t *input = (input_plugin_t *) input_gen;
   demux_idcin_t  *this;
 
-  if (! (input->get_capabilities(input) & INPUT_CAP_SEEKABLE)) {
-    printf(_("demux_idcin.c: input not seekable, can not handle!\n"));
-    return NULL;
-  }
-
   this         = xine_xmalloc (sizeof (demux_idcin_t));
   this->stream = stream;
   this->input  = input;
@@ -507,8 +469,6 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   this->demux_plugin.dispose           = demux_idcin_dispose;
   this->demux_plugin.get_status        = demux_idcin_get_status;
   this->demux_plugin.get_stream_length = demux_idcin_get_stream_length;
-  this->demux_plugin.get_video_frame   = NULL;
-  this->demux_plugin.got_video_frame_cb= NULL;
   this->demux_plugin.get_capabilities  = demux_idcin_get_capabilities;
   this->demux_plugin.get_optional_data = demux_idcin_get_optional_data;
   this->demux_plugin.demux_class       = class_gen;
@@ -517,6 +477,19 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   switch (stream->content_detection_method) {
 
+  case METHOD_BY_EXTENSION: {
+    char *extensions, *mrl;
+
+    mrl = input->get_mrl (input);
+    extensions = class_gen->get_extensions (class_gen);
+
+    if (!_x_demux_check_extension (mrl, extensions)) {
+      free (this);
+      return NULL;
+    }
+  }
+  /* falling through is intended */
+
   case METHOD_BY_CONTENT:
   case METHOD_EXPLICIT:
 
@@ -527,39 +500,11 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   break;
 
-  case METHOD_BY_EXTENSION: {
-    char *ending, *mrl;
-
-    mrl = input->get_mrl (input);
-
-    ending = strrchr(mrl, '.');
-
-    if (!ending) {
-      free (this);
-      return NULL;
-    }
-
-    if (strncasecmp (ending, ".cin", 4)) {
-      free (this);
-      return NULL;
-    }
-
-    if (!open_idcin_file(this)) {
-      free (this);
-      return NULL;
-    }
-
-  }
-
-  break;
-
   default:
     free (this);
     return NULL;
   }
 
-  strncpy (this->last_mrl, input->get_mrl (input), 1024);
-
   return &this->demux_plugin;
 }
 
@@ -581,19 +526,15 @@ static char *get_mimetypes (demux_class_t *this_gen) {
 }
 
 static void class_dispose (demux_class_t *this_gen) {
-
   demux_idcin_class_t *this = (demux_idcin_class_t *) this_gen;
 
   free (this);
 }
 
 void *demux_idcin_init_plugin (xine_t *xine, void *data) {
-
   demux_idcin_class_t     *this;
 
   this         = xine_xmalloc (sizeof (demux_idcin_class_t));
-  this->config = xine->config;
-  this->xine   = xine;
 
   this->demux_class.open_plugin     = open_plugin;
   this->demux_class.get_description = get_description;
@@ -604,15 +545,3 @@ void *demux_idcin_init_plugin (xine_t *xine, void *data) {
 
   return this;
 }
-
-/*
- * exported plugin catalog entry
- */
-
-#if 0
-plugin_info_t xine_plugin_info[] = {
-  /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_DEMUX, 20, "idcin", XINE_VERSION_CODE, NULL, demux_idcin_init_plugin },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
-};
-#endif
