@@ -19,7 +19,7 @@
  */
 
 /*
- * $Id: demux_ogg.c,v 1.160 2004/12/21 05:30:40 conrad Exp $
+ * $Id: demux_ogg.c,v 1.160.2.2 2005/07/17 21:49:40 miguelfreitas Exp $
  *
  * demultiplexer for ogg streams
  *
@@ -114,7 +114,9 @@ typedef struct stream_info_s {
   int64_t               quotient;
   int                   resync;
   char                  *language;
-  /* Annodex-specific stream information */
+  /* CMML, Ogg Skeleton stream information */
+  int                   granuleshift;
+  /* Annodex v2 stream information */
   int                   hide_first_header;
   int                   delivered_bos;
   int                   delivered_eos;
@@ -217,17 +219,17 @@ static int64_t get_pts (demux_ogg_t *this, int stream_num , int64_t granulepos )
       return 1;
     } else
       return 0;
-
-#ifdef HAVE_THEORA
-  } else  if (this->si[stream_num]->buf_types == BUF_VIDEO_THEORA) {
-    int64_t iframe,pframe;
-    int keyframe_granule_shift;
-    keyframe_granule_shift=intlog(this->t_info.keyframe_frequency_force-1);
-    iframe=granulepos>>keyframe_granule_shift;
-    pframe=granulepos-(iframe<<keyframe_granule_shift);
-    return 1+((iframe + pframe)*this->frame_duration);
-#endif
-
+  } else if (this->si[stream_num]->buf_types == BUF_VIDEO_THEORA ||
+	     (this->si[stream_num]->buf_types & 0xFFFF0000) == BUF_SPU_CMML) {
+    int64_t iframe, pframe;
+    int granuleshift;
+    granuleshift = this->si[stream_num]->granuleshift;
+    iframe = granulepos >> granuleshift;
+    pframe = granulepos - (iframe << granuleshift);
+    if (this->si[stream_num]->quotient)
+      return 1+((iframe+pframe) * this->si[stream_num]->factor / this->si[stream_num]->quotient);
+    else
+      return 0;
   } else if (this->si[stream_num]->quotient)
     return 1+(granulepos * this->si[stream_num]->factor / this->si[stream_num]->quotient);
   else
@@ -698,7 +700,6 @@ static void send_ogg_buf (demux_ogg_t *this,
       }
     }
   } else if ((this->si[stream_num]->buf_types & 0xFFFF0000) == BUF_SPU_CMML) {
-
     buf_element_t *buf;
     uint32_t *val;
     char *str;
@@ -707,8 +708,7 @@ static void send_ogg_buf (demux_ogg_t *this,
 
     buf->type = this->si[stream_num]->buf_types;
 
-    /* CMML granulepos is (1000 * seconds); pts is (90000 * seconds) */
-    buf->pts = op->granulepos * (90000 / 1000);
+    buf->pts = get_pts (this, stream_num, op->granulepos);
 
     val = (uint32_t * )buf->content;
     str = (char *)val;
@@ -722,7 +722,6 @@ static void send_ogg_buf (demux_ogg_t *this,
              stream_num, op->bytes, buf->pts, str);
 
     this->video_fifo->put (this->video_fifo, buf);
-
   } else if ((this->si[stream_num]->buf_types & 0xFF000000) == BUF_SPU_BASE) {
 
     buf_element_t *buf;
@@ -1143,9 +1142,16 @@ static void decode_theora_header (demux_ogg_t *this, const int stream_num, ogg_p
     this->num_video_streams++;
 
     this->si[stream_num]->factor = (int64_t) 90000 * (int64_t) this->t_info.fps_denominator;
+
+    if (!this->t_info.fps_numerator) {
+      this->t_info.fps_numerator = 1;   /* FIXME: default value ? */
+    }
     this->si[stream_num]->quotient = this->t_info.fps_numerator;
 
-    this->frame_duration = ((int64_t) 90000*this->t_info.fps_denominator)/this->t_info.fps_numerator;
+    this->frame_duration = ((int64_t) 90000*this->t_info.fps_denominator);
+    this->frame_duration /= this->t_info.fps_numerator;
+
+    this->si[stream_num]->granuleshift = intlog(this->t_info.keyframe_frequency_force-1);
 
     this->si[stream_num]->headers=3;
     this->si[stream_num]->buf_types = BUF_VIDEO_THEORA;
@@ -1153,13 +1159,15 @@ static void decode_theora_header (demux_ogg_t *this, const int stream_num, ogg_p
     _x_meta_info_set(this->stream, XINE_META_INFO_VIDEOCODEC, "theora");
     _x_stream_info_set(this->stream, XINE_STREAM_INFO_VIDEO_WIDTH, this->t_info.frame_width);
     _x_stream_info_set(this->stream, XINE_STREAM_INFO_VIDEO_HEIGHT, this->t_info.frame_height);
-    _x_stream_info_set(this->stream, XINE_STREAM_INFO_FRAME_DURATION,
-                       ((int64_t) 90000 * this->t_info.fps_denominator) / this->t_info.fps_numerator);
+    _x_stream_info_set(this->stream, XINE_STREAM_INFO_FRAME_DURATION, this->frame_duration);
 
     /*currently aspect_nominator and -denumerator are 0?*/
-    if (this->t_info.aspect_denominator)
-      _x_stream_info_set(this->stream, XINE_STREAM_INFO_VIDEO_RATIO,
-                         ((int64_t) this->t_info.aspect_numerator * 10000) / this->t_info.aspect_denominator);
+    if (this->t_info.aspect_denominator) {
+      int64_t ratio = ((int64_t) this->t_info.aspect_numerator * 10000);
+
+      ratio /= this->t_info.aspect_denominator;
+      _x_stream_info_set(this->stream, XINE_STREAM_INFO_VIDEO_RATIO, ratio);
+    }
 
     lprintf ("decoded theora header \n");
     lprintf ("frameduration %d\n",this->frame_duration);
@@ -1241,6 +1249,7 @@ static void decode_anxdata_header (demux_ogg_t *this, const int stream_num, ogg_
     unsigned int channel = this->num_spu_streams++;
     this->si[stream_num]->headers = 0;
     this->si[stream_num]->buf_types = BUF_SPU_CMML | channel;
+    this->si[stream_num]->granuleshift = 0;
   } else {
     this->si[stream_num]->buf_types = BUF_CONTROL_NOP;
   }
@@ -1251,6 +1260,10 @@ static void decode_cmml_header (demux_ogg_t *this, const int stream_num, ogg_pac
     unsigned int channel = this->num_spu_streams++;
     this->si[stream_num]->headers = 0;
     this->si[stream_num]->buf_types = BUF_SPU_CMML | channel;
+
+    this->si[stream_num]->factor = 90000 * LE_64(&op->packet[20]);
+    this->si[stream_num]->quotient = LE_64(&op->packet[12]);
+    this->si[stream_num]->granuleshift = (int)op->packet[28];
 }
 
 /*
@@ -1468,7 +1481,7 @@ static int demux_ogg_send_chunk (demux_plugin_t *this_gen) {
         int keyframe_granule_shift;
         int64_t pframe=-1,iframe=-1;
 
-        keyframe_granule_shift=intlog(this->t_info.keyframe_frequency_force-1);
+	keyframe_granule_shift = this->si[stream_num]->granuleshift;
 
         if(op.granulepos>=0){
           iframe=op.granulepos>>keyframe_granule_shift;
@@ -2113,7 +2126,7 @@ demuxer_info_t demux_info_ogg = {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_DEMUX, 25, "ogg", XINE_VERSION_CODE, &demux_info_ogg, ogg_init_class },
-  { PLUGIN_DEMUX, 25, "anx", XINE_VERSION_CODE, &demux_info_anx, anx_init_class },
+  { PLUGIN_DEMUX, 26, "ogg", XINE_VERSION_CODE, &demux_info_ogg, ogg_init_class },
+  { PLUGIN_DEMUX, 26, "anx", XINE_VERSION_CODE, &demux_info_anx, anx_init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
