@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2005 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -31,7 +31,7 @@
  *   
  *   Based on FFmpeg's libav/rm.c.
  *
- * $Id: demux_real.c,v 1.103 2004/08/19 19:59:14 jstembridge Exp $
+ * $Id: demux_real.c,v 1.103.2.2 2005/07/17 21:49:58 miguelfreitas Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -1626,7 +1626,7 @@ static char *get_identifier (demux_class_t *this_gen) {
 }
 
 static char *get_extensions (demux_class_t *this_gen) {
-  return "rm ram";
+  return "rm rmvb ram";
 }
 
 static char *get_mimetypes (demux_class_t *this_gen) {
@@ -1666,6 +1666,6 @@ demuxer_info_t demux_info_real = {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_DEMUX, 25, "real", XINE_VERSION_CODE, &demux_info_real, init_class },
+  { PLUGIN_DEMUX, 26, "real", XINE_VERSION_CODE, &demux_info_real, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
