@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -16,13 +16,15 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
+ */
+
+/*
  * File Demuxer for Wing Commander III MVE movie files
  *   by Mike Melanson (melanson@pcisys.net)
  * For more information on the MVE file format, visit:
  *   http://www.pcisys.net/~melanson/codecs/
  *
- * $Id: demux_wc3movie.c,v 1.33 2003/01/17 16:52:40 miguelfreitas Exp $
+ * $Id: demux_wc3movie.c,v 1.52 2004/06/13 21:28:55 miguelfreitas Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -35,18 +37,21 @@
 #include <string.h>
 #include <stdlib.h>
 
+/********** logging **********/
+#define LOG_MODULE "demux_wc3movie"
+#define LOG_VERBOSE
+/*
+#define LOG
+*/
+
 #include "xine_internal.h"
 #include "xineutils.h"
 #include "compat.h"
 #include "demux.h"
 #include "bswap.h"
+#include "group_games.h"
 
-#define FOURCC_TAG( ch0, ch1, ch2, ch3 ) \
-        ( (long)(unsigned char)(ch3) | \
-        ( (long)(unsigned char)(ch2) << 8 ) | \
-        ( (long)(unsigned char)(ch1) << 16 ) | \
-        ( (long)(unsigned char)(ch0) << 24 ) )
-
+#define FOURCC_TAG BE_FOURCC
 #define FORM_TAG FOURCC_TAG('F', 'O', 'R', 'M')
 #define MOVE_TAG FOURCC_TAG('M', 'O', 'V', 'E')
 #define PC_TAG   FOURCC_TAG('_', 'P', 'C', '_')
@@ -67,29 +72,19 @@
 #define WC3_PTS_INC (90000 / 15)
 #define WC3_USUAL_WIDTH 320
 #define WC3_USUAL_HEIGHT 165
-
+#define WC3_HEADER_SIZE 16
 #define PREAMBLE_SIZE 8
 
 typedef struct {
-
   demux_plugin_t       demux_plugin;
 
   xine_stream_t       *stream;
-
-  config_values_t     *config;
-
   fifo_buffer_t       *video_fifo;
   fifo_buffer_t       *audio_fifo;
-
   input_plugin_t      *input;
-
   int                  status;
 
-  unsigned int         fps;
-  unsigned int         frame_pts_inc;
-  unsigned int         video_width;
-  unsigned int         video_height;
-
+  xine_bmiheader       bih;
   xine_waveformatex    wave;
 
   palette_entry_t     *palettes;
@@ -102,18 +97,10 @@ typedef struct {
   off_t                data_size;
 
   int64_t              video_pts;
-
-  char                 last_mrl[1024];
 } demux_mve_t;
 
 typedef struct {
-
   demux_class_t     demux_class;
-
-  /* class-wide, global variables here */
-
-  xine_t           *xine;
-  config_values_t  *config;
 } demux_mve_class_t;
 
 /* bizarre palette lookup table */
@@ -153,8 +140,8 @@ const unsigned char wc3_pal_lookup[] = {
 };
 
 static int demux_mve_send_chunk(demux_plugin_t *this_gen) {
-
   demux_mve_t *this = (demux_mve_t *) this_gen;
+
   buf_element_t *buf = NULL;
   int64_t text_pts = 0;
   int64_t audio_pts = 0;
@@ -177,25 +164,18 @@ static int demux_mve_send_chunk(demux_plugin_t *this_gen) {
     chunk_size = (BE_32(&preamble[4]) + 1) & (~1);
 
     if (chunk_tag == BRCH_TAG) {
-
       /* empty chunk; do nothing */
-
     } else if (chunk_tag == SHOT_TAG) {
-
       if (this->seek_flag) {
-
         /* reset pts */
         this->video_pts = 0;
-        xine_demux_control_newpts(this->stream, 0, BUF_FLAG_SEEK);
+        _x_demux_control_newpts(this->stream, 0, BUF_FLAG_SEEK);
         this->seek_flag = 0;
-
       } else {
-
         /* record the offset of the SHOT chunk */
-        this->shot_offsets[this->current_shot] = 
+        this->shot_offsets[this->current_shot] =
           this->input->get_current_pos(this->input) - PREAMBLE_SIZE;
       }
-
       this->current_shot++;
 
       /* this is the start of a new shot; send a new palette */
@@ -207,14 +187,14 @@ static int demux_mve_send_chunk(demux_plugin_t *this_gen) {
 
       if (palette_number >= this->number_of_shots) {
         xine_log(this->stream->xine, XINE_LOG_MSG,
-          _("demux_wc3movie: SHOT chunk referenced invalid palette (%d >= %d)\n"),
+		 _("demux_wc3movie: SHOT chunk referenced invalid palette (%d >= %d)\n"),
           palette_number, this->number_of_shots);
         this->status = DEMUX_FINISHED;
         return this->status;
       }
 
       buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-      buf->decoder_flags = BUF_FLAG_SPECIAL;
+      buf->decoder_flags = BUF_FLAG_SPECIAL|BUF_FLAG_HEADER;
       buf->decoder_info[1] = BUF_SPECIAL_PALETTE;
       buf->decoder_info[2] = PALETTE_SIZE;
       buf->decoder_info_ptr[2] = &this->palettes[PALETTE_SIZE * palette_number];
@@ -223,16 +203,14 @@ static int demux_mve_send_chunk(demux_plugin_t *this_gen) {
       this->video_fifo->put (this->video_fifo, buf);
 
     } else if (chunk_tag == AUDI_TAG) {
-
       if( this->audio_fifo ) {
-
         audio_pts = this->video_pts - WC3_PTS_INC;
   
         while (chunk_size) {
           buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
           buf->type = BUF_AUDIO_LPCM_LE;
-          buf->extra_info->input_pos = current_file_pos;
-          buf->extra_info->input_length = this->data_size;
+          if( this->data_size )
+            buf->extra_info->input_normpos = (int)( (double) current_file_pos * 65535 / this->data_size);
           buf->extra_info->input_time = audio_pts / 90;
           buf->pts = audio_pts;
   
@@ -258,12 +236,11 @@ static int demux_mve_send_chunk(demux_plugin_t *this_gen) {
         this->input->seek(this->input, chunk_size, SEEK_CUR);
       }
     } else if (chunk_tag == VGA_TAG) {
-
       while (chunk_size) {
         buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
         buf->type = BUF_VIDEO_WC3;
-        buf->extra_info->input_pos = current_file_pos;
-        buf->extra_info->input_length = this->data_size;
+        if( this->data_size )
+          buf->extra_info->input_normpos = (int)( (double) current_file_pos * 65535 / this->data_size);
         buf->extra_info->input_time = this->video_pts / 90;
         buf->pts = this->video_pts;
 
@@ -286,18 +263,14 @@ static int demux_mve_send_chunk(demux_plugin_t *this_gen) {
         this->video_fifo->put (this->video_fifo, buf);
       }
       this->video_pts += WC3_PTS_INC;
-
     } else if (chunk_tag == TEXT_TAG) {
-
       text_pts = this->video_pts - WC3_PTS_INC;
 
       /* unhandled thus far */
       this->input->seek(this->input, chunk_size, SEEK_CUR);
-
     } else {
-
       /* report an unknown chunk and skip it */
-      printf (_("demux_wc3movie: encountered unknown chunk: %c%c%c%c\n"),
+      lprintf("encountered unknown chunk: %c%c%c%c\n",
         (chunk_tag >> 24) & 0xFF,
         (chunk_tag >> 16) & 0xFF,
         (chunk_tag >>  8) & 0xFF,
@@ -310,7 +283,6 @@ static int demux_mve_send_chunk(demux_plugin_t *this_gen) {
 }
 
 static void demux_mve_send_headers(demux_plugin_t *this_gen) {
-
   demux_mve_t *this = (demux_mve_t *) this_gen;
   buf_element_t *buf;
 
@@ -320,34 +292,31 @@ static void demux_mve_send_headers(demux_plugin_t *this_gen) {
   this->status = DEMUX_OK;
 
   /* load stream information */
-  this->stream->stream_info[XINE_STREAM_INFO_HAS_VIDEO] = 1;
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_HAS_VIDEO, 1);
   /* this is not strictly correct-- some WC3 MVE files do not contain 
    * audio, but I'm too lazy to check if that is the case */
-  this->stream->stream_info[XINE_STREAM_INFO_HAS_AUDIO] = 1;
-  this->stream->stream_info[XINE_STREAM_INFO_VIDEO_WIDTH] = this->video_width;
-  this->stream->stream_info[XINE_STREAM_INFO_VIDEO_HEIGHT] = this->video_height;
-  this->stream->stream_info[XINE_STREAM_INFO_AUDIO_CHANNELS] =
-    this->wave.nChannels;
-  this->stream->stream_info[XINE_STREAM_INFO_AUDIO_SAMPLERATE] =
-    this->wave.nSamplesPerSec;
-  this->stream->stream_info[XINE_STREAM_INFO_AUDIO_BITS] =
-    this->wave.wBitsPerSample;
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_HAS_AUDIO, 1);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_VIDEO_WIDTH,
+                       this->bih.biWidth);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_VIDEO_HEIGHT,
+                       this->bih.biHeight);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_CHANNELS,
+                       this->wave.nChannels);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_SAMPLERATE,
+                       this->wave.nSamplesPerSec);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_BITS,
+                       this->wave.wBitsPerSample);
 
   /* send start buffers */
-  xine_demux_control_start(this->stream);
+  _x_demux_control_start(this->stream);
 
   /* send init info to decoders */
   buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-  buf->decoder_flags = BUF_FLAG_HEADER;
-  buf->decoder_info[0] = 0;
-  buf->decoder_info[1] = WC3_PTS_INC;  /* initial video_step */
-  /* really be a rebel: No structure at all, just put the video width
-   * and height straight into the buffer, BE_16 format */
-  buf->content[0] = (this->video_width >> 8) & 0xFF;
-  buf->content[1] = (this->video_width >> 0) & 0xFF;
-  buf->content[2] = (this->video_height >> 8) & 0xFF;
-  buf->content[3] = (this->video_height >> 0) & 0xFF;
-  buf->size = 4;
+  buf->decoder_flags = BUF_FLAG_HEADER|BUF_FLAG_STDHEADER|BUF_FLAG_FRAMERATE|
+                       BUF_FLAG_FRAME_END;
+  buf->decoder_info[0] = WC3_PTS_INC;  /* initial video_step */
+  buf->content = (void *)&this->bih;
+  buf->size = sizeof(this->bih);
   buf->type = BUF_VIDEO_WC3;
   this->video_fifo->put (this->video_fifo, buf);
 
@@ -361,7 +330,7 @@ static void demux_mve_send_headers(demux_plugin_t *this_gen) {
 
     buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
     buf->type = BUF_AUDIO_LPCM_LE;
-    buf->decoder_flags = BUF_FLAG_HEADER;
+    buf->decoder_flags = BUF_FLAG_HEADER|BUF_FLAG_STDHEADER|BUF_FLAG_FRAME_END;
     buf->decoder_info[0] = 0;
     buf->decoder_info[1] = this->wave.nSamplesPerSec;
     buf->decoder_info[2] = this->wave.wBitsPerSample;
@@ -382,14 +351,10 @@ static int open_mve_file(demux_mve_t *this) {
   int i, j;
   unsigned char r, g, b;
   int temp;
-  unsigned char header[16];
+  unsigned char header[WC3_HEADER_SIZE];
+  void *title;
 
-  /* these are the frame dimensions unless others are found */
-  this->video_width = WC3_USUAL_WIDTH;
-  this->video_height = WC3_USUAL_HEIGHT;
-
-  this->input->seek(this->input, 0, SEEK_SET);
-  if (this->input->read(this->input, header, 16) != 16)
+  if (_x_demux_read_header(this->input, header, WC3_HEADER_SIZE) != WC3_HEADER_SIZE)
     return 0;
 
   if ((BE_32(&header[0]) != FORM_TAG) ||
@@ -397,6 +362,13 @@ static int open_mve_file(demux_mve_t *this) {
       (BE_32(&header[12]) != PC_TAG))
     return 0;
 
+  /* file is qualified */
+
+  this->bih.biSize = sizeof(xine_bmiheader);
+  /* these are the frame dimensions unless others are found */
+  this->bih.biWidth = WC3_USUAL_WIDTH;
+  this->bih.biHeight = WC3_USUAL_HEIGHT;
+
   /* load the number of palettes, the only interesting piece of information
    * in the _PC_ chunk; take it for granted that it will always appear at
    * position 0x1C */
@@ -420,20 +392,28 @@ static int open_mve_file(demux_mve_t *this) {
   for (i = 0; i < this->number_of_shots; i++) {
     /* make sure there was a valid palette chunk preamble */
     if (this->input->read(this->input, preamble, PREAMBLE_SIZE) !=
-      PREAMBLE_SIZE)
+      PREAMBLE_SIZE) {
+      free (this->palettes);
+      free (this->shot_offsets);
       return 0;
+    }
 
     if ((BE_32(&preamble[0]) != PALT_TAG) || 
         (BE_32(&preamble[4]) != PALETTE_CHUNK_SIZE)) {
       xine_log(this->stream->xine, XINE_LOG_MSG,
-        _("demux_wc3movie: There was a problem while loading palette chunks\n"));
+	       _("demux_wc3movie: There was a problem while loading palette chunks\n"));
+      free (this->palettes);
+      free (this->shot_offsets);
       return 0;
     }
 
     /* load the palette chunk */
     if (this->input->read(this->input, disk_palette, PALETTE_CHUNK_SIZE) !=
-      PALETTE_CHUNK_SIZE)
+      PALETTE_CHUNK_SIZE) {
+      free (this->palettes);
+      free (this->shot_offsets);
       return 0;
+    }
 
     /* convert and store the palette */
     for (j = 0; j < PALETTE_SIZE; j++) {
@@ -457,11 +437,16 @@ static int open_mve_file(demux_mve_t *this) {
    * BNAM, SIZE and perhaps others; traverse chunks until first BRCH
    * chunk is found */
   chunk_tag = 0;
+  title = NULL;
   while (chunk_tag != BRCH_TAG) {
 
     if (this->input->read(this->input, preamble, PREAMBLE_SIZE) !=
-      PREAMBLE_SIZE)
+      PREAMBLE_SIZE) {
+      free (title);
+      free (this->palettes);
+      free (this->shot_offsets);
       return 0;
+    }
 
     chunk_tag = BE_32(&preamble[0]);
     /* round up to the nearest even size */
@@ -475,22 +460,27 @@ static int open_mve_file(demux_mve_t *this) {
 
       case BNAM_TAG:
         /* load the name into the stream attributes */
-        this->stream->meta_info[XINE_META_INFO_TITLE] = 
-          xine_xmalloc(chunk_size);
-        if (this->input->read(this->input, 
-          this->stream->meta_info[XINE_META_INFO_TITLE], chunk_size) !=
-          chunk_size)
+        title = realloc (title, chunk_size);
+        if (this->input->read(this->input, title, chunk_size) != chunk_size) {
+          free (title);
+          free (this->palettes);
+          free (this->shot_offsets);
           return 0;
+        }
         break;
 
       case SIZE_TAG:
         /* override the default width and height */
         /* reuse the preamble bytes */
         if (this->input->read(this->input, preamble, PREAMBLE_SIZE) !=
-          PREAMBLE_SIZE)
+          PREAMBLE_SIZE) {
+          free (title);
+          free (this->palettes);
+          free (this->shot_offsets);
           return 0;
-        this->video_width = BE_32(&preamble[0]);
-        this->video_height = BE_32(&preamble[4]);
+        }
+        this->bih.biWidth = BE_32(&preamble[0]);
+        this->bih.biHeight = BE_32(&preamble[4]);
         break;
 
       case INDX_TAG:
@@ -500,7 +490,7 @@ static int open_mve_file(demux_mve_t *this) {
 
       default:
         /* report an unknown chunk and skip it */
-        printf (_("demux_wc3movie: encountered unknown chunk: %c%c%c%c\n"),
+        lprintf("encountered unknown chunk: %c%c%c%c\n",
           (chunk_tag >> 24) & 0xFF,
           (chunk_tag >> 16) & 0xFF,
           (chunk_tag >>  8) & 0xFF,
@@ -508,21 +498,20 @@ static int open_mve_file(demux_mve_t *this) {
         this->input->seek(this->input, chunk_size, SEEK_CUR);
         break;
     }
-
   }
 
   /* note the data start offset */
   this->data_start = this->input->get_current_pos(this->input);
+  this->data_size  = this->input->get_length(this->input) - this->data_start;
+  this->video_pts  = 0;
 
-  this->data_size = this->input->get_length(this->input) - this->data_start;
-
-  this->video_pts = 0;
+  _x_meta_info_set(this->stream, XINE_META_INFO_TITLE, title);
 
   return 1;
 }
 
 static int demux_mve_seek (demux_plugin_t *this_gen,
-                           off_t start_pos, int start_time) {
+                           off_t start_pos, int start_time, int playing) {
 
   /*
    * MVE files are comprised of a series of SHOTs. A SHOT begins when the
@@ -543,6 +532,19 @@ static int demux_mve_seek (demux_plugin_t *this_gen,
   unsigned int chunk_size;
   int new_shot = -1;
 
+  start_time /= 1000;
+  start_pos = (off_t) ( (double) start_pos / 65535 *
+              this->data_size );
+  
+  this->status = DEMUX_OK;
+  _x_demux_flush_engine(this->stream);
+  this->seek_flag = 1;
+
+  /* if input is non-seekable, do not proceed with the rest of this
+   * seek function */  
+  if (!INPUT_IS_SEEKABLE(this->input))
+    return this->status;
+
   /* make sure the first shot has been recorded */
   if (this->shot_offsets[0] == 0) {
 
@@ -618,31 +620,17 @@ static int demux_mve_seek (demux_plugin_t *this_gen,
     new_shot = this->number_of_shots - 1;
   this->current_shot = new_shot;
 
-  /* reposition the stream and signal the demux loop to reset pts */
+  /* reposition the stream at new shot */
   this->input->seek(this->input, this->shot_offsets[new_shot], SEEK_SET);
-  this->seek_flag = 1;
 
-  this->status = DEMUX_OK;
-
-  xine_demux_flush_engine(this->stream);
-  
-  /* if thread is not running, initialize demuxer */
-  if( !this->stream->demux_thread_running ) {
-
-    /* send new pts */
-    xine_demux_control_newpts(this->stream, 0, 0);
-
-    this->status = DEMUX_OK;
-    this->seek_flag = 0;
-  }
-
-  return 0;
+  return this->status;
 }
 
 static void demux_mve_dispose (demux_plugin_t *this_gen) {
   demux_mve_t *this = (demux_mve_t *) this_gen;
 
   free(this->palettes);
+  free(this->shot_offsets);
   free(this);
 }
 
@@ -653,7 +641,6 @@ static int demux_mve_get_status (demux_plugin_t *this_gen) {
 }
 
 static int demux_mve_get_stream_length (demux_plugin_t *this_gen) {
-
   return 0;
 }
 
@@ -667,16 +654,10 @@ static int demux_mve_get_optional_data(demux_plugin_t *this_gen,
 }
 
 static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *stream,
-                                    input_plugin_t *input_gen) {
+                                    input_plugin_t *input) {
 
-  input_plugin_t *input = (input_plugin_t *) input_gen;
   demux_mve_t    *this;
 
-  if (! (input->get_capabilities(input) & INPUT_CAP_SEEKABLE)) {
-    printf(_("demux_mve.c: input not seekable, can not handle!\n"));
-    return NULL;
-  }
-
   this         = xine_xmalloc (sizeof (demux_mve_t));
   this->stream = stream;
   this->input  = input;
@@ -687,8 +668,6 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   this->demux_plugin.dispose           = demux_mve_dispose;
   this->demux_plugin.get_status        = demux_mve_get_status;
   this->demux_plugin.get_stream_length = demux_mve_get_stream_length;
-  this->demux_plugin.get_video_frame   = NULL;
-  this->demux_plugin.got_video_frame_cb= NULL;
   this->demux_plugin.get_capabilities  = demux_mve_get_capabilities;
   this->demux_plugin.get_optional_data = demux_mve_get_optional_data;
   this->demux_plugin.demux_class       = class_gen;
@@ -697,6 +676,19 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   switch (stream->content_detection_method) {
 
+  case METHOD_BY_EXTENSION: {
+    char *extensions, *mrl;
+
+    mrl = input->get_mrl (input);
+    extensions = class_gen->get_extensions (class_gen);
+
+    if (!_x_demux_check_extension (mrl, extensions)) {
+      free (this);
+      return NULL;
+    }
+  }
+  /* falling through is intended */
+
   case METHOD_BY_CONTENT:
   case METHOD_EXPLICIT:
 
@@ -707,39 +699,11 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   break;
 
-  case METHOD_BY_EXTENSION: {
-    char *ending, *mrl;
-
-    mrl = input->get_mrl (input);
-
-    ending = strrchr(mrl, '.');
-
-    if (!ending) {
-      free (this);
-      return NULL;
-    }
-
-    if (strncasecmp (ending, ".mve", 4)) {
-      free (this);
-      return NULL;
-    }
-
-    if (!open_mve_file(this)) {
-      free (this);
-      return NULL;
-    }
-
-  }
-
-  break;
-
   default:
     free (this);
     return NULL;
   }
 
-  strncpy (this->last_mrl, input->get_mrl (input), 1024);
-
   return &this->demux_plugin;
 }
 
@@ -760,19 +724,15 @@ static char *get_mimetypes (demux_class_t *this_gen) {
 }
 
 static void class_dispose (demux_class_t *this_gen) {
-
   demux_mve_class_t *this = (demux_mve_class_t *) this_gen;
 
   free (this);
 }
 
 void *demux_wc3movie_init_plugin (xine_t *xine, void *data) {
-
   demux_mve_class_t     *this;
 
-  this         = xine_xmalloc (sizeof (demux_mve_class_t));
-  this->config = xine->config;
-  this->xine   = xine;
+  this = xine_xmalloc (sizeof (demux_mve_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
   this->demux_class.get_description = get_description;
@@ -783,15 +743,3 @@ void *demux_wc3movie_init_plugin (xine_t *xine, void *data) {
 
   return this;
 }
-
-/*
- * exported plugin catalog entry
- */
-
-#if 0
-plugin_info_t xine_plugin_info[] = {
-  /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_DEMUX, 20, "wc3movie", XINE_VERSION_CODE, NULL, demux_wc3movie_init_plugin },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
-};
-#endif
