@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,21 +17,20 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: dxr3.h,v 1.5 2002/10/26 14:35:04 mroi Exp $
+ * $Id: dxr3.h,v 1.8 2004/04/10 15:29:57 mroi Exp $
  */
 
 #ifndef HAVE_DXR3_H
 #define HAVE_DXR3_H
 
-#include <linux/em8300.h>
+#include "em8300.h"
 
 #include "xine_internal.h"
 
 /* data for the device name config entry */
-#define CONF_LOOKUP "dxr3.devicename"
-#define CONF_DEFAULT "/dev/em8300-0"
-#define CONF_NAME _("Dxr3: Device Name")
-#define CONF_HELP _("The device file of the dxr3 mpeg decoder card control device.")
+#define CONF_KEY  "dxr3.device_number"
+#define CONF_NAME _("DXR3 device number")
+#define CONF_HELP _("If you have more than one DXR3 in your computer, you can specify which one to use here.")
 
 /* image format used by dxr3_decoder to tag undecoded mpeg data */
 #define XINE_IMGFMT_DXR3 (('3'<<24)|('R'<<16)|('X'<<8)|'D')
