@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_file.c,v 1.98 2004/12/20 21:22:20 mroi Exp $
+ * $Id: input_file.c,v 1.98.2.2 2005/07/17 21:49:59 miguelfreitas Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -241,6 +241,7 @@ static char *decode_uri (char *uri) {
 static int file_plugin_open (input_plugin_t *this_gen ) {
   file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
   char                *filename;
+  struct stat          sbuf;
 
   lprintf("file_plugin_open\n");
 
@@ -274,7 +275,7 @@ static int file_plugin_open (input_plugin_t *this_gen ) {
     }
     else
       this->fh = open(this->mrl, O_RDONLY|O_BINARY);
-    
+
     if (this->fh == -1) {
       if (errno == EACCES) {
         _x_message(this->stream, XINE_MSG_PERMISSION_ERROR, this->mrl, NULL);
@@ -290,6 +291,20 @@ static int file_plugin_open (input_plugin_t *this_gen ) {
     }
   }
 
+  /* don't check length of fifo */
+  if (fstat (this->fh, &sbuf) == 0) {
+    if (S_ISFIFO(sbuf.st_mode))
+      return 1;
+  }
+
+  if (file_plugin_get_length (this_gen) == 0) {
+      _x_message(this->stream, XINE_MSG_FILE_EMPTY, this->mrl, NULL);
+      close (this->fh);
+      xine_log (this->stream->xine, XINE_LOG_MSG,
+		_("input_file: File empty: >%s<\n"), this->mrl);
+      return 0;
+  }
+
   return 1;
 }
 
@@ -891,6 +906,6 @@ static void *init_plugin (xine_t *xine, void *data) {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_INPUT, 15, "FILE", XINE_VERSION_CODE, NULL, init_plugin },
+  { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 16, "FILE", XINE_VERSION_CODE, NULL, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
