@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002 the xine project
+ * Copyright (C) 2002-2003 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_mms.c,v 1.34 2003/01/18 17:19:45 tmattern Exp $
+ * $Id: input_mms.c,v 1.54.2.1 2005/04/20 17:21:08 mroi Exp $
  *
  * mms input plugin based on work from major mms
  */
@@ -28,7 +28,6 @@
 
 #include <unistd.h>
 #include <stdio.h>
-#include <assert.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netdb.h>
@@ -36,8 +35,14 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include "bswap.h"
 
+#define LOG_MODULE "input_mms"
+#define LOG_VERBOSE
+/*
+#define LOG
+*/
+
+#include "bswap.h"
 #include "xine_internal.h"
 #include "xineutils.h"
 #include "input_plugin.h"
@@ -46,10 +51,6 @@
 #include "mmsh.h"
 #include "net_buf_ctrl.h"
 
-/*
-#define LOG
-*/
-
 #define PROTOCOL_UNDEFINED 0
 #define PROTOCOL_MMST      1
 #define PROTOCOL_MMSH      2
@@ -69,16 +70,18 @@ const char * mms_bandwidth_strs[]={"14.4 Kbps (Modem)", "19.2 Kbps (Modem)",
                                    "393.2 Kbps (Cable/DSL)","524.3 Kbps (Cable/DSL)",
                                    "1.5 Mbps (T1)", "10.5 Mbps (LAN)", NULL};
 
+/* connection methods */
+const char *mms_protocol_strs[]={"auto", "TCP", "HTTP", NULL};
+
 typedef struct {
   input_plugin_t   input_plugin;
 
+  xine_stream_t   *stream;
   mms_t           *mms;
   mmsh_t          *mmsh;
 
   char            *mrl;
 
-  off_t            curpos;
-
   nbc_t           *nbc; 
 
   char             scratch[1025];
@@ -93,6 +96,7 @@ typedef struct {
   input_class_t       input_class;
   
   mms_input_plugin_t *ip;
+  int                 protocol;    /* autoprobe, mmst or mmsh */
 
   xine_t             *xine;
 } mms_input_class_t;
@@ -102,12 +106,7 @@ static off_t mms_plugin_read (input_plugin_t *this_gen,
   mms_input_plugin_t *this = (mms_input_plugin_t *) this_gen;
   off_t               n = 0;
 
-#ifdef LOG
-  printf ("input_mms: mms_plugin_read: %lld bytes ...\n",
-          len);
-#endif
-
-  nbc_check_buffers (this->nbc);
+  lprintf ("mms_plugin_read: %lld bytes ...\n", len);
 
   switch (this->protocol) {
     case PROTOCOL_MMST:
@@ -117,22 +116,17 @@ static off_t mms_plugin_read (input_plugin_t *this_gen,
       n = mmsh_read (this->mmsh, buf, len);
       break;
   }
-              
-  this->curpos += n;
 
   return n;
 }
 
-static buf_element_t *mms_plugin_read_block (input_plugin_t *this_gen, 
+static buf_element_t *mms_plugin_read_block (input_plugin_t *this_gen,
                                              fifo_buffer_t *fifo, off_t todo) {
   /*mms_input_plugin_t   *this = (mms_input_plugin_t *) this_gen; */
   buf_element_t        *buf = fifo->buffer_pool_alloc (fifo);
   int                   total_bytes;
 
-#ifdef LOG
-  printf ("input_mms: mms_plugin_read_block: %lld bytes...\n",
-          todo);
-#endif
+  lprintf ("mms_plugin_read_block: %lld bytes...\n", todo);
 
   buf->content = buf->mem;
   buf->type = BUF_DEMUX_BLOCK;
@@ -151,40 +145,45 @@ static buf_element_t *mms_plugin_read_block (input_plugin_t *this_gen,
 
 static off_t mms_plugin_seek (input_plugin_t *this_gen, off_t offset, int origin) {
   mms_input_plugin_t   *this = (mms_input_plugin_t *) this_gen; 
-  off_t                 dest = this->curpos;
+  off_t                 dest = 0;
+  off_t                 curpos = 0;
 
-#ifdef LOG
-  printf ("input_mms: mms_plugin_seek: %lld offset, %d origin...\n",
-          offset, origin);
-#endif
+  lprintf ("mms_plugin_seek: %lld offset, %d origin...\n", offset, origin);
 
+
+  switch (this->protocol) {
+    case PROTOCOL_MMST:
+      curpos = mms_get_current_pos (this->mms);
+      break;
+    case PROTOCOL_MMSH:
+      curpos = mmsh_get_current_pos (this->mmsh);
+      break;
+  }
+  
   switch (origin) {
-  case SEEK_SET:
-    dest = offset;
-    break;
-  case SEEK_CUR:
-    dest = this->curpos + offset;
-    break;
-  case SEEK_END:
-    printf ("input_mms: SEEK_END not implemented!\n");
-    return this->curpos;
-  default:
-    printf ("input_mms: unknown origin in seek!\n");
-    return this->curpos;
+    case SEEK_SET:
+      dest = offset;
+      break;
+    case SEEK_CUR:
+      dest = curpos + offset;
+      break;
+    default:
+      printf ("input_mms: unknown origin in seek!\n");
+      return curpos;
   }
 
-  if (this->curpos > dest) {
+  if (curpos > dest) {
     printf ("input_mms: cannot seek back!\n");
-    return this->curpos;
+    return curpos;
   }
-
-  while (this->curpos<dest) {
+    
+  while (curpos < dest) {
     int n = 0;
     int diff;
 
-    diff = dest - this->curpos;
+    diff = dest - curpos;
 
-    if (diff>1024)
+    if (diff > 1024)
       diff = 1024;
 
     switch (this->protocol) {
@@ -196,14 +195,14 @@ static off_t mms_plugin_seek (input_plugin_t *this_gen, off_t offset, int origin
         break;
     }
     
-    this->curpos += n;
+    curpos += n;
 
     if (n < diff)
-      return this->curpos;
+      return curpos;
 
   }
 
-  return this->curpos;
+  return curpos;
 }
 
 static off_t mms_plugin_get_length (input_plugin_t *this_gen) {
@@ -222,9 +221,7 @@ static off_t mms_plugin_get_length (input_plugin_t *this_gen) {
       break;
   }
 
-#ifdef LOG
-  printf ("input_mms: length is %lld\n", length);
-#endif
+  lprintf ("length is %lld\n", length);
 
   return length;
 
@@ -240,29 +237,31 @@ static uint32_t mms_plugin_get_blocksize (input_plugin_t *this_gen) {
 
 static off_t mms_plugin_get_current_pos (input_plugin_t *this_gen){
   mms_input_plugin_t *this = (mms_input_plugin_t *) this_gen;
+  off_t curpos = 0;
+  
+  switch (this->protocol) {
+    case PROTOCOL_MMST:
+      curpos = mms_get_current_pos(this->mms);
+      break;
+    case PROTOCOL_MMSH:
+      curpos = mmsh_get_current_pos(this->mmsh);
+      break;
+  }
 
-  /*
-  printf ("current pos is %lld\n", this->curpos);
-  */
-
-  return this->curpos;
+  return curpos;
 }
 
 static void mms_plugin_dispose (input_plugin_t *this_gen) {
   mms_input_plugin_t *this = (mms_input_plugin_t *) this_gen;
 
-  if (this->mms) {
-    switch (this->protocol) {
-      case PROTOCOL_MMST:
-        mms_close (this->mms);
-        break;
-      case PROTOCOL_MMSH:
-        mmsh_close (this->mmsh);
-        break;
-    }
-    this->mms  = NULL;
-    this->mmsh = NULL;
-  }
+  if (this->mms)
+    mms_close (this->mms);
+  
+  if (this->mmsh)
+    mmsh_close (this->mmsh);
+  
+  this->mms  = NULL;
+  this->mmsh = NULL;
 
   if (this->nbc) {
     nbc_close (this->nbc);
@@ -290,10 +289,10 @@ static int mms_plugin_get_optional_data (input_plugin_t *this_gen,
   case INPUT_OPTIONAL_DATA_PREVIEW:
     switch (this->protocol) {
       case PROTOCOL_MMST:
-        return mms_peek_header (this->mms, data);
+        return mms_peek_header (this->mms, data, MAX_PREVIEW_SIZE);
         break;
       case PROTOCOL_MMSH:
-        return mmsh_peek_header (this->mmsh, data);
+        return mmsh_peek_header (this->mmsh, data, MAX_PREVIEW_SIZE);
         break;
     }
     break;
@@ -307,12 +306,11 @@ static int mms_plugin_get_optional_data (input_plugin_t *this_gen,
   return INPUT_OPTIONAL_UNSUPPORTED;
 }
 
-void bandwidth_changed_cb (void *this_gen, xine_cfg_entry_t *entry) {
+static void bandwidth_changed_cb (void *this_gen, xine_cfg_entry_t *entry) {
   mms_input_class_t *class = (mms_input_class_t*) this_gen;
 
-#ifdef LOG
-  printf ("input_mms: bandwidth_changed_cb %d\n", entry->num_value);
-#endif
+  lprintf ("bandwidth_changed_cb %d\n", entry->num_value);
+
   if(!class)
    return;
 
@@ -323,23 +321,61 @@ void bandwidth_changed_cb (void *this_gen, xine_cfg_entry_t *entry) {
   }
 }
 
-static input_plugin_t *open_plugin (input_class_t *cls_gen, xine_stream_t *stream, 
+static void protocol_changed_cb (void *this_gen, xine_cfg_entry_t *entry) {
+  mms_input_class_t *class = (mms_input_class_t*) this_gen;
+
+  lprintf ("protocol_changed_cb %d\n", entry->num_value);
+
+  if (!class) return;
+  class->protocol = entry->num_value;
+}
+
+static int mms_plugin_open (input_plugin_t *this_gen) {
+  mms_input_plugin_t *this = (mms_input_plugin_t *) this_gen;
+  mms_t              *mms  = NULL;
+  mmsh_t             *mmsh = NULL;
+  
+  switch (this->protocol) {
+    case PROTOCOL_UNDEFINED:
+      mms = mms_connect (this->stream, this->mrl, this->bandwidth);
+      if (mms) {
+        this->protocol = PROTOCOL_MMST;
+      } else {
+        mmsh = mmsh_connect (this->stream, this->mrl, this->bandwidth);
+        this->protocol = PROTOCOL_MMSH;
+      }
+      break;
+    case PROTOCOL_MMST:
+      mms = mms_connect (this->stream, this->mrl, this->bandwidth);
+      break;
+    case PROTOCOL_MMSH:
+      mmsh = mmsh_connect (this->stream, this->mrl, this->bandwidth);
+      break;
+  }
+  
+  if (!mms && !mmsh) {
+    return 0;
+  }
+  
+  this->mms      = mms;
+  this->mmsh     = mmsh;
+  
+  return 1;
+}
+
+static input_plugin_t *mms_class_get_instance (input_class_t *cls_gen, xine_stream_t *stream, 
 				    const char *data) {
 
   mms_input_class_t  *cls = (mms_input_class_t *) cls_gen;
   mms_input_plugin_t *this;
-  mms_t              *mms  = NULL;
-  mmsh_t             *mmsh = NULL;
   char               *mrl  = strdup(data);
   xine_cfg_entry_t    bandwidth_entry;
   int                 protocol;
   
-#ifdef LOG
-  printf ("input_mms: trying to open '%s'\n", mrl);
-#endif
+  lprintf ("trying to open '%s'\n", mrl);
 
   if (!strncasecmp (mrl, "mms://", 6)) {
-    protocol = PROTOCOL_UNDEFINED;
+    protocol = cls->protocol;
   } else if (!strncasecmp (mrl, "mmst://", 7)) {
     protocol =   PROTOCOL_MMST;
   } else if (!strncasecmp (mrl, "mmsh://", 7)) {
@@ -349,44 +385,21 @@ static input_plugin_t *open_plugin (input_class_t *cls_gen, xine_stream_t *strea
     return NULL;
   }
 
-  this = (mms_input_plugin_t *) malloc (sizeof (mms_input_plugin_t));
+  this = (mms_input_plugin_t *) xine_xmalloc (sizeof (mms_input_plugin_t));
   cls->ip = this;
-  
-  if (xine_config_lookup_entry (stream->xine, "input.mms_network_bandwidth", 
+  this->stream   = stream;
+  this->mms      = NULL;
+  this->mmsh     = NULL;
+  this->protocol = protocol;
+  this->mrl      = mrl; 
+  this->nbc      = nbc_init (this->stream);
+
+  if (xine_config_lookup_entry (stream->xine, "media.network.bandwidth", 
                                 &bandwidth_entry)) {
     bandwidth_changed_cb(cls, &bandwidth_entry);
   }
-    
-  switch (protocol) {
-    case PROTOCOL_UNDEFINED:
-      mms = mms_connect (stream, mrl, this->bandwidth);
-      if (mms) {
-        protocol = PROTOCOL_MMST;
-      } else {
-        mmsh = mmsh_connect (stream, mrl, this->bandwidth);
-        protocol = PROTOCOL_MMSH;
-      }
-      break;
-    case PROTOCOL_MMST:
-      mms = mms_connect (stream, mrl, this->bandwidth);
-      break;
-    case PROTOCOL_MMSH:
-      mmsh = mmsh_connect (stream, mrl, this->bandwidth);
-      break;
-  }
-  
-  if (!mms && !mmsh) {
-    free (mrl);
-    return NULL;
-  }
-  
-  this->mms      = mms;
-  this->mmsh     = mmsh;
-  this->protocol = protocol;
-  this->mrl      = mrl; 
-  this->curpos   = 0;
-  this->nbc      = nbc_init (stream);
 
+  this->input_plugin.open              = mms_plugin_open;
   this->input_plugin.get_capabilities  = mms_plugin_get_capabilities;
   this->input_plugin.read              = mms_plugin_read;
   this->input_plugin.read_block        = mms_plugin_read_block;
@@ -411,7 +424,7 @@ static char *mms_class_get_description (input_class_t *this_gen) {
   return _("mms streaming input plugin");
 }
 
-static char *mms_class_get_identifier (input_class_t *this_gen) {
+static const char *mms_class_get_identifier (input_class_t *this_gen) {
   return "mms";
 }
 
@@ -430,7 +443,7 @@ static void *init_class (xine_t *xine, void *data) {
   this->xine   = xine;
   this->ip                             = NULL;
 
-  this->input_class.open_plugin        = open_plugin;
+  this->input_class.get_instance       = mms_class_get_instance;
   this->input_class.get_identifier     = mms_class_get_identifier;
   this->input_class.get_description    = mms_class_get_description;
   this->input_class.get_dir            = NULL;
@@ -438,11 +451,23 @@ static void *init_class (xine_t *xine, void *data) {
   this->input_class.dispose            = mms_class_dispose;
   this->input_class.eject_media        = NULL;
 
-  xine->config->register_enum(xine->config, "input.mms_network_bandwidth", 10,
+  xine->config->register_enum(xine->config, "media.network.bandwidth", 10,
 			      (char **)mms_bandwidth_strs,
-			      "Network bandwidth",
-			      NULL, 0, bandwidth_changed_cb, (void*) this);
-  
+			      _("network bandwidth"),
+			      _("Specify the bandwidth of your internet connection here. "
+			        "This will be used when streaming servers offer different versions "
+				"with different bandwidth requirements of the same stream."),
+			      0, bandwidth_changed_cb, (void*) this);
+
+  this->protocol = xine->config->register_enum(xine->config,
+    "media.network.mms_protocol", 
+    0, 
+    (char **)mms_protocol_strs, 
+    _("MMS protocol"),
+    _("Select the protocol to encapsulate MMS.\nTCP is better but you may need HTTP behind a firewall."),
+     20, 
+     protocol_changed_cb, (void*)this);
+
   return this;
 }
 
@@ -452,6 +477,6 @@ static void *init_class (xine_t *xine, void *data) {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_INPUT, 11, "mms", XINE_VERSION_CODE, NULL, init_class },
+  { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 16, "mms", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
