@@ -44,8 +44,8 @@
 #define ACCEL          (MOTION_ACCEL | IDCT_ACCEL)
 
 #include "vlc.h"
-
 /* original (non-patched) scan tables */
+
 static uint8_t mpeg2_scan_norm_orig[64] ATTR_ALIGN(16) =
 {
     /* Zig-Zag scan pattern */
@@ -59,7 +59,7 @@ static uint8_t mpeg2_scan_norm_orig[64] ATTR_ALIGN(16) =
     53,60,61,54,47,55,62,63
 };
 
-uint8_t mpeg2_scan_alt_orig[64] ATTR_ALIGN(16) =
+static uint8_t mpeg2_scan_alt_orig[64] ATTR_ALIGN(16) =
 {
     /* Alternate scan pattern */
     0,8,16,24,1,9,2,10,17,25,32,40,48,56,57,49,
@@ -68,10 +68,24 @@ uint8_t mpeg2_scan_alt_orig[64] ATTR_ALIGN(16) =
     53,61,22,30,7,15,23,31,38,46,54,62,39,47,55,63
 };
 
+uint8_t mpeg2_scan_alt_ptable[64] ATTR_ALIGN(16);
+uint8_t mpeg2_scan_norm_ptable[64] ATTR_ALIGN(16);
+uint8_t mpeg2_scan_orig_ptable[64] ATTR_ALIGN(16);
+
+void xvmc_setup_scan_ptable( void )
+{
+    int i;
+    for (i=0; i<64; ++i) {
+	mpeg2_scan_norm_ptable[mpeg2_scan_norm_orig[i]] = mpeg2_scan_norm[i];
+	mpeg2_scan_alt_ptable[mpeg2_scan_alt_orig[i]] = mpeg2_scan_alt[i];
+	mpeg2_scan_orig_ptable[i] = i;
+    }
+}
+    
 
 static int non_linear_quantizer_scale [] = {
-     0,  1,  2,  3,  4,  5,   6,   7,
-     8, 10, 12, 14, 16, 18,  20,  22,
+    0,  1,  2,  3,  4,  5,   6,   7,
+    8, 10, 12, 14, 16, 18,  20,  22,
     24, 28, 32, 36, 40, 44,  48,  52,
     56, 64, 72, 80, 88, 96, 104, 112
 };
@@ -389,8 +403,10 @@ static void get_xvmc_intra_block_B14 (picture_t * picture)
 {
     int i;
     int j;
+    int l;
     int val;
     uint8_t * scan = picture->scan;
+    uint8_t * scan_ptable = mpeg2_scan_orig_ptable;
     uint8_t * quant_matrix = picture->intra_quantizer_matrix;
     int quantizer_scale = picture->quantizer_scale;
     int mismatch;
@@ -401,15 +417,17 @@ static void get_xvmc_intra_block_B14 (picture_t * picture)
     int16_t * dest;
 
     dest = picture->mc->blockptr;
-
-    /* XvMC's IDCT must use non-patched scan tables */
+    
     if( picture->mc->xvmc_accel & IDCT_ACCEL ) {
-      if( scan == mpeg2_scan_norm )
-        scan = mpeg2_scan_norm_orig;
-      else
-        scan = mpeg2_scan_alt_orig;
+	if ( scan == mpeg2_scan_norm ) {
+	    scan =  mpeg2_scan_norm_orig; 
+	    scan_ptable = mpeg2_scan_norm_ptable;
+	} else {
+	    scan = mpeg2_scan_alt_orig;
+	    scan_ptable = mpeg2_scan_alt_ptable;
+	}
     }
-
+	    
     i = 0;
     mismatch = ~dest[0];
 
@@ -429,10 +447,11 @@ static void get_xvmc_intra_block_B14 (picture_t * picture)
 		break;	/* end of block */
 
 	normal_code:
-	    j = scan[i];
+	    l = scan_ptable[j = scan[i]];
+	    
 	    bit_buf <<= tab->len;
 	    bits += tab->len + 1;
-	    val = (tab->level * quantizer_scale * quant_matrix[j]) >> 4;
+	    val = (tab->level * quantizer_scale * quant_matrix[l]) >> 4;
 
 	    /* if (bitstream_get (1)) val = -val; */
 	    val = (val ^ SBITS (bit_buf, 1)) - SBITS (bit_buf, 1);
@@ -460,12 +479,12 @@ static void get_xvmc_intra_block_B14 (picture_t * picture)
 	    if (i >= 64)
 		break;	/* illegal, check needed to avoid buffer overflow */
 
-	    j = scan[i];
+	    l = scan_ptable[j = scan[i]];
 
 	    DUMPBITS (bit_buf, bits, 12);
 	    NEEDBITS (bit_buf, bits, bit_ptr);
 	    val = (SBITS (bit_buf, 12) *
-		   quantizer_scale * quant_matrix[j]) / 16;
+		   quantizer_scale * quant_matrix[l]) / 16;
 
 	    SATURATE (val);
 	    dest[j] = val;
@@ -513,8 +532,10 @@ static void get_xvmc_intra_block_B15 (picture_t * picture)
 {
     int i;
     int j;
+    int l;
     int val;
     uint8_t * scan = picture->scan;
+    uint8_t * scan_ptable = mpeg2_scan_orig_ptable;
     uint8_t * quant_matrix = picture->intra_quantizer_matrix;
     int quantizer_scale = picture->quantizer_scale;
     int mismatch;
@@ -526,14 +547,16 @@ static void get_xvmc_intra_block_B15 (picture_t * picture)
 
     dest = picture->mc->blockptr;
 
-    /* XvMC's IDCT must use non-patched scan tables */
     if( picture->mc->xvmc_accel & IDCT_ACCEL ) {
-      if( scan == mpeg2_scan_norm )
-        scan = mpeg2_scan_norm_orig;
-      else
-        scan = mpeg2_scan_alt_orig;
+	if ( scan == mpeg2_scan_norm ) {
+	    scan =  mpeg2_scan_norm_orig; 
+	    scan_ptable = mpeg2_scan_norm_ptable;
+	} else {
+	    scan = mpeg2_scan_alt_orig;
+	    scan_ptable = mpeg2_scan_alt_ptable;
+	}
     }
-
+	    	    
     i = 0;
     mismatch = ~dest[0];
 
@@ -552,10 +575,10 @@ static void get_xvmc_intra_block_B15 (picture_t * picture)
 	    if (i < 64) {
 
 	    normal_code:
-		j = scan[i];
+		l = scan_ptable[j = scan[i]];
 		bit_buf <<= tab->len;
 		bits += tab->len + 1;
-		val = (tab->level * quantizer_scale * quant_matrix[j]) >> 4;
+		val = (tab->level * quantizer_scale * quant_matrix[l]) >> 4;
 
 		/* if (bitstream_get (1)) val = -val; */
 		val = (val ^ SBITS (bit_buf, 1)) - SBITS (bit_buf, 1);
@@ -578,19 +601,18 @@ static void get_xvmc_intra_block_B15 (picture_t * picture)
 
 		/* escape code */
 
-		i += UBITS (bit_buf << 6, 6) - 64;
+                i += UBITS (bit_buf << 6, 6) - 64;
 		if (i >= 64)
 		    break;	/* illegal, check against buffer overflow */
 
-		j = scan[i];
+		l = scan_ptable[j = scan[i]];
 
 		DUMPBITS (bit_buf, bits, 12);
 		NEEDBITS (bit_buf, bits, bit_ptr);
 		val = (SBITS (bit_buf, 12) *
-		       quantizer_scale * quant_matrix[j]) / 16;
+		       quantizer_scale * quant_matrix[l]) / 16;
 
 		SATURATE (val);
-
 		dest[j] = val;
 		mismatch ^= val;
 
@@ -637,8 +659,10 @@ static void get_xvmc_non_intra_block (picture_t * picture)
 {
     int i;
     int j;
+    int l;
     int val;
     uint8_t * scan = picture->scan;
+    uint8_t * scan_ptable = mpeg2_scan_orig_ptable;
     uint8_t * quant_matrix = picture->non_intra_quantizer_matrix;
     int quantizer_scale = picture->quantizer_scale;
     int mismatch;
@@ -653,14 +677,16 @@ static void get_xvmc_non_intra_block (picture_t * picture)
 
     dest = picture->mc->blockptr;
 
-    /* XvMC's IDCT must use non-patched scan tables */
     if( picture->mc->xvmc_accel & IDCT_ACCEL ) {
-      if( scan == mpeg2_scan_norm )
-        scan = mpeg2_scan_norm_orig;
-      else
-        scan = mpeg2_scan_alt_orig;
+	if ( scan == mpeg2_scan_norm ) {
+	    scan =  mpeg2_scan_norm_orig; 
+	    scan_ptable = mpeg2_scan_norm_ptable;
+	} else {
+	    scan = mpeg2_scan_alt_orig;
+	    scan_ptable = mpeg2_scan_alt_ptable;
+	}
     }
-
+	    
     bit_buf = picture->bitstream_buf;
     bits = picture->bitstream_bits;
     bit_ptr = picture->bitstream_ptr;
@@ -683,10 +709,10 @@ static void get_xvmc_non_intra_block (picture_t * picture)
 		break;	/* end of block */
 
 	normal_code:
-	    j = scan[i];
+	    l = scan_ptable[j = scan[i]];
 	    bit_buf <<= tab->len;
 	    bits += tab->len + 1;
-	    val = ((2*tab->level+1) * quantizer_scale * quant_matrix[j]) >> 5;
+	    val = ((2*tab->level+1) * quantizer_scale * quant_matrix[l]) >> 5;
 
 	    /* if (bitstream_get (1)) val = -val; */
 	    val = (val ^ SBITS (bit_buf, 1)) - SBITS (bit_buf, 1);
@@ -713,16 +739,16 @@ static void get_xvmc_non_intra_block (picture_t * picture)
 
 	    /* escape code */
 
-	    i += UBITS (bit_buf << 6, 6) - 64;
+            i += UBITS (bit_buf << 6, 6) - 64;
 	    if (i >= 64)
 		break;	/* illegal, check needed to avoid buffer overflow */
 
-	    j = scan[i];
+	    l = scan_ptable[j = scan[i]];
 
 	    DUMPBITS (bit_buf, bits, 12);
 	    NEEDBITS (bit_buf, bits, bit_ptr);
 	    val = 2 * (SBITS (bit_buf, 12) + SBITS (bit_buf, 1)) + 1;
-	    val = (val * quantizer_scale * quant_matrix[j]) / 32;
+	    val = (val * quantizer_scale * quant_matrix[l]) / 32;
 
 	    SATURATE (val);
 	    dest[j] = val;
@@ -758,7 +784,6 @@ static void get_xvmc_non_intra_block (picture_t * picture)
 	}
 	break;	/* illegal, check needed to avoid buffer overflow */
     }
-
     dest[63] ^= mismatch & 1;
     DUMPBITS (bit_buf, bits, 2);	/* dump end of block code */
     picture->bitstream_buf = bit_buf;
@@ -770,8 +795,10 @@ static void get_xvmc_mpeg1_intra_block (picture_t * picture)
 {
     int i;
     int j;
+    int l;
     int val;
     uint8_t * scan = picture->scan;
+    uint8_t * scan_ptable = mpeg2_scan_orig_ptable;
     uint8_t * quant_matrix = picture->intra_quantizer_matrix;
     int quantizer_scale = picture->quantizer_scale;
     DCTtab * tab;
@@ -784,14 +811,16 @@ static void get_xvmc_mpeg1_intra_block (picture_t * picture)
 
     dest = picture->mc->blockptr;
 
-    /* XvMC's IDCT must use non-patched scan tables */
     if( picture->mc->xvmc_accel & IDCT_ACCEL ) {
-      if( scan == mpeg2_scan_norm )
-        scan = mpeg2_scan_norm_orig;
-      else
-        scan = mpeg2_scan_alt_orig;
+	if ( scan == mpeg2_scan_norm ) {
+	    scan =  mpeg2_scan_norm_orig; 
+	    scan_ptable = mpeg2_scan_norm_ptable;
+	} else {
+	    scan = mpeg2_scan_alt_orig;
+	    scan_ptable = mpeg2_scan_alt_ptable;
+	}
     }
-
+	    
     bit_buf = picture->bitstream_buf;
     bits = picture->bitstream_bits;
     bit_ptr = picture->bitstream_ptr;
@@ -808,10 +837,10 @@ static void get_xvmc_mpeg1_intra_block (picture_t * picture)
 		break;	/* end of block */
 
 	normal_code:
-	    j = scan[i];
+	    l = scan_ptable[j = scan[i]];
 	    bit_buf <<= tab->len;
 	    bits += tab->len + 1;
-	    val = (tab->level * quantizer_scale * quant_matrix[j]) >> 4;
+	    val = (tab->level * quantizer_scale * quant_matrix[l]) >> 4;
 
 	    /* oddification */
 	    val = (val - 1) | 1;
@@ -837,11 +866,11 @@ static void get_xvmc_mpeg1_intra_block (picture_t * picture)
 
 	    /* escape code */
 
-	    i += UBITS (bit_buf << 6, 6) - 64;
+            i += UBITS (bit_buf << 6, 6) - 64;
 	    if (i >= 64)
 		break;	/* illegal, check needed to avoid buffer overflow */
 
-	    j = scan[i];
+	    l = scan_ptable[j = scan[i]];
 
 	    DUMPBITS (bit_buf, bits, 12);
 	    NEEDBITS (bit_buf, bits, bit_ptr);
@@ -850,7 +879,7 @@ static void get_xvmc_mpeg1_intra_block (picture_t * picture)
 		DUMPBITS (bit_buf, bits, 8);
 		val = UBITS (bit_buf, 8) + 2 * val;
 	    }
-	    val = (val * quantizer_scale * quant_matrix[j]) / 16;
+	    val = (val * quantizer_scale * quant_matrix[l]) / 16;
 
 	    /* oddification */
 	    val = (val + ~SBITS (val, 1)) | 1;
@@ -898,8 +927,10 @@ static void get_xvmc_mpeg1_non_intra_block (picture_t * picture)
 {
     int i;
     int j;
+    int l;
     int val;
     uint8_t * scan = picture->scan;
+    uint8_t * scan_ptable = mpeg2_scan_orig_ptable;
     uint8_t * quant_matrix = picture->non_intra_quantizer_matrix;
     int quantizer_scale = picture->quantizer_scale;
     DCTtab * tab;
@@ -912,14 +943,16 @@ static void get_xvmc_mpeg1_non_intra_block (picture_t * picture)
 
     dest = picture->mc->blockptr;
 
-    /* XvMC's IDCT must use non-patched scan tables */
     if( picture->mc->xvmc_accel & IDCT_ACCEL ) {
-      if( scan == mpeg2_scan_norm )
-        scan = mpeg2_scan_norm_orig;
-      else
-        scan = mpeg2_scan_alt_orig;
+	if ( scan == mpeg2_scan_norm ) {
+	    scan =  mpeg2_scan_norm_orig; 
+	    scan_ptable = mpeg2_scan_norm_ptable;
+	} else {
+	    scan = mpeg2_scan_alt_orig;
+	    scan_ptable = mpeg2_scan_alt_ptable;
+	}
     }
-
+	    
     bit_buf = picture->bitstream_buf;
     bits = picture->bitstream_bits;
     bit_ptr = picture->bitstream_ptr;
@@ -942,10 +975,10 @@ static void get_xvmc_mpeg1_non_intra_block (picture_t * picture)
 		break;	/* end of block */
 
 	normal_code:
-	    j = scan[i];
+	    l = scan_ptable[j = scan[i]];
 	    bit_buf <<= tab->len;
 	    bits += tab->len + 1;
-	    val = ((2*tab->level+1) * quantizer_scale * quant_matrix[j]) >> 5;
+	    val = ((2*tab->level+1) * quantizer_scale * quant_matrix[l]) >> 5;
 
 	    /* oddification */
 	    val = (val - 1) | 1;
@@ -978,7 +1011,7 @@ static void get_xvmc_mpeg1_non_intra_block (picture_t * picture)
 	    if (i >= 64)
 		break;	/* illegal, check needed to avoid buffer overflow */
 
-	    j = scan[i];
+	    l = scan_ptable[j = scan[i]];
 
 	    DUMPBITS (bit_buf, bits, 12);
 	    NEEDBITS (bit_buf, bits, bit_ptr);
@@ -988,7 +1021,7 @@ static void get_xvmc_mpeg1_non_intra_block (picture_t * picture)
 		val = UBITS (bit_buf, 8) + 2 * val;
 	    }
 	    val = 2 * (val + SBITS (val, 1)) + 1;
-	    val = (val * quantizer_scale * quant_matrix[j]) / 32;
+	    val = (val * quantizer_scale * quant_matrix[l]) / 32;
 
 	    /* oddification */
 	    val = (val + ~SBITS (val, 1)) | 1;
@@ -1052,10 +1085,7 @@ static inline void slice_xvmc_intra_DCT (picture_t * picture, int cc,
 
     mpeg2_zero_block(picture->mc->blockptr);
 
-    picture->mc->blockptr[0] =
-	picture->dc_dct_pred[cc] << (3 - picture->intra_dc_precision);
-    //    memset (picture->mc->blockptr + 1, 0, 63 * sizeof (int16_t));
-    // TODO do we need to zero mem here?
+    picture->mc->blockptr[0] = picture->dc_dct_pred[cc] << (3 - picture->intra_dc_precision);
 
     if (picture->mpeg1) {
 	if (picture->picture_coding_type != D_TYPE)
@@ -1078,7 +1108,7 @@ static inline void slice_xvmc_intra_DCT (picture_t * picture, int cc,
 static inline void slice_xvmc_non_intra_DCT (picture_t * picture, uint8_t * dest,
 					int stride)
 {
-    mpeg2_zero_block(picture->mc->blockptr);
+  mpeg2_zero_block(picture->mc->blockptr);
 
     if (picture->mpeg1)
 	get_xvmc_mpeg1_non_intra_block (picture);
@@ -1092,31 +1122,6 @@ static inline void slice_xvmc_non_intra_DCT (picture_t * picture, uint8_t * dest
     picture->mc->blockptr += 64;
 }
 
-#define MOTION(table,ref,motion_x,motion_y,size,y)			      \
-    pos_x = 2 * picture->offset + motion_x;				      \
-    pos_y = 2 * picture->v_offset + motion_y + 2 * y;			      \
-    if ((pos_x > picture->limit_x) || (pos_y > picture->limit_y_ ## size))    \
-	return;								      \
-    xy_half = ((pos_y & 1) << 1) | (pos_x & 1);				      \
-    table[xy_half] (picture->dest[0] + y * picture->pitches[0] +	      \
-		    picture->offset, ref[0] + (pos_x >> 1) +		      \
-		    (pos_y >> 1) * picture->pitches[0], picture->pitches[0],  \
-		    size);						      \
-    motion_x /= 2;	motion_y /= 2;					      \
-    xy_half = ((motion_y & 1) << 1) | (motion_x & 1);			      \
-    table[4+xy_half] (picture->dest[1] + y/2 * picture->pitches[1] +	      \
-		      (picture->offset >> 1), ref[1] +			      \
-		      (((picture->offset + motion_x) >> 1) +		      \
-		       ((((picture->v_offset + motion_y) >> 1) + y/2) *	      \
-		        picture->pitches[1])), picture->pitches[1], size/2);  \
-    table[4+xy_half] (picture->dest[2] + y/2 * picture->pitches[2] +	      \
-		      (picture->offset >> 1), ref[2] +			      \
-		      (((picture->offset + motion_x) >> 1) +		      \
-		       ((((picture->v_offset + motion_y) >> 1) + y/2) *	      \
-		        picture->pitches[2])), picture->pitches[2], size/2)   \
-
-#define MOTION_FIELD(table,ref,motion_x,motion_y,dest_field,op,src_field)     \
-
 static void motion_mp1 (picture_t * picture, motion_t * motion,
 			void (** table) (uint8_t *, uint8_t *, int, int))
 {
@@ -1124,7 +1129,6 @@ static void motion_mp1 (picture_t * picture, motion_t * motion,
 #define bits (picture->bitstream_bits)
 #define bit_ptr (picture->bitstream_ptr)
     int motion_x, motion_y;
-    unsigned int pos_x, pos_y, xy_half;
 
     NEEDBITS (bit_buf, bits, bit_ptr);
     motion_x = (motion->pmv[0][0] +
@@ -1142,7 +1146,6 @@ static void motion_mp1 (picture_t * picture, motion_t * motion,
 				    motion->f_code[0] + motion->f_code[1]);
     motion->pmv[0][1] = motion_y;
 
-    MOTION (table, motion->ref[0], motion_x, motion_y, 16, 0);
 #undef bit_buf
 #undef bits
 #undef bit_ptr
@@ -1155,7 +1158,6 @@ static void motion_fr_frame (picture_t * picture, motion_t * motion,
 #define bits (picture->bitstream_bits)
 #define bit_ptr (picture->bitstream_ptr)
     int motion_x, motion_y;
-    unsigned int pos_x, pos_y, xy_half;
 
     NEEDBITS (bit_buf, bits, bit_ptr);
     motion_x = motion->pmv[0][0] + get_xvmc_motion_delta (picture,
@@ -1169,7 +1171,6 @@ static void motion_fr_frame (picture_t * picture, motion_t * motion,
     motion_y = bound_motion_vector (motion_y, motion->f_code[1]);
     motion->pmv[1][1] = motion->pmv[0][1] = motion_y;
 
-    MOTION (table, motion->ref[0], motion_x, motion_y, 16, 0);
 #undef bit_buf
 #undef bits
 #undef bit_ptr
@@ -1201,8 +1202,6 @@ static void motion_fr_field (picture_t * picture, motion_t * motion,
     /* motion_y = bound_motion_vector (motion_y, motion->f_code[1]); */
     motion->pmv[0][1] = motion_y << 1;
 
-    //    MOTION_FIELD (table, motion->ref[0], motion_x, motion_y, 0, & ~1, field);
-
     NEEDBITS (bit_buf, bits, bit_ptr);
     field = UBITS (bit_buf, 1);
     //TODO look at field select need bob  (weave ok)
@@ -1220,7 +1219,6 @@ static void motion_fr_field (picture_t * picture, motion_t * motion,
     /* motion_y = bound_motion_vector (motion_y, motion->f_code[1]); */
     motion->pmv[1][1] = motion_y << 1;
 
-    //    MOTION_FIELD (table, motion->ref[0], motion_x, motion_y, 1, & ~1, field);
 #undef bit_buf
 #undef bits
 #undef bit_ptr
@@ -1232,8 +1230,7 @@ static void motion_fr_dmv (picture_t * picture, motion_t * motion,
 #define bit_buf (picture->bitstream_buf)
 #define bits (picture->bitstream_bits)
 #define bit_ptr (picture->bitstream_ptr)
-    int motion_x, motion_y, dmv_x, dmv_y;
-    unsigned int xy_half, offset;
+    int motion_x, motion_y;
 
     // TODO field select ?? possible need to be 0
     picture->XvMC_mv_field_sel[0][0] = picture->XvMC_mv_field_sel[1][0] = 0;
@@ -1244,56 +1241,12 @@ static void motion_fr_dmv (picture_t * picture, motion_t * motion,
     motion_x = bound_motion_vector (motion_x, motion->f_code[0]);
     motion->pmv[1][0] = motion->pmv[0][0] = motion_x;
     NEEDBITS (bit_buf, bits, bit_ptr);
-    dmv_x = get_xvmc_dmv (picture);
 
     motion_y = (motion->pmv[0][1] >> 1) + get_xvmc_motion_delta (picture,
 							    motion->f_code[1]);
     /* motion_y = bound_motion_vector (motion_y, motion->f_code[1]); */
     motion->pmv[1][1] = motion->pmv[0][1] = motion_y << 1;
-    dmv_y = get_xvmc_dmv (picture);
 
-    //    m = picture->top_field_first ? 1 : 3;
-    //    other_x = ((motion_x * m + (motion_x > 0)) >> 1) + dmv_x;
-    //    other_y = ((motion_y * m + (motion_y > 0)) >> 1) + dmv_y - 1;
-    //    MOTION_FIELD (mpeg2_mc.put, motion->ref[0], other_x, other_y, 0, | 1, 0);
-
-    //    m = picture->top_field_first ? 3 : 1;
-    //    other_x = ((motion_x * m + (motion_x > 0)) >> 1) + dmv_x;
-    //    other_y = ((motion_y * m + (motion_y > 0)) >> 1) + dmv_y + 1;
-    //    MOTION_FIELD (mpeg2_mc.put, motion->ref[0], other_x, other_y, 1, & ~1, 0);
-
-    xy_half = ((motion_y & 1) << 1) | (motion_x & 1);
-    offset = (picture->offset + (motion_x >> 1) +
-	      (picture->v_offset + (motion_y & ~1)) * picture->pitches[0]);
-    mpeg2_mc.avg[xy_half]
-	(picture->dest[0] + picture->offset,
-	 motion->ref[0][0] + offset, 2 * picture->pitches[0], 8);
-    mpeg2_mc.avg[xy_half]
-	(picture->dest[0] + picture->pitches[0] + picture->offset,
-	 motion->ref[0][0] + picture->pitches[0] + offset,
-	 2 * picture->pitches[0], 8);
-    motion_x /= 2;	motion_y /= 2;
-    xy_half = ((motion_y & 1) << 1) | (motion_x & 1);
-    offset = (((picture->offset + motion_x) >> 1) +
-	      (((picture->v_offset >> 1) + (motion_y & ~1)) *
-	       picture->pitches[1]));
-    mpeg2_mc.avg[4+xy_half]
-	(picture->dest[1] + (picture->offset >> 1),
-	 motion->ref[0][1] + offset, 2 * picture->pitches[1], 4);
-    mpeg2_mc.avg[4+xy_half]
-	(picture->dest[1] + picture->pitches[1] + (picture->offset >> 1),
-	 motion->ref[0][1] + picture->pitches[1] + offset,
-	 2 * picture->pitches[1], 4);
-    offset = (((picture->offset + motion_x) >> 1) +
-	      (((picture->v_offset >> 1) + (motion_y & ~1)) *
-	       picture->pitches[2]));
-    mpeg2_mc.avg[4+xy_half]
-	(picture->dest[2] + (picture->offset >> 1),
-	 motion->ref[0][2] + offset, 2 * picture->pitches[2], 4);
-    mpeg2_mc.avg[4+xy_half]
-	(picture->dest[2] + picture->pitches[2] + (picture->offset >> 1),
-	 motion->ref[0][2] + picture->pitches[2] + offset,
-	 2 * picture->pitches[2], 4);
 #undef bit_buf
 #undef bits
 #undef bit_ptr
@@ -1303,12 +1256,10 @@ static void motion_reuse (picture_t * picture, motion_t * motion,
 			  void (** table) (uint8_t *, uint8_t *, int, int))
 {
     int motion_x, motion_y;
-    unsigned int pos_x, pos_y, xy_half;
 
     motion_x = motion->pmv[0][0];
     motion_y = motion->pmv[0][1];
 
-    MOTION (table, motion->ref[0], motion_x, motion_y, 16, 0);
 }
 
 /* like motion_frame, but parsing without actual motion compensation */
@@ -1345,7 +1296,6 @@ static void motion_fi_field (picture_t * picture, motion_t * motion,
 #define bit_ptr (picture->bitstream_ptr)
     int motion_x, motion_y;
     uint8_t ** ref_field;
-    unsigned int pos_x, pos_y, xy_half;
 
     NEEDBITS (bit_buf, bits, bit_ptr);
     ref_field = motion->ref2[UBITS (bit_buf, 1)];
@@ -1366,7 +1316,6 @@ static void motion_fi_field (picture_t * picture, motion_t * motion,
     motion_y = bound_motion_vector (motion_y, motion->f_code[1]);
     motion->pmv[1][1] = motion->pmv[0][1] = motion_y;
 
-    MOTION (table, ref_field, motion_x, motion_y, 16, 0);
 #undef bit_buf
 #undef bits
 #undef bit_ptr
@@ -1380,7 +1329,6 @@ static void motion_fi_16x8 (picture_t * picture, motion_t * motion,
 #define bit_ptr (picture->bitstream_ptr)
     int motion_x, motion_y;
     uint8_t ** ref_field;
-    unsigned int pos_x, pos_y, xy_half;
 
     NEEDBITS (bit_buf, bits, bit_ptr);
     ref_field = motion->ref2[UBITS (bit_buf, 1)];
@@ -1401,7 +1349,6 @@ static void motion_fi_16x8 (picture_t * picture, motion_t * motion,
     motion_y = bound_motion_vector (motion_y, motion->f_code[1]);
     motion->pmv[0][1] = motion_y;
 
-    MOTION (table, ref_field, motion_x, motion_y, 8, 0);
 
     NEEDBITS (bit_buf, bits, bit_ptr);
     ref_field = motion->ref2[UBITS (bit_buf, 1)];
@@ -1422,7 +1369,6 @@ static void motion_fi_16x8 (picture_t * picture, motion_t * motion,
     motion_y = bound_motion_vector (motion_y, motion->f_code[1]);
     motion->pmv[1][1] = motion_y;
 
-    MOTION (table, ref_field, motion_x, motion_y, 8, 8);
 #undef bit_buf
 #undef bits
 #undef bit_ptr
@@ -1434,8 +1380,7 @@ static void motion_fi_dmv (picture_t * picture, motion_t * motion,
 #define bit_buf (picture->bitstream_buf)
 #define bits (picture->bitstream_bits)
 #define bit_ptr (picture->bitstream_ptr)
-    int motion_x, motion_y, other_x, other_y;
-    unsigned int pos_x, pos_y, xy_half;
+    int motion_x, motion_y;
 
     NEEDBITS (bit_buf, bits, bit_ptr);
     motion_x = motion->pmv[0][0] + get_xvmc_motion_delta (picture,
@@ -1443,25 +1388,21 @@ static void motion_fi_dmv (picture_t * picture, motion_t * motion,
     motion_x = bound_motion_vector (motion_x, motion->f_code[0]);
     motion->pmv[1][0] = motion->pmv[0][0] = motion_x;
     NEEDBITS (bit_buf, bits, bit_ptr);
-    other_x = ((motion_x + (motion_x > 0)) >> 1) + get_xvmc_dmv (picture);
 
     motion_y = motion->pmv[0][1] + get_xvmc_motion_delta (picture,
 						     motion->f_code[1]);
     motion_y = bound_motion_vector (motion_y, motion->f_code[1]);
     motion->pmv[1][1] = motion->pmv[0][1] = motion_y;
-    other_y = (((motion_y + (motion_y > 0)) >> 1) + get_xvmc_dmv (picture) +
-	       picture->dmv_offset);
 
     // TODO field select may need to do something here for bob  (weave ok)
     picture->XvMC_mv_field_sel[0][0] = picture->XvMC_mv_field_sel[1][0] = 0;
 
-    MOTION (mpeg2_mc.put, motion->ref[0], motion_x, motion_y, 16, 0);
-    MOTION (mpeg2_mc.avg, motion->ref[1], other_x, other_y, 16, 0);
 #undef bit_buf
 #undef bits
 #undef bit_ptr
 }
 
+
 static void motion_fi_conceal (picture_t * picture)
 {
 #define bit_buf (picture->bitstream_buf)
@@ -1631,7 +1572,8 @@ static inline int slice_xvmc_init (picture_t * picture, int code)
     picture->limit_y = height - 16;
 
     //TODO conversion to signed format signed format
-    if(picture->mc->xvmc_accel == MOTION_ACCEL) {
+    if((picture->mc->xvmc_accel & ACCEL) == MOTION_ACCEL &&
+       !(picture->mc->xvmc_accel & SIGNED_INTRA)) {
       //Motion Comp only unsigned intra
       // original:
       picture->dc_dct_pred[0] = picture->dc_dct_pred[1] =
@@ -1696,7 +1638,7 @@ static inline int slice_xvmc_init (picture_t * picture, int code)
 #undef bit_ptr
 }
 
-void mpeg2_xvmc_slice (mpeg2dec_t *mpeg2dec, picture_t * picture, int code, uint8_t * buffer)
+void mpeg2_xvmc_slice (mpeg2dec_accel_t *accel, picture_t * picture, int code, uint8_t * buffer)
 {
 #define bit_buf (picture->bitstream_buf)
 #define bits (picture->bitstream_bits)
@@ -1705,10 +1647,11 @@ void mpeg2_xvmc_slice (mpeg2dec_t *mpeg2dec, picture_t * picture, int code, uint
     xine_xvmc_t *xvmc = (xine_xvmc_t *) picture->current_frame->accel_data;
 
     if (1 == code) {
-      mpeg2dec->xvmc_last_slice_code = 0;
+      accel->xvmc_last_slice_code = 0;
     }
-    if (code != mpeg2dec->xvmc_last_slice_code + 1)
-      return;
+    if ((code != accel->xvmc_last_slice_code + 1) &&
+	(code != accel->xvmc_last_slice_code))
+	return;
     
     bitstream_init (picture, buffer);
 
@@ -1887,7 +1830,8 @@ void mpeg2_xvmc_slice (mpeg2dec_t *mpeg2dec, picture_t * picture, int code, uint
 					 picture->pitches[2]); // cc2 croma
 	    }
 
-	    if(picture->mc->xvmc_accel == MOTION_ACCEL) {
+            if((picture->mc->xvmc_accel & ACCEL) == MOTION_ACCEL &&
+	       !(picture->mc->xvmc_accel & SIGNED_INTRA)) {
 	        // original:
 	        picture->dc_dct_pred[0] = picture->dc_dct_pred[1] =
 		    picture->dc_dct_pred[2] = 128 << picture->intra_dc_precision;
@@ -1936,7 +1880,7 @@ void mpeg2_xvmc_slice (mpeg2dec_t *mpeg2dec, picture_t * picture, int code, uint
 	    default:	/* end of slice, or error */
 		if (mpeg2_cpu_state_restore)
 		    mpeg2_cpu_state_restore (&cpu_state);
-		mpeg2dec->xvmc_last_slice_code++;
+		accel->xvmc_last_slice_code = code;
 		return;
 	    }
 	}
@@ -1944,7 +1888,8 @@ void mpeg2_xvmc_slice (mpeg2dec_t *mpeg2dec, picture_t * picture, int code, uint
 	mba_inc += mba->mba;
 	if (mba_inc) {
 	    //TODO  conversion to signed format signed format
-	  if(picture->mc->xvmc_accel == MOTION_ACCEL) {
+          if((picture->mc->xvmc_accel & ACCEL) == MOTION_ACCEL &&
+	     !(picture->mc->xvmc_accel & SIGNED_INTRA)) {
 	    // original:
 	    picture->dc_dct_pred[0] = picture->dc_dct_pred[1] =
 	      picture->dc_dct_pred[2] = 128 << picture->intra_dc_precision;
@@ -2010,7 +1955,6 @@ void mpeg2_xvmc_slice (mpeg2dec_t *mpeg2dec, picture_t * picture, int code, uint
 			}
 
 			picture->XvMC_mb_type = macroblock_modes & 0x1E;
-
 			picture->XvMC_x = picture->offset/16;
 			picture->XvMC_y = picture->v_offset/16;
 
@@ -2036,7 +1980,7 @@ void mpeg2_xvmc_slice (mpeg2dec_t *mpeg2dec, picture_t * picture, int code, uint
 	    }
 	}
     }
-    mpeg2dec->xvmc_last_slice_code++;
+    accel->xvmc_last_slice_code = code;
 #undef bit_buf
 #undef bits
 #undef bit_ptr
