@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * Copyright (C) Christian Vogler 
  *               cvogler@gradient.cis.upenn.edu - December 2001
@@ -20,7 +20,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: cc_decoder.c,v 1.17 2002/09/04 23:31:09 guenter Exp $
+ * $Id: cc_decoder.c,v 1.26 2004/08/27 19:33:21 valtri Exp $
  *
  * stuff needed to provide closed captioning decoding and display
  *
@@ -43,12 +43,6 @@
 #include "osd.h"
 #include "video_out/alphablend.h"
 #include "cc_decoder.h"
-
-#ifdef XINE_COMPILE
-#include "libspudec/spu.h"
-#else
-#include "spu.h"
-#endif
 #include "osd.h"
 
 /*
@@ -68,8 +62,15 @@
 /* number of text colors specified by EIA-608 standard */
 #define NUM_FG_COL 7
 
+#ifndef WIN32
 /* colors specified by the EIA 608 standard */
 enum { WHITE, GREEN, BLUE, CYAN, RED, YELLOW, MAGENTA, BLACK, TRANSPARENT };
+#else
+/* colors specified by the EIA 608 standard */
+enum { WHITE, GREEN, BLUE, CYAN, RED, YELLOW, MAGENTA, BLACK };
+#endif
+
+
 
 /* color mapping to OSD text color indices */
 static int text_colormap[NUM_FG_COL] = {
@@ -230,8 +231,8 @@ char *cc_schemes[NUM_CC_PALETTES + 1] = {
 /* mapping from PAC row code to actual CC row */
 static int  rowdata[] = {10, -1, 0, 1, 2, 3, 11, 12, 13, 14, 4, 5, 6,
 			 7, 8, 9};
-#warning "FIXME: do real TM" 
-/*  must be mapped as a music note in the captioning font */ 
+/* FIXME: do real TM */
+/* must be mapped as a music note in the captioning font */ 
 static char specialchar[] = {'','','','','T','','','','',
 			     TRANSP_SPACE,'','','','','',''};
 
@@ -266,7 +267,7 @@ struct cc_renderer_s {
      not yet displayed. So we remember the vpts of the show event, and use
      that as the vpts of the hide event upon an osd free.
   */
-#warning "FIXME: bug in OSD or SPU?"
+/*FIXME: bug in OSD or SPU?*/
   int64_t display_vpts;       /* vpts of currently displayed caption */
 
   /* this variable is an even worse hack: in some rare cases, the pts
@@ -285,7 +286,7 @@ struct cc_renderer_s {
 
   metronom_t *metronom;       /* the active xine metronom */
 
-  cc_config_t *cc_cfg;        /* captioning configuration */
+  cc_state_t *cc_state;        /* captioning configuration */
 };
 
 
@@ -352,13 +353,11 @@ struct cc_decoder_s {
 
   /* active OSD renderer */
   osd_renderer_t     *renderer;
-  /* caption display object */
-  osd_object_t       *cap_display;
   /* true when caption currently is displayed */
   int displayed;
 
   /* configuration and intrinsics of CC decoder */
-  cc_config_t *cc_cfg;
+  cc_state_t *cc_state;
 
   metronom_t *metronom;
 };
@@ -377,9 +376,14 @@ static void get_font_metrics(osd_renderer_t *renderer,
   *maxh = 0;
 
   renderer->set_font(testc, (char *) fontname, font_size);
+  renderer->set_encoding(testc, "iso-8859-1");
   for (c = 32; c < 256; c++) {
     int tw, th;
-    char buf[2] = { (char) c, '\0' };
+    char buf[2]; 
+
+    buf[0] = (char)c;
+    buf[1] = '\0';
+
     renderer->get_text_size(testc, buf, &tw, &th);
     *maxw = MAX(*maxw, tw);
     *maxh = MAX(*maxh, th);
@@ -524,7 +528,7 @@ static void ccrow_set_attributes(cc_renderer_t *renderer, cc_row_t *this,
 {
   const cc_attribute_t *attr = &this->cells[pos].attributes;
   const char *fontname;
-  cc_config_t *cap_info = renderer->cc_cfg;
+  cc_config_t *cap_info = renderer->cc_state->cc_cfg;
 
   if (attr->italic)
     fontname = cap_info->italic_font;
@@ -540,7 +544,7 @@ static void ccrow_render(cc_renderer_t *renderer, cc_row_t *this, int rownum)
   char buf[CC_COLUMNS + 1];
   int base_y;
   int pos = ccrow_find_next_text_part(this, 0);
-  cc_config_t *cap_info = renderer->cc_cfg;
+  cc_config_t *cap_info = renderer->cc_state->cc_cfg;
   osd_renderer_t *osd_renderer = renderer->osd_renderer;
 
   /* find y coordinate of caption */
@@ -630,7 +634,7 @@ static void ccrow_render(cc_renderer_t *renderer, cc_row_t *this, int rownum)
       /* make caption background a uniform box. Without this line, the */
       /* background is uneven for superscript characters. */
       /* Also pad left & right ends of caption to make it more readable */
-#warning "FIXME: There may be off-by one errors in the rendering - check with Miguel"
+/*FIXME: There may be off-by one errors in the rendering - check with Miguel*/
       if (seg == 0)
 	box_x1 -= renderer->max_char_width;
       if (seg == num_seg - 1)
@@ -643,8 +647,10 @@ static void ccrow_render(cc_renderer_t *renderer, cc_row_t *this, int rownum)
 	buf[i - seg_pos[seg]] = this->cells[i].c;
       buf[seg_pos[seg + 1] - seg_pos[seg]] = '\0';
       ccrow_set_attributes(renderer, this, seg_attr[seg]);
+
+      /* text is already mapped from EIA-608 into iso-8859-1 */
       osd_renderer->render_text(renderer->cap_display,
-				x + cumulative_seg_width[seg], y, buf,
+				x + cumulative_seg_width[seg], y, buf,          
 				textcol);
     }
 
@@ -791,7 +797,7 @@ static void ccmem_init(cc_memory_t *this)
 
 static void ccmem_exit(cc_memory_t *this)
 {
-#warning "FIXME: anything to deallocate?"
+/*FIXME: anything to deallocate?*/
 }
 
 
@@ -800,8 +806,8 @@ static void ccmem_exit(cc_memory_t *this)
 static void cc_renderer_build_palette(cc_renderer_t *this)
 {
   int i, j;
-  colorinfo_t *cc_text = cc_text_palettes[this->cc_cfg->cc_scheme];
-  uint8_t *cc_alpha = cc_alpha_palettes[this->cc_cfg->cc_scheme];
+  colorinfo_t *cc_text = cc_text_palettes[this->cc_state->cc_cfg->cc_scheme];
+  uint8_t *cc_alpha = cc_alpha_palettes[this->cc_state->cc_cfg->cc_scheme];
   
   memset(this->cc_palette, 0, sizeof (this->cc_palette));
   memset(this->cc_trans, 0, sizeof (this->cc_trans));
@@ -914,18 +920,19 @@ static void cc_renderer_adjust_osd_object(cc_renderer_t *this)
 						     this->height);
   this->osd_renderer->set_palette(this->cap_display, this->cc_palette,
 				  this->cc_trans);
+  this->osd_renderer->set_encoding(this->cap_display, "iso-8859-1");
 }
 
 
 cc_renderer_t *cc_renderer_open(osd_renderer_t *osd_renderer,
-				metronom_t *metronom, cc_config_t *cc_cfg,
+				metronom_t *metronom, cc_state_t *cc_state,
 				int video_width, int video_height)
 {
   cc_renderer_t *this = (cc_renderer_t *) xine_xmalloc(sizeof (cc_renderer_t));
 
   this->osd_renderer = osd_renderer;
   this->metronom = metronom;
-  this->cc_cfg = cc_cfg;
+  this->cc_state = cc_state;
   cc_renderer_update_cfg(this, video_width, video_height);
 #ifdef LOG_DEBUG
   printf("spucc: cc_renderer: open\n");
@@ -965,12 +972,12 @@ void cc_renderer_update_cfg(cc_renderer_t *this_obj, int video_width,
 
   /* find maximum text width and height for normal & italic captioning */
   /* font */
-  get_font_metrics(this_obj->osd_renderer, this_obj->cc_cfg->font,
-		   this_obj->cc_cfg->font_size, &fontw, &fonth);
+  get_font_metrics(this_obj->osd_renderer, this_obj->cc_state->cc_cfg->font,
+		   this_obj->cc_state->cc_cfg->font_size, &fontw, &fonth);
   this_obj->max_char_width = fontw;
   this_obj->max_char_height = fonth;
-  get_font_metrics(this_obj->osd_renderer, this_obj->cc_cfg->italic_font,
-		   this_obj->cc_cfg->font_size, &fontw, &fonth);
+  get_font_metrics(this_obj->osd_renderer, this_obj->cc_state->cc_cfg->italic_font,
+		   this_obj->cc_state->cc_cfg->font_size, &fontw, &fonth);
   this_obj->max_char_width = MAX(fontw, this_obj->max_char_width);
   this_obj->max_char_height = MAX(fonth, this_obj->max_char_height);
 #ifdef LOG_DEBUG
@@ -1000,11 +1007,11 @@ void cc_renderer_update_cfg(cc_renderer_t *this_obj, int video_width,
 
   if (required_w <= this_obj->video_width && 
       required_h <= this_obj->video_height) {
-    this_obj->cc_cfg->can_cc = 1;
+    this_obj->cc_state->can_cc = 1;
     cc_renderer_adjust_osd_object(this_obj);
   }
   else {
-    this_obj->cc_cfg->can_cc = 0;
+    this_obj->cc_state->can_cc = 0;
     cc_renderer_free_osd_object(this_obj);
     printf("spucc: required captioning area %dx%d exceeds screen %dx%d!\n"
 	   "       Captions disabled. Perhaps you should choose a smaller"
@@ -1045,13 +1052,13 @@ static void cc_hide_displayed(cc_decoder_t *this)
   printf("cc_decoder: cc_hide_displayed\n");
 #endif
 
-  if (cc_renderer_on_display(this->cc_cfg->renderer)) {
-    int64_t vpts = cc_renderer_calc_vpts(this->cc_cfg->renderer, this->pts,
+  if (cc_renderer_on_display(this->cc_state->renderer)) {
+    int64_t vpts = cc_renderer_calc_vpts(this->cc_state->renderer, this->pts,
 					  this->f_offset); 
 #ifdef LOG_DEBUG
     printf("cc_decoder: cc_hide_displayed: hiding caption %u at vpts %u\n", this->capid, vpts);
 #endif    
-    cc_renderer_hide_caption(this->cc_cfg->renderer, vpts);
+    cc_renderer_hide_caption(this->cc_state->renderer, vpts);
   }
 }
 
@@ -1063,13 +1070,13 @@ static void cc_show_displayed(cc_decoder_t *this)
 #endif
 
   if (cc_onscreen_displayable(this)) {
-    int64_t vpts = cc_renderer_calc_vpts(this->cc_cfg->renderer, this->pts,
+    int64_t vpts = cc_renderer_calc_vpts(this->cc_state->renderer, this->pts,
 					  this->f_offset);
 #ifdef LOG_DEBUG
     printf("cc_decoder: cc_show_displayed: showing caption %u at vpts %u\n", this->capid, vpts);
 #endif    
     this->capid++;
-    cc_renderer_show_caption(this->cc_cfg->renderer,
+    cc_renderer_show_caption(this->cc_state->renderer,
 			     &this->on_buf->channel[this->on_buf->channel_no],
 			     vpts);
   }
@@ -1436,11 +1443,11 @@ void decode_cc(cc_decoder_t *this, uint8_t *buffer, uint32_t buf_len,
 
 
 
-cc_decoder_t *cc_decoder_open(cc_config_t *cc_cfg)
+cc_decoder_t *cc_decoder_open(cc_state_t *cc_state)
 {
   cc_decoder_t *this = (cc_decoder_t *) xine_xmalloc(sizeof (cc_decoder_t));
   /* configfile stuff */
-  this->cc_cfg = cc_cfg;
+  this->cc_state = cc_state;
 
   ccmem_init(&this->buffer[0]);
   ccmem_init(&this->buffer[1]);
