@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_goom.c,v 1.24 2003/01/29 17:21:14 miguelfreitas Exp $
+ * $Id: xine_goom.c,v 1.56.2.1 2005/07/17 21:50:06 miguelfreitas Exp $
  *
  * GOOM post plugin.
  *
@@ -27,18 +27,22 @@
  */
 
 #include <stdio.h>
+#include <stdlib.h>
+#include <time.h>
+
+#define LOG_MODULE "goom"
+#define LOG_VERBOSE
+/*
+#define LOG
+*/
 
 #include "config.h"
 #include "xine_internal.h"
 #include "xineutils.h"
 #include "post.h"
 
-#include "goom_config.h"
-#include "goom_core.h"
+#include "goom.h"
 
-/*
-#define LOG
-*/
 #define NUMSAMPLES  512
 #define FPS          10
 
@@ -48,7 +52,7 @@
 /* colorspace conversion methods */
 const char * goom_csc_methods[]={
   "Fast but not photorealistic",
-  "Slow but looks better (mmx)",
+  "Slow but looks better",
   NULL
 };
 
@@ -68,34 +72,36 @@ struct post_plugin_goom_s {
 
   /* private data */
   xine_video_port_t *vo_port;
-  xine_stream_t     *stream;
+  post_out_t         video_output;
   
-  post_class_goom_t *class;  
+  post_class_goom_t *class;
+  
+  /* private metronom for syncing the video */
+  metronom_t        *metronom;
+  
+  /* goom context */
+  PluginInfo        *goom;
   
   int data_idx;
   gint16 data [2][512];
   audio_buffer_t buf;   /* dummy buffer just to hold a copy of audio data */
   
-  int bits;
-  int mode;
   int channels;
   int sample_rate;
   int sample_counter;
   int samples_per_frame;
   int width, height;
   int width_back, height_back;
+  double ratio;
   int fps;
-  int use_asm;
   int csc_method;
 
-  yuv_planes_t yuv;  
+  yuv_planes_t yuv;
+  
+  /* frame skipping */
+  int skip_frame;
 };
 
-typedef struct post_goom_out_s post_goom_out_t;
-struct post_goom_out_s {
-  xine_post_out_t     out;
-  post_plugin_goom_t *post;
-};
 
 /* plugin class initialization function */
 static void *goom_init_plugin(xine_t *xine, void *);
@@ -108,7 +114,7 @@ post_info_t goom_special_info = {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_POST | PLUGIN_MUST_PRELOAD, 2, "goom", XINE_VERSION_CODE, &goom_special_info, &goom_init_plugin },
+  { PLUGIN_POST | PLUGIN_MUST_PRELOAD, 9, "goom", XINE_VERSION_CODE, &goom_special_info, &goom_init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
 
@@ -125,7 +131,6 @@ static void           goom_class_dispose(post_class_t *class_gen);
 static void           goom_dispose(post_plugin_t *this_gen);
 
 /* rewire function */
-static int            goom_rewire_audio(xine_post_out_t *output, void *data);
 static int            goom_rewire_video(xine_post_out_t *output, void *data);
 
 static int goom_port_open(xine_audio_port_t *this, xine_stream_t *stream,
@@ -140,8 +145,14 @@ static void fps_changed_cb(void *data, xine_cfg_entry_t *cfg) {
   
   if(class->ip) {
     post_plugin_goom_t *this = class->ip;
+
+    if (cfg->num_value < 1)
+      cfg->num_value = 1;
+
     this->fps = cfg->num_value;
-    this->samples_per_frame = this->sample_rate / this->fps;
+
+    if(this->sample_rate)
+      this->samples_per_frame = this->sample_rate / this->fps;
   }
 }
 
@@ -163,16 +174,6 @@ static void height_changed_cb(void *data, xine_cfg_entry_t *cfg) {
   }
 }
 
-static void use_asm_changed_cb(void *data, xine_cfg_entry_t *cfg) {
-  post_class_goom_t *class = (post_class_goom_t*) data;
-  
-  if(class->ip) {
-    post_plugin_goom_t *this = class->ip;
-    this->use_asm = cfg->num_value;
-    goom_setAsmUse(this->use_asm);
-  }
-}
-
 static void csc_method_changed_cb(void *data, xine_cfg_entry_t *cfg) {
   post_class_goom_t *class = (post_class_goom_t*) data;
   
@@ -184,8 +185,8 @@ static void csc_method_changed_cb(void *data, xine_cfg_entry_t *cfg) {
 
 static void *goom_init_plugin(xine_t *xine, void *data)
 {
-  post_class_goom_t *this = (post_class_goom_t *)malloc(sizeof(post_class_goom_t));
-  config_values_t *cfg;
+  post_class_goom_t *this = (post_class_goom_t *)xine_xmalloc(sizeof(post_class_goom_t));
+  config_values_t   *cfg;
 
   if (!this)
     return NULL;
@@ -199,37 +200,29 @@ static void *goom_init_plugin(xine_t *xine, void *data)
   
   cfg = xine->config;
 
-  cfg->register_num (cfg, "post.goom_fps", FPS,
-                                 _("Frames per second to generate with Goom"),
-                                 NULL, 10, fps_changed_cb, this);
+  cfg->register_num (cfg, "effects.goom.fps", FPS,
+                                 _("frames per second to generate"),
+                                 _("With more frames per second, the animation will get "
+				   "smoother and faster, but will also require more CPU power."),
+				 10, fps_changed_cb, this);
 
-  cfg->register_num (cfg, "post.goom_width", GOOM_WIDTH,
-                                   _("Goom image width in pixels"),
-                                   NULL, 20, width_changed_cb, this);
+  cfg->register_num (cfg, "effects.goom.width", GOOM_WIDTH,
+                                   _("goom image width"),
+				   _("The width in pixels of the image to be generated."),
+                                   10, width_changed_cb, this);
   
-  cfg->register_num (cfg, "post.goom_height", GOOM_HEIGHT,
-                                    _("Goom image height in pixels"),
-                                    NULL, 20, height_changed_cb, this);
+  cfg->register_num (cfg, "effects.goom.height", GOOM_HEIGHT,
+                                    _("goom image height"),
+				    _("The height in pixels of the image to be generated."),
+                                    10, height_changed_cb, this);
   
 
-#ifdef ARCH_X86
-  if (xine_mm_accel() & MM_ACCEL_X86_MMX) {
-    cfg->register_bool (cfg, "post.goom_use_asm", 1,
-                             _("Use Goom asm optimizations"),
-                             NULL, 10, use_asm_changed_cb, this);
-  }
-#endif
-  
-#ifdef ARCH_PPC
-  cfg->register_bool (cfg, "post.goom_use_asm", 1,
-                           _("Use Goom asm optimizations"),
-                           NULL, 10, use_asm_changed_cb, this);
-#endif
-  
-  cfg->register_enum (cfg, "post.goom_csc_method", 0,
+  cfg->register_enum (cfg, "effects.goom.csc_method", 0,
                            (char **)goom_csc_methods,
-                           _("Colorspace conversion method used by Goom"),
-                           NULL, 20, csc_method_changed_cb, this);
+                           _("colorspace conversion method"),
+                           _("You can choose the colorspace conversion method used by goom.\n"
+			     "The available selections should be self-explaining."),
+			   20, csc_method_changed_cb, this);
 
   return &this->class;
 }
@@ -239,97 +232,77 @@ static post_plugin_t *goom_open_plugin(post_class_t *class_gen, int inputs,
 					 xine_audio_port_t **audio_target,
 					 xine_video_port_t **video_target)
 {
-  post_plugin_goom_t *this   = (post_plugin_goom_t *)malloc(sizeof(post_plugin_goom_t));
-  post_class_goom_t  *class  = (post_class_goom_t*) class_gen;
-  xine_post_in_t     *input  = (xine_post_in_t *)malloc(sizeof(xine_post_in_t));
-  post_goom_out_t    *output = (post_goom_out_t *)malloc(sizeof(post_goom_out_t));
-  post_goom_out_t    *outputv = (post_goom_out_t *)malloc(sizeof(post_goom_out_t));
+  post_plugin_goom_t *this  = (post_plugin_goom_t *)xine_xmalloc(sizeof(post_plugin_goom_t));
+  post_class_goom_t  *class = (post_class_goom_t*) class_gen;
+  post_in_t          *input;
+  post_out_t         *output;
+  post_out_t         *outputv;
   post_audio_port_t  *port;
-  xine_cfg_entry_t    fps_entry, width_entry, height_entry, use_asm_entry, csc_method_entry;
+  xine_cfg_entry_t    fps_entry, width_entry, height_entry, csc_method_entry;
 
-  if (!this || !input || !output || !outputv || !video_target || !video_target[0] ||
-      !audio_target || !audio_target[0] ) {
+  if (!this || !video_target || !video_target[0] || !audio_target || !audio_target[0]) {
     free(this);
-    free(input);
-    free(output);
-    free(outputv);
     return NULL;
   }
   
+  _x_post_init(&this->post, 1, 0);
+  
   /*
    * Lookup config entries.
    */
-  class->ip = this;
-#ifdef LOG
-  printf("goom: goom_open_plugin\n");
-#endif
+  this->class = class;
+  class->ip   = this;
+  this->vo_port = video_target[0];
+  
+  this->metronom = _x_metronom_init(1, 0, class->xine);
 
-  if(xine_config_lookup_entry(class->xine, "post.goom_fps",
+  lprintf("goom_open_plugin\n");
+
+  if(xine_config_lookup_entry(class->xine, "effects.goom.fps",
                               &fps_entry)) 
     fps_changed_cb(class, &fps_entry);
 
-  if(xine_config_lookup_entry(class->xine, "post.goom_width",
+  if(xine_config_lookup_entry(class->xine, "effects.goom.width",
                               &width_entry)) 
     width_changed_cb(class, &width_entry);
 
-  if(xine_config_lookup_entry(class->xine, "post.goom_height",
+  if(xine_config_lookup_entry(class->xine, "effects.goom.height",
                               &height_entry)) 
     height_changed_cb(class, &height_entry);
 
-  if(xine_config_lookup_entry(class->xine, "post.goom_use_asm",
-                              &use_asm_entry)) 
-    use_asm_changed_cb(class, &use_asm_entry);
-
-  if(xine_config_lookup_entry(class->xine, "post.goom_csc_method",
+  if(xine_config_lookup_entry(class->xine, "effects.goom.csc_method",
                               &csc_method_entry)) 
     csc_method_changed_cb(class, &csc_method_entry);
 
   this->width_back  = this->width;
   this->height_back = this->height;
-  goom_init (this->width_back, this->height_back, 0);
+
+  srand((unsigned int)time((time_t *)NULL));
+  this->goom = goom_init (this->width_back, this->height_back);
+
+  this->ratio = (double)this->width_back/(double)this->height_back;
 
   this->sample_counter = 0;
-  this->stream  = NULL;
-  this->vo_port = video_target[0];
   this->buf.mem = NULL;
   this->buf.mem_size = 0;  
 
-  port = post_intercept_audio_port(&this->post, audio_target[0]);
-  port->port.open = goom_port_open;
-  port->port.close = goom_port_close;
-  port->port.put_buffer = goom_port_put_buffer;
+  port = _x_post_intercept_audio_port(&this->post, audio_target[0], &input, &output);
+  port->new_port.open       = goom_port_open;
+  port->new_port.close      = goom_port_close;
+  port->new_port.put_buffer = goom_port_put_buffer;
   
-  input->name = "audio in";
-  input->type = XINE_POST_DATA_AUDIO;
-  input->data = (xine_audio_port_t *)&port->port;
-
-  output->out.name   = "audio out";
-  output->out.type   = XINE_POST_DATA_AUDIO;
-  output->out.data   = (xine_audio_port_t **)&port->original_port;
-  output->out.rewire = goom_rewire_audio;
-  output->post       = this;
-  
-  outputv->out.name   = "generated video";
-  outputv->out.type   = XINE_POST_DATA_VIDEO;
-  outputv->out.data   = (xine_video_port_t **)&this->vo_port;
-  outputv->out.rewire = goom_rewire_video;
-  outputv->post       = this;
-  
-  this->post.xine_post.audio_input    = (xine_audio_port_t **)malloc(sizeof(xine_audio_port_t *) * 2);
-  this->post.xine_post.audio_input[0] = &port->port;
-  this->post.xine_post.audio_input[1] = NULL;
-  this->post.xine_post.video_input    = (xine_video_port_t **)malloc(sizeof(xine_video_port_t *) * 1);
-  this->post.xine_post.video_input[0] = NULL;
-  
-  this->post.input  = xine_list_new();
-  this->post.output = xine_list_new();
-  
-  xine_list_append_content(this->post.input, input);
-  xine_list_append_content(this->post.output, output);
+  outputv                  = &this->video_output;
+  outputv->xine_out.name   = "generated video";
+  outputv->xine_out.type   = XINE_POST_DATA_VIDEO;
+  outputv->xine_out.data   = (xine_video_port_t **)&this->vo_port;
+  outputv->xine_out.rewire = goom_rewire_video;
+  outputv->post            = &this->post;
   xine_list_append_content(this->post.output, outputv);
   
+  this->post.xine_post.audio_input[0] = &port->new_port;
+
   this->post.dispose = goom_dispose;
-  
+
   return &this->post;
 }
 
@@ -351,95 +324,62 @@ static void goom_class_dispose(post_class_t *class_gen)
 
 static void goom_dispose(post_plugin_t *this_gen)
 {
-  post_plugin_goom_t *this = (post_plugin_goom_t *)this_gen;
-  xine_post_out_t *output = (xine_post_out_t *)xine_list_first_content(this_gen->output);
-  xine_video_port_t *port = *(xine_video_port_t **)output->data;
+  post_plugin_goom_t *this   = (post_plugin_goom_t *)this_gen;
 
-  goom_close();
+  if (_x_post_dispose(this_gen)) {
+    this->class->ip = NULL;
 
-  if (this->stream)
-    port->close(port, this->stream);
-    
-  free(this->post.xine_post.audio_input);
-  free(this->post.xine_post.video_input);
-  free(xine_list_first_content(this->post.input));
-  free(xine_list_first_content(this->post.output));
-  xine_list_free(this->post.input);
-  xine_list_free(this->post.output);
-  if(this->buf.mem)
-    free(this->buf.mem);
-  free(this);
-}
+    goom_close(this->goom);
 
+    this->metronom->exit(this->metronom);
 
-static int goom_rewire_audio(xine_post_out_t *output_gen, void *data)
-{
-  post_goom_out_t *output = (post_goom_out_t *)output_gen;
-  xine_audio_port_t *old_port = *(xine_audio_port_t **)output_gen->data;
-  xine_audio_port_t *new_port = (xine_audio_port_t *)data;
-  post_plugin_goom_t *this = (post_plugin_goom_t *)output->post;
-  
-  if (!data)
-    return 0;
-  if (this->stream) {
-    /* register our stream at the new output port */
-    old_port->close(old_port, this->stream);
-    new_port->open(new_port, this->stream, this->bits, this->sample_rate, this->mode);
+    if(this->buf.mem)
+      free(this->buf.mem);
+    free(this);
   }
-  /* reconnect ourselves */
-  *(xine_audio_port_t **)output_gen->data = new_port;
-  return 1;
 }
 
+
 static int goom_rewire_video(xine_post_out_t *output_gen, void *data)
 {
-  post_goom_out_t *output = (post_goom_out_t *)output_gen;
+  post_out_t *output = (post_out_t *)output_gen;
   xine_video_port_t *old_port = *(xine_video_port_t **)output_gen->data;
   xine_video_port_t *new_port = (xine_video_port_t *)data;
   post_plugin_goom_t *this = (post_plugin_goom_t *)output->post;
   
   if (!data)
     return 0;
-  if (this->stream) {
-    /* register our stream at the new output port */
-    old_port->close(old_port, this->stream);
-    new_port->open(new_port, this->stream);
-  }
+  /* register our stream at the new output port */
+  old_port->close(old_port, XINE_ANON_STREAM);
+  new_port->open(new_port, XINE_ANON_STREAM);
   /* reconnect ourselves */
-  *(xine_video_port_t **)output_gen->data = new_port;
+  this->vo_port = new_port;
   return 1;
 }
 
-static int mode_channels( int mode ) {
-  switch( mode ) {
-  case AO_CAP_MODE_MONO:
-    return 1;
-  case AO_CAP_MODE_STEREO:
-    return 2;
-  case AO_CAP_MODE_4CHANNEL:
-    return 4;
-  case AO_CAP_MODE_5CHANNEL:
-    return 5;
-  case AO_CAP_MODE_5_1CHANNEL:
-    return 6;
-  }
-  return 0;
-} 
-
 static int goom_port_open(xine_audio_port_t *port_gen, xine_stream_t *stream,
 		   uint32_t bits, uint32_t rate, int mode) {
 
   post_audio_port_t  *port = (post_audio_port_t *)port_gen;
   post_plugin_goom_t *this = (post_plugin_goom_t *)port->post;
 
-  this->bits = bits;
-  this->mode = mode;
-  this->channels = mode_channels(mode);
+  _x_post_rewire(&this->post);
+  _x_post_inc_usage(port);
+  
+  port->stream = stream;
+  port->bits = bits;
+  port->rate = rate;
+  port->mode = mode;
+  
+  this->channels = _x_ao_mode2channels(mode);
+  this->sample_rate = rate;
   this->samples_per_frame = rate / this->fps;
-  this->sample_rate = rate; 
-  this->stream = stream;
   this->data_idx = 0;
   init_yuv_planes(&this->yuv, this->width, this->height);
+  this->skip_frame = 0;
+  
+  this->vo_port->open(this->vo_port, XINE_ANON_STREAM);
+  this->metronom->set_master(this->metronom, stream->metronom);
 
   return port->original_port->open(port->original_port, stream, bits, rate, mode );
 }
@@ -451,9 +391,14 @@ static void goom_port_close(xine_audio_port_t *port_gen, xine_stream_t *stream )
 
   free_yuv_planes(&this->yuv);
 
-  this->stream = NULL;
+  port->stream = NULL;
+  
+  this->vo_port->close(this->vo_port, XINE_ANON_STREAM);
+  this->metronom->set_master(this->metronom, NULL);
  
   port->original_port->close(port->original_port, stream );
+  
+  _x_post_dec_usage(port);
 }
 
 static void goom_port_put_buffer (xine_audio_port_t *port_gen, 
@@ -462,12 +407,11 @@ static void goom_port_put_buffer (xine_audio_port_t *port_gen,
   post_audio_port_t  *port = (post_audio_port_t *)port_gen;
   post_plugin_goom_t *this = (post_plugin_goom_t *)port->post;
   vo_frame_t         *frame;
-  /* uint32_t *goom_frame; */
   uint8_t *goom_frame, *goom_frame_end;
   int16_t *data;
   int8_t *data8;
   int samples_used = 0;
-  uint64_t vpts = buf->vpts;
+  int64_t pts = buf->vpts;
   int i, j;
   uint8_t *dest_ptr;
   int width, height;
@@ -478,11 +422,11 @@ static void goom_port_put_buffer (xine_audio_port_t *port_gen,
     this->buf.mem_size = buf->mem_size;
   }
   memcpy(this->buf.mem, buf->mem, 
-         buf->num_frames*this->channels*((this->bits == 8)?1:2));
+         buf->num_frames*this->channels*((port->bits == 8)?1:2));
   this->buf.num_frames = buf->num_frames;
   
   /* pass data to original port */
-  port->original_port->put_buffer(port->original_port, buf, stream );  
+  port->original_port->put_buffer(port->original_port, buf, stream);  
   
   /* we must not use original data anymore, it should have already being moved
    * to the fifo of free audio buffers. just use our private copy instead.
@@ -495,7 +439,7 @@ static void goom_port_put_buffer (xine_audio_port_t *port_gen,
 
   do {
     
-    if( this->bits == 8 ) {
+    if( port->bits == 8 ) {
       data8 = (int8_t *)buf->mem;
       data8 += samples_used * this->channels;
   
@@ -518,91 +462,101 @@ static void goom_port_put_buffer (xine_audio_port_t *port_gen,
   
     if( this->sample_counter >= this->samples_per_frame &&
         this->data_idx == NUMSAMPLES ) {
-  
       this->data_idx = 0;
       samples_used += this->samples_per_frame;
 
-      goom_frame = (uint8_t *)goom_update (this->data, 0, 0, NULL, NULL);
-
       frame = this->vo_port->get_frame (this->vo_port, this->width_back, this->height_back,
-                                        XINE_VO_ASPECT_SQUARE, XINE_IMGFMT_YUY2,
+                                        this->ratio, XINE_IMGFMT_YUY2,
                                         VO_BOTH_FIELDS);
       
-      frame->pts = vpts;
-      vpts = 0;
+      frame->extra_info->invalid = 1;
+      frame->bad_frame = 0;
       frame->duration = 90000 * this->samples_per_frame / this->sample_rate;
+      frame->pts = pts;
+      this->metronom->got_video_frame(this->metronom, frame);
+      
       this->sample_counter -= this->samples_per_frame;
-  
-      /* Try to be fast */
-      dest_ptr = frame -> base[0];
-      goom_frame_end = goom_frame + 4 * (this->width_back * this->height_back);
 
-      if ((this->csc_method == 1) && 
-          (xine_mm_accel() & MM_ACCEL_X86_MMX)) {
-        int plane_ptr = 0;
+      if (!this->skip_frame) {
+        /* Try to be fast */
+        goom_frame = (uint8_t *)goom_update (this->goom, this->data, 0, 0, NULL, NULL);
 
-        while (goom_frame < goom_frame_end) {
-          uint8_t r, g, b;
+        dest_ptr = frame -> base[0];
+        goom_frame_end = goom_frame + 4 * (this->width_back * this->height_back);
+
+        if ((this->csc_method == 1) && 
+            (xine_mm_accel() & MM_ACCEL_X86_MMX)) {
+          int plane_ptr = 0;
+
+          while (goom_frame < goom_frame_end) {
+            uint8_t r, g, b;
       
-          /* don't take endianness into account since MMX is only available
-           * on Intel processors */
-          b = *goom_frame; goom_frame++;
-          g = *goom_frame; goom_frame++;
-          r = *goom_frame; goom_frame += 2;
+            /* don't take endianness into account since MMX is only available
+             * on Intel processors */
+            b = *goom_frame; goom_frame++;
+            g = *goom_frame; goom_frame++;
+            r = *goom_frame; goom_frame += 2;
 
-          this->yuv.y[plane_ptr] = COMPUTE_Y(r, g, b);
-          this->yuv.u[plane_ptr] = COMPUTE_U(r, g, b);
-          this->yuv.v[plane_ptr] = COMPUTE_V(r, g, b);
-          plane_ptr++;
-        }
+            this->yuv.y[plane_ptr] = COMPUTE_Y(r, g, b);
+            this->yuv.u[plane_ptr] = COMPUTE_U(r, g, b);
+            this->yuv.v[plane_ptr] = COMPUTE_V(r, g, b);
+            plane_ptr++;
+          }
 
-        yuv444_to_yuy2(&this->yuv, frame->base[0], frame->pitches[0]);
+          yuv444_to_yuy2(&this->yuv, frame->base[0], frame->pitches[0]);
 
-      } else {
+        } else {
+
+          while (goom_frame < goom_frame_end) {
+            uint8_t r1, g1, b1, r2, g2, b2;
 
-        while (goom_frame < goom_frame_end) {
-          uint8_t r1, g1, b1, r2, g2, b2;
-      
 #ifdef __BIG_ENDIAN__
-          goom_frame ++;
-          r1 = *goom_frame; goom_frame++;
-          g1 = *goom_frame; goom_frame++;
-          b1 = *goom_frame; goom_frame += 2;
-          r2 = *goom_frame; goom_frame++;
-          g2 = *goom_frame; goom_frame++;
-          b2 = *goom_frame; goom_frame++;
+            goom_frame ++;
+            r1 = *goom_frame; goom_frame++;
+            g1 = *goom_frame; goom_frame++;
+            b1 = *goom_frame; goom_frame += 2;
+            r2 = *goom_frame; goom_frame++;
+            g2 = *goom_frame; goom_frame++;
+            b2 = *goom_frame; goom_frame++;
 #else
-          b1 = *goom_frame; goom_frame++;
-          g1 = *goom_frame; goom_frame++;
-          r1 = *goom_frame; goom_frame += 2;
-          b2 = *goom_frame; goom_frame++;
-          g2 = *goom_frame; goom_frame++;
-          r2 = *goom_frame; goom_frame += 2;
+            b1 = *goom_frame; goom_frame++;
+            g1 = *goom_frame; goom_frame++;
+            r1 = *goom_frame; goom_frame += 2;
+            b2 = *goom_frame; goom_frame++;
+            g2 = *goom_frame; goom_frame++;
+            r2 = *goom_frame; goom_frame += 2;
 #endif
       
-          *dest_ptr = COMPUTE_Y(r1, g1, b1);
-          dest_ptr++;
-          *dest_ptr = COMPUTE_U(r1, g1, b1);
-          dest_ptr++;
-          *dest_ptr = COMPUTE_Y(r2, g2, b2);
-          dest_ptr++;
-          *dest_ptr = COMPUTE_V(r2, g2, b2);
-          dest_ptr++;
+            *dest_ptr = COMPUTE_Y(r1, g1, b1);
+            dest_ptr++;
+            *dest_ptr = COMPUTE_U(r1, g1, b1);
+            dest_ptr++;
+            *dest_ptr = COMPUTE_Y(r2, g2, b2);
+            dest_ptr++;
+            *dest_ptr = COMPUTE_V(r2, g2, b2);
+            dest_ptr++;
+          }
         }
-      }
 
-      frame->draw(frame, stream);
+        this->skip_frame = frame->draw(frame, XINE_ANON_STREAM);
+      } else {
+        frame->bad_frame = 1;
+        frame->draw(frame, XINE_ANON_STREAM);
+	this->skip_frame--;
+      }
       frame->free(frame);
       
       width  = this->width;
       height = this->height;
       if ((width != this->width_back) || (height != this->height_back)) {
-        goom_close();
-        goom_init (this->width, this->height, 0);
-        this->width_back = width;
-        this->height_back = height;
-      }
-      
+          goom_close(this->goom);
+          this->goom = goom_init (this->width, this->height);
+          this->width_back = width;
+          this->height_back = height;
+	  this->ratio = (double)width/(double)height;
+	  free_yuv_planes(&this->yuv);
+	  init_yuv_planes(&this->yuv, this->width, this->height);
+       }
     }
   } while( this->sample_counter >= this->samples_per_frame );
 }
