@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: video_out_xxmc.c,v 1.11 2004/12/12 22:01:30 mroi Exp $
+ * $Id: video_out_xxmc.c,v 1.11.2.2 2005/07/17 21:50:07 miguelfreitas Exp $
  *
  * video_out_xxmc.c, X11 decoding accelerated video extension interface for xine
  *
@@ -31,7 +31,7 @@
  *
  * overlay support by James Courtier-Dutton <James@superbug.demon.co.uk> - July 2001
  * X11 unscaled overlay support by Miguel Freitas - Nov 2003
- * XvMC VLD implementation by Thomas Hellstrm - August-Oct 2004
+ * XvMC VLD implementation by Thomas Hellstrm - 2004, 2005.
  * XvMC merge by Thomas Hellstrm - Sep 2004
  *
  */
@@ -39,9 +39,12 @@
 
 
 #include "xxmc.h"
+#include <unistd.h>
+
 
 static int gX11Fail;
-static void xxmc_frame_updates(xxmc_driver_t *driver, xxmc_frame_t *frame);
+static void xxmc_frame_updates(xxmc_driver_t *driver, xxmc_frame_t *frame, 
+			       int init_macroblocks);
 static void dispose_ximage (xxmc_driver_t *this, XShmSegmentInfo *shminfo,
 			    XvImage *myimage);
 
@@ -348,7 +351,59 @@ static void xxmc_xvmc_free_subpicture(xxmc_driver_t *this, XvMCSubpicture *sub)
 }
 
 /*
- * Here follows a number of callback function.
+ * Callback used by decoder to check that surfaces are still valid,
+ * and to lock the context so that it won't get destroyed during 
+ * decoding.
+ */
+
+
+static int xxmc_lock_and_validate_surfaces(vo_frame_t *cur_frame,
+					   vo_frame_t *fw_frame,
+					   vo_frame_t *bw_frame,
+					   unsigned pc_type)
+{
+  xxmc_driver_t 
+    *driver = (xxmc_driver_t *) cur_frame->driver;
+  xxmc_frame_t 
+    *frame;
+
+  xvmc_context_reader_lock( &driver->xvmc_lock );
+
+  switch(pc_type) {
+  case XINE_PICT_B_TYPE:
+    frame = (xxmc_frame_t *) bw_frame;
+    if (!xxmc_xvmc_surface_valid( driver, frame->xvmc_surf)) break;
+    /* fall through */
+  case XINE_PICT_P_TYPE:
+    frame = (xxmc_frame_t *) fw_frame;
+    if (!xxmc_xvmc_surface_valid( driver, frame->xvmc_surf)) break;
+    /* fall through */
+  default:
+    frame = (xxmc_frame_t *) cur_frame;
+    if (!xxmc_xvmc_surface_valid( driver, frame->xvmc_surf)) break;
+    return 0;
+  }
+  
+  xvmc_context_reader_unlock( &driver->xvmc_lock );
+  return -1;
+}
+
+/*
+ * Callback for decoder. Decoding temporarily halted. Release the context.
+ */
+
+static void xxmc_unlock_surfaces(vo_driver_t *this_gen)
+{
+  xxmc_driver_t 
+    *driver = (xxmc_driver_t *) this_gen;
+  
+  xvmc_context_reader_unlock( &driver->xvmc_lock );
+}
+
+/*
+ * Callback for decoder. 
+ * Check that the surface is vaid and 
+ * flush outstanding rendering requests on this surface.
  */
 
 static void xvmc_flush(vo_frame_t *this_gen) 
@@ -369,7 +424,6 @@ static void xvmc_flush(vo_frame_t *this_gen)
 
   XVMCLOCKDISPLAY( driver->display );
   frame->xxmc_data.result = XvMCFlushSurface( driver->display, frame->xvmc_surf );
-  frame->xxmc_data.result = XvMCSyncSurface( driver->display, frame->xvmc_surf );
   XVMCUNLOCKDISPLAY( driver->display );
 
   xvmc_context_reader_unlock( &driver->xvmc_lock );
@@ -377,13 +431,13 @@ static void xvmc_flush(vo_frame_t *this_gen)
 }
 
 
-
 /*
  * Callback function for the VO-loop to duplicate frame data.
  * YV12 and YUY2 formats are taken care of in the xine-engine.
+ * This one only deals with hardware surfaces and duplicates them
+ * using a call to XvMCBlendSubpicture2 with a blank subpicture.
  */
 
-
 static void xxmc_duplicate_frame_data(vo_frame_t *this_gen, 
 				      vo_frame_t *original) 
 {
@@ -409,7 +463,7 @@ static void xxmc_duplicate_frame_data(vo_frame_t *this_gen,
   this->format = original->format;
   this->ratio = original->ratio;
   
-  xxmc_frame_updates(driver,this); 
+  xxmc_frame_updates(driver,this,0); 
 
   /*
    * Allocate a dummy subpicture and copy using 
@@ -453,17 +507,7 @@ static uint32_t xxmc_get_capabilities (vo_driver_t *this_gen) {
 
 static void xxmc_frame_field (vo_frame_t *vo_img, int which_field) 
 {
-  xxmc_driver_t *this = (xxmc_driver_t *) vo_img->driver;
-
-  
   lprintf ("xvmc_frame_field\n");
-  if (this->xvmc_cap) {
-    if (this->xvmc_accel & (XINE_XVMC_ACCEL_IDCT | XINE_XVMC_ACCEL_MOCOMP)) {
-      this->macroblocks.num_blocks       = 0;
-      this->macroblocks.macroblockptr    = this->macroblocks.macroblockbaseptr;
-      this->macroblocks.xine_mc.blockptr = this->macroblocks.xine_mc.blockbaseptr;
-    }
-  }
 }
 
 static void xxmc_frame_dispose (vo_frame_t *vo_img) {
@@ -709,7 +753,6 @@ static XvImage *create_ximage (xxmc_driver_t *this, XShmSegmentInfo *shminfo,
 static void xxmc_dispose_context(xxmc_driver_t *driver)
 {
   if (driver->contextActive) {
-   
     if (driver->xvmc_accel & (XINE_XVMC_ACCEL_MOCOMP | XINE_XVMC_ACCEL_IDCT)) {
       xvmc_macroblocks_t *macroblocks = &driver->macroblocks;
 
@@ -739,6 +782,13 @@ static void xxmc_dispose_context(xxmc_driver_t *driver)
   }
 }
 
+/*
+ * Find a suitable XvMC Context according to the acceleration request
+ * passed to us in the xxmc variable, and to the acceleration type
+ * priority set up in this plugin. Result is returned in 
+ * driver->xvmc_cur_cap.
+ */
+
 static int xxmc_find_context(xxmc_driver_t *driver, xine_xxmc_t *xxmc,
 			     unsigned width, unsigned height)
 {
@@ -972,7 +1022,8 @@ static int xxmc_xvmc_update_context(xxmc_driver_t *driver, xxmc_frame_t *frame,
 }
 
 static void xxmc_frame_updates(xxmc_driver_t *driver, 
-			       xxmc_frame_t *frame)
+			       xxmc_frame_t *frame, 
+			       int init_macroblocks)
 {
   xine_xxmc_t *xxmc = &frame->xxmc_data;
 
@@ -997,46 +1048,47 @@ static void xxmc_frame_updates(xxmc_driver_t *driver,
   if (frame->xvmc_surf == NULL) {
     if (NULL == (frame->xvmc_surf = 
 		 xxmc_xvmc_alloc_surface( driver, &driver->context))) {
-      printf("video_out_xxmc: ERROR: Accelerated surface allocation failed.\n"
-	     "video_out_xxmc: You are probably out of framebuffer memory.\n"
-	     "video_out_xxmc: Falling back to software decoding.\n");
+      fprintf(stderr, "video_out_xxmc: ERROR: Accelerated surface allocation failed.\n"
+	      "video_out_xxmc: You are probably out of framebuffer memory.\n"
+	      "video_out_xxmc: Falling back to software decoding.\n");
       driver->xvmc_accel = 0;
       xxmc_dispose_context( driver );
       return;
     }        
-  }
+    xxmc->xvmc.macroblocks = (xine_macroblocks_t *) &driver->macroblocks;
+    xxmc->xvmc.macroblocks->xvmc_accel = (driver->unsigned_intra) ?
+      0 : XINE_VO_SIGNED_INTRA;
+    switch(driver->xvmc_accel) {
+    case XINE_XVMC_ACCEL_IDCT:
+      xxmc->xvmc.macroblocks->xvmc_accel |= XINE_VO_IDCT_ACCEL;
+      break;
+    case XINE_XVMC_ACCEL_MOCOMP:
+      xxmc->xvmc.macroblocks->xvmc_accel |= XINE_VO_MOTION_ACCEL;
+      break;
+    default:
+      xxmc->xvmc.macroblocks->xvmc_accel = 0;
+    }
 
-  xxmc->acceleration = driver->xvmc_accel;
-  xxmc->xvmc.macroblocks = (xine_macroblocks_t *) &driver->macroblocks;
-  xxmc->xvmc.macroblocks->xvmc_accel = (driver->unsigned_intra) ? 
-    0 : XINE_VO_SIGNED_INTRA;
-  switch(driver->xvmc_accel) {
-  case XINE_XVMC_ACCEL_IDCT:
-    xxmc->xvmc.macroblocks->xvmc_accel |= XINE_VO_IDCT_ACCEL;
-    break;
-  case XINE_XVMC_ACCEL_MOCOMP:
-    xxmc->xvmc.macroblocks->xvmc_accel |= XINE_VO_MOTION_ACCEL; 
-    break;
-  default:
-    xxmc->xvmc.macroblocks->xvmc_accel = 0;
-  }
-  driver->macroblocks.num_blocks = 0;
-  driver->macroblocks.macroblockptr = driver->macroblocks.macroblockbaseptr;
-  driver->macroblocks.xine_mc.blockptr = 
-    driver->macroblocks.xine_mc.blockbaseptr;
 
-  /*
-   * Accelerated callbacks.
-   */
-
-  xxmc->proc_xxmc_flush = xvmc_flush;
-  xxmc->xvmc.proc_macro_block = xxmc_xvmc_proc_macro_block;
-  frame->vo_frame.proc_duplicate_frame_data = xxmc_duplicate_frame_data;
+    xxmc->proc_xxmc_flush = xvmc_flush;
+    xxmc->proc_xxmc_lock_valid = xxmc_lock_and_validate_surfaces;
+    xxmc->proc_xxmc_unlock = xxmc_unlock_surfaces;
+    
+    xxmc->xvmc.proc_macro_block = xxmc_xvmc_proc_macro_block;
+    frame->vo_frame.proc_duplicate_frame_data = xxmc_duplicate_frame_data;
 #ifdef HAVE_VLDXVMC
-  xxmc->proc_xxmc_begin = xvmc_vld_frame;
-  xxmc->proc_xxmc_slice = xvmc_vld_slice;
+    xxmc->proc_xxmc_begin = xvmc_vld_frame;
+    xxmc->proc_xxmc_slice = xvmc_vld_slice;
 #endif
+  }
 
+  if (init_macroblocks) {
+    driver->macroblocks.num_blocks = 0;
+    driver->macroblocks.macroblockptr = driver->macroblocks.macroblockbaseptr;
+    driver->macroblocks.xine_mc.blockptr = 
+      driver->macroblocks.xine_mc.blockbaseptr;
+  }
+  xxmc->acceleration = driver->xvmc_accel;
 }
 
 
@@ -1117,6 +1169,45 @@ static void xxmc_do_update_frame_xv(vo_driver_t *this_gen,
   frame->vo_frame.format = frame->format;
 }
 
+/*
+ * Check if we need to change XvMC context due to an
+ * acceleration request change.
+ */
+
+static int xxmc_accel_update(xxmc_driver_t *driver,
+			     uint32_t last_request,
+			     uint32_t new_request)
+{
+  int k;
+
+  /*
+   * Same acceleration request. No need to change.
+   */ 
+
+  if (last_request == new_request) return 0;
+
+  /*
+   * Current acceleration not valid. Change.
+   */
+
+  if ((driver->xvmc_accel & new_request) == 0) return 1;
+
+  /*
+   * Test for possible use of a higher acceleration level.
+   */
+
+  for (k = 0; k < NUM_ACCEL_PRIORITY; ++k) {
+    if (last_request & accel_priority[k]) return 0;
+    if (new_request & accel_priority[k]) return 1;
+  }
+
+  /* 
+   * Should never get here.
+   */
+
+  return 0;
+}
+
 
 static void xxmc_do_update_frame(vo_driver_t *this_gen,
 				 vo_frame_t *frame_gen,
@@ -1130,16 +1221,18 @@ static void xxmc_do_update_frame(vo_driver_t *this_gen,
     xine_xxmc_t *xxmc = &frame->xxmc_data;
 
     xvmc_context_writer_lock( &this->xvmc_lock);
-    if ((this->last_accel_request != xxmc->acceleration) ||
+    if (xxmc_accel_update(this, this->last_accel_request, xxmc->acceleration) ||
 	(this->xvmc_mpeg != xxmc->mpeg) ||
 	(this->xvmc_width != width) ||
 	(this->xvmc_height != height)) {
       this->last_accel_request = xxmc->acceleration;
       xxmc_xvmc_update_context(this, frame, width, height);
-    } 
+    } else {
+      this->last_accel_request = xxmc->acceleration;
+    }
 
     if (this->contextActive) 
-      xxmc_frame_updates(this, frame);
+      xxmc_frame_updates(this, frame, 1);
 
     xxmc_do_update_frame_xv(this_gen, frame_gen, width, height, ratio, 
 			    xxmc->fallback_format, flags);
@@ -1477,6 +1570,7 @@ static void xxmc_display_frame (vo_driver_t *this_gen, vo_frame_t *frame_gen)
   xxmc_driver_t  *this  = (xxmc_driver_t *) this_gen;
   xxmc_frame_t   *frame = (xxmc_frame_t *) frame_gen;
   xine_xxmc_t *xxmc = &frame->xxmc_data;
+  int first_field;
 
   /*
    * queue frames (deinterlacing)
@@ -1512,17 +1606,37 @@ static void xxmc_display_frame (vo_driver_t *this_gen, vo_frame_t *frame_gen)
    * ask for offset and output size
    */
 
+  first_field = (frame->vo_frame.top_field_first) ? XVMC_TOP_FIELD : XVMC_BOTTOM_FIELD;
+  first_field = (this->bob) ? first_field : XVMC_TOP_FIELD;
+  this->cur_field = (this->deinterlace_enabled) ? first_field : XVMC_FRAME_PICTURE;
+
   xxmc_redraw_needed (this_gen);
   if (frame->format == XINE_IMGFMT_XXMC) {
     XVMCLOCKDISPLAY( this->display );
+    XvMCSyncSurface( this->display, frame->xvmc_surf );
     XvMCPutSurface( this->display, frame->xvmc_surf , this->drawable,
 		    this->sc.displayed_xoffset, this->sc.displayed_yoffset,
 		    this->sc.displayed_width, this->sc.displayed_height,
 		    this->sc.output_xoffset, this->sc.output_yoffset,
 		    this->sc.output_width, this->sc.output_height, 
-		    ((this->deinterlace_enabled) ? 
-		     XVMC_TOP_FIELD : XVMC_FRAME_PICTURE));
+		    this->cur_field);
     XVMCUNLOCKDISPLAY( this->display );
+    if (this->deinterlace_enabled && this->bob) {
+      unsigned 
+	ms_per_field = 500 * frame->vo_frame.duration / 90000 - 2;
+      
+      usleep(ms_per_field*1000);
+      this->cur_field = (frame->vo_frame.top_field_first) ? XVMC_BOTTOM_FIELD : XVMC_TOP_FIELD;
+
+      XVMCLOCKDISPLAY( this->display );
+      XvMCPutSurface( this->display, frame->xvmc_surf , this->drawable,
+		      this->sc.displayed_xoffset, this->sc.displayed_yoffset,
+		      this->sc.displayed_width, this->sc.displayed_height,
+		      this->sc.output_xoffset, this->sc.output_yoffset,
+		      this->sc.output_width, this->sc.output_height, 
+		      this->cur_field);
+      XVMCUNLOCKDISPLAY( this->display );
+    }      
   } else {
     XLockDisplay (this->display);
     if (this->use_shm) {
@@ -1711,12 +1825,13 @@ static int xxmc_gui_data_exchange (vo_driver_t *this_gen,
 	xxmc_clean_output_area(this,(frame->format == XINE_IMGFMT_XXMC));
       if (frame->format == XINE_IMGFMT_XXMC) {
 	XVMCLOCKDISPLAY( this->display );
+        XvMCSyncSurface( this->display, frame->xvmc_surf );
 	XvMCPutSurface( this->display, frame->xvmc_surf, this->drawable,
 			this->sc.displayed_xoffset, this->sc.displayed_yoffset,
 			this->sc.displayed_width, this->sc.displayed_height,
 			this->sc.output_xoffset, this->sc.output_yoffset,
 			this->sc.output_width, this->sc.output_height, 
-			((this->deinterlace_enabled) ? XVMC_TOP_FIELD : XVMC_FRAME_PICTURE)); 
+			this->cur_field); 
 	XVMCUNLOCKDISPLAY( this->display );
       } else {
 	XLockDisplay (this->display);
@@ -1967,6 +2082,12 @@ static void xxmc_update_nvidia_fix(void *this_gen, xine_cfg_entry_t *entry) {
   this->reverse_nvidia_palette = entry->num_value;
 }
 
+static void xxmc_update_bob(void *this_gen, xine_cfg_entry_t *entry) {
+  xxmc_driver_t *this = (xxmc_driver_t *) this_gen;
+
+  this->bob = entry->num_value;
+}
+
 
 static void checkXvMCCap( xxmc_driver_t *this, XvPortID xv_port) 
 {
@@ -2429,8 +2550,16 @@ static vo_driver_t *open_plugin (video_driver_class_t *class_gen, const void *vi
 			   _("There's a bug in NVIDIA's XvMC lib that makes red OSD colors\n"
 			     "look blue and vice versa. This option provides a workaround.\n"),
 			   10, xxmc_update_nvidia_fix, this);
+  this->bob =
+    config->register_bool (config, "video.device.xvmc_bob_deinterlacing", 0,
+			   _("Use bob as accelerated deinterlace method."),
+			   _("When interlacing is enabled for hardware accelerated frames,\n"
+			     "Alternate between top and bottom field at double the frame rate.\n"),
+			   10, xxmc_update_bob, this);
 
   this->deinterlace_enabled = 0;
+  this->cur_field = XVMC_FRAME_PICTURE;
+
 #ifdef HAVE_VLDXVMC
   printf("video_out_xxmc: Unichrome CPU saving is %s.\n",
 	 (this->cpu_save_enabled) ? "on":"off"); 
