@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2004 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_interface.c,v 1.38 2003/01/13 02:15:07 miguelfreitas Exp $
+ * $Id: xine_interface.c,v 1.87.2.2 2005/07/17 21:50:14 miguelfreitas Exp $
  *
  * convenience/abstraction layer, functions to implement
  * libxine's public interface
@@ -41,6 +41,8 @@
 #include <machine/endian.h>
 #endif
 
+#define XINE_ENGINE_INTERNAL
+
 #include "xine_internal.h"
 #include "audio_out.h"
 #include "video_out.h"
@@ -52,7 +54,11 @@
  */
 
 const char *xine_get_version_string(void) {
-  return VERSION;
+  return VERSION
+#ifndef NDEBUG
+    "[DEBUG]"
+#endif
+    ;
 }
 
 void xine_get_version (int *major, int *minor, int *sub) {
@@ -164,8 +170,7 @@ int xine_config_register_bool (xine_t *self,
  * and return status
  */
 
-static int xine_config_get_current_entry (xine_t *this, 
-					  xine_cfg_entry_t *entry) {
+static int config_get_current_entry (xine_t *this, xine_cfg_entry_t *entry) {
 
   config_values_t *config = this->config;
 
@@ -176,7 +181,6 @@ static int xine_config_get_current_entry (xine_t *this,
   entry->type           = config->cur->type;
   entry->str_value      = config->cur->str_value;
   entry->str_default    = config->cur->str_default;
-  entry->str_sticky     = config->cur->str_sticky;
   entry->num_value      = config->cur->num_value;
   entry->num_default    = config->cur->num_default;
   entry->range_min      = config->cur->range_min;
@@ -203,9 +207,9 @@ int  xine_config_get_first_entry (xine_t *this, xine_cfg_entry_t *entry) {
   config->cur = config->first;
 
   /* do not hand out unclaimed entries */
-  while (config->cur && config->cur->type == CONFIG_TYPE_UNKNOWN)
+  while (config->cur && config->cur->type == XINE_CONFIG_TYPE_UNKNOWN)
     config->cur = config->cur->next;
-  result = xine_config_get_current_entry (this, entry);
+  result = config_get_current_entry (this, entry);
   pthread_mutex_unlock(&config->config_lock);
 
   return result;
@@ -230,8 +234,8 @@ int xine_config_get_next_entry (xine_t *this, xine_cfg_entry_t *entry) {
   /* do not hand out unclaimed entries */
   do {
     config->cur = config->cur->next;
-  } while (config->cur && config->cur->type == CONFIG_TYPE_UNKNOWN);
-  result = xine_config_get_current_entry (this, entry);
+  } while (config->cur && config->cur->type == XINE_CONFIG_TYPE_UNKNOWN);
+  result = config_get_current_entry (this, entry);
   pthread_mutex_unlock(&config->config_lock);
 
   return result;
@@ -247,12 +251,12 @@ int xine_config_lookup_entry (xine_t *this, const char *key,
   int result;
   config_values_t *config = this->config;
 
-  pthread_mutex_lock(&config->config_lock);
   config->cur = config->lookup_entry (config, key);
+  pthread_mutex_lock(&config->config_lock);
   /* do not hand out unclaimed entries */
-  if (config->cur && config->cur->type == CONFIG_TYPE_UNKNOWN)
+  if (config->cur && config->cur->type == XINE_CONFIG_TYPE_UNKNOWN)
     config->cur = NULL;
-  result = xine_config_get_current_entry (this, entry);
+  result = config_get_current_entry (this, entry);
   pthread_mutex_unlock(&config->config_lock);
 
   return result;
@@ -277,9 +281,9 @@ void xine_config_update_entry (xine_t *this, const xine_cfg_entry_t *entry) {
     break;
 
   default:
-    printf ("xine_interface: error, unknown config entry type %d\n",
-	    entry->type);
-    abort();
+    xprintf (this, XINE_VERBOSITY_DEBUG, 
+	     "xine_interface: error, unknown config entry type %d\n", entry->type);
+    _x_abort();
   }
 }
   
@@ -304,19 +308,36 @@ void xine_config_reset (xine_t *this) {
   config->last = NULL;
   pthread_mutex_unlock(&config->config_lock);
 }
-  
+
+#ifndef XINE_DISABLE_DEPRECATED_FEATURES
 int xine_gui_send_vo_data (xine_stream_t *stream,
 			   int type, void *data) {
 
   return stream->video_driver->gui_data_exchange (stream->video_driver, 
 						  type, data);
 }
+#endif
+
+int xine_port_send_gui_data (xine_video_port_t *vo,
+			   int type, void *data) {
+
+  return vo->driver->gui_data_exchange (vo->driver, 
+						  type, data);
+}
 
 void xine_set_param (xine_stream_t *stream, int param, int value) {
 
   switch (param) {
   case XINE_PARAM_SPEED:
-    xine_set_speed (stream, value);
+    pthread_mutex_lock (&stream->frontend_lock);
+    _x_set_speed (stream, value);
+    pthread_mutex_unlock (&stream->frontend_lock);
+    break;
+  
+  case XINE_PARAM_FINE_SPEED:
+    pthread_mutex_lock (&stream->frontend_lock);
+    _x_set_fine_speed (stream, value);
+    pthread_mutex_unlock (&stream->frontend_lock);
     break;
 
   case XINE_PARAM_AV_OFFSET:
@@ -336,7 +357,7 @@ void xine_set_param (xine_stream_t *stream, int param, int value) {
     break;
 
   case XINE_PARAM_SPU_CHANNEL:
-    xine_select_spu_channel (stream, value);
+    _x_select_spu_channel (stream, value);
     break;
 
   case XINE_PARAM_VIDEO_CHANNEL:
@@ -348,134 +369,266 @@ void xine_set_param (xine_stream_t *stream, int param, int value) {
     break;
 
   case XINE_PARAM_AUDIO_VOLUME:
+    stream->xine->port_ticket->acquire(stream->xine->port_ticket, 0);
     if (stream->audio_out)
       stream->audio_out->set_property (stream->audio_out, AO_PROP_MIXER_VOL, value);
+    stream->xine->port_ticket->release(stream->xine->port_ticket, 0);
     break;
 
   case XINE_PARAM_AUDIO_MUTE:
+    stream->xine->port_ticket->acquire(stream->xine->port_ticket, 0);
     if (stream->audio_out)
       stream->audio_out->set_property (stream->audio_out, AO_PROP_MUTE_VOL, value);
+    stream->xine->port_ticket->release(stream->xine->port_ticket, 0);
     break;
     
   case XINE_PARAM_AUDIO_COMPR_LEVEL:
+    stream->xine->port_ticket->acquire(stream->xine->port_ticket, 0);
     if (stream->audio_out)
       stream->audio_out->set_property (stream->audio_out, AO_PROP_COMPRESSOR, value);
+    stream->xine->port_ticket->release(stream->xine->port_ticket, 0);
     break;
     
+  case XINE_PARAM_AUDIO_AMP_LEVEL:
+    stream->xine->port_ticket->acquire(stream->xine->port_ticket, 0);
+    if (stream->audio_out)
+      stream->audio_out->set_property (stream->audio_out, AO_PROP_AMP, value);
+    stream->xine->port_ticket->release(stream->xine->port_ticket, 0);
+    break;
+
+  case XINE_PARAM_AUDIO_AMP_MUTE:
+    stream->xine->port_ticket->acquire(stream->xine->port_ticket, 0);
+    if (stream->audio_out)
+      stream->audio_out->set_property (stream->audio_out, AO_PROP_AMP_MUTE, value);
+    stream->xine->port_ticket->release(stream->xine->port_ticket, 0);
+    break;
+
+  case XINE_PARAM_AUDIO_CLOSE_DEVICE:
+    stream->xine->port_ticket->acquire(stream->xine->port_ticket, 0);
+    if (stream->audio_out)
+      stream->audio_out->set_property (stream->audio_out, AO_PROP_CLOSE_DEVICE, value);
+    stream->xine->port_ticket->release(stream->xine->port_ticket, 0);
+    break;
+
+  case XINE_PARAM_EQ_30HZ:
+  case XINE_PARAM_EQ_60HZ:
+  case XINE_PARAM_EQ_125HZ:
+  case XINE_PARAM_EQ_250HZ:
+  case XINE_PARAM_EQ_500HZ:
+  case XINE_PARAM_EQ_1000HZ:
+  case XINE_PARAM_EQ_2000HZ:
+  case XINE_PARAM_EQ_4000HZ:
+  case XINE_PARAM_EQ_8000HZ:
+  case XINE_PARAM_EQ_16000HZ:
+    stream->xine->port_ticket->acquire(stream->xine->port_ticket, 0);
+    if (stream->audio_out)
+      stream->audio_out->set_property (stream->audio_out, 
+				       param - XINE_PARAM_EQ_30HZ + AO_PROP_EQ_30HZ, 
+				       value);
+    stream->xine->port_ticket->release(stream->xine->port_ticket, 0);
+    break;
+
   case XINE_PARAM_VERBOSITY:
     stream->xine->verbosity = value;
-    
+    break;
+
   case XINE_PARAM_VO_HUE:
   case XINE_PARAM_VO_SATURATION:
   case XINE_PARAM_VO_CONTRAST:
-  case XINE_PARAM_VO_BRIGHTNESS: {
-    int v, min_v, max_v, range_v;
-
-
-    stream->video_driver->get_property_min_max (stream->video_driver,
-						param & 0xffffff,
-						&min_v, &max_v);
-
-    range_v = max_v - min_v;
-
-    v = (value * range_v) / 65535 + min_v;
-
-    stream->video_driver->set_property(stream->video_driver, param & 0xffffff, v);
-  }
-    break;
-
+  case XINE_PARAM_VO_BRIGHTNESS:
   case XINE_PARAM_VO_DEINTERLACE:
   case XINE_PARAM_VO_ASPECT_RATIO:
   case XINE_PARAM_VO_ZOOM_X:
   case XINE_PARAM_VO_ZOOM_Y:
-  case XINE_PARAM_VO_PAN_SCAN:
   case XINE_PARAM_VO_TVMODE:
-    stream->video_driver->set_property(stream->video_driver, param & 0xffffff, value);
+  case XINE_PARAM_VO_CROP_LEFT:
+  case XINE_PARAM_VO_CROP_RIGHT:
+  case XINE_PARAM_VO_CROP_TOP:
+  case XINE_PARAM_VO_CROP_BOTTOM:
+    stream->xine->port_ticket->acquire(stream->xine->port_ticket, 0);
+    stream->video_out->set_property(stream->video_out, param, value);
+    stream->xine->port_ticket->release(stream->xine->port_ticket, 0);
+    break;
+
+  case XINE_PARAM_IGNORE_VIDEO:
+    _x_stream_info_set(stream, XINE_STREAM_INFO_IGNORE_VIDEO, value);
+    break;
+    
+  case XINE_PARAM_IGNORE_AUDIO:
+    _x_stream_info_set(stream, XINE_STREAM_INFO_IGNORE_AUDIO, value);
+    break;
+  
+  case XINE_PARAM_IGNORE_SPU:
+    _x_stream_info_set(stream, XINE_STREAM_INFO_IGNORE_SPU, value);
+    break;
+  
+  case XINE_PARAM_METRONOM_PREBUFFER:
+    stream->metronom->set_option(stream->metronom, METRONOM_PREBUFFER, value);
+    break;
+
+  case XINE_PARAM_BROADCASTER_PORT:
+    if( !stream->broadcaster && value ) {
+      stream->broadcaster = _x_init_broadcaster(stream, value);
+    } else if ( stream->broadcaster && !value ) {
+      _x_close_broadcaster(stream->broadcaster);
+      stream->broadcaster = NULL;
+    }
     break;
     
   default:
-    printf ("xine_interface: unknown param %d\n", param);
-  }
+    xprintf (stream->xine, XINE_VERBOSITY_DEBUG,
+	     "xine_interface: unknown or deprecated stream param %d set\n", param);
+  }  
 }
 
-int  xine_get_param (xine_stream_t *stream, int param) {
+int xine_get_param (xine_stream_t *stream, int param) {
+  int ret;
 
   switch (param) {
   case XINE_PARAM_SPEED:
-    return stream->xine->clock->speed;
+    ret = _x_get_speed(stream);
+    break;
+  
+  case XINE_PARAM_FINE_SPEED:
+    ret = _x_get_fine_speed(stream);
+    break;
 
   case XINE_PARAM_AV_OFFSET:
-    return stream->metronom->get_option (stream->metronom, METRONOM_AV_OFFSET);
+    ret = stream->metronom->get_option (stream->metronom, METRONOM_AV_OFFSET);
+    break;
   
   case XINE_PARAM_SPU_OFFSET:
-    return stream->metronom->get_option (stream->metronom, METRONOM_SPU_OFFSET);
+    ret = stream->metronom->get_option (stream->metronom, METRONOM_SPU_OFFSET);
+    break;
 
   case XINE_PARAM_AUDIO_CHANNEL_LOGICAL:
-    return stream->audio_channel_user;
+    ret = stream->audio_channel_user;
+    break;
 
   case XINE_PARAM_SPU_CHANNEL:
-    return stream->spu_channel_user;
+    ret = stream->spu_channel_user;
+    break;
 
   case XINE_PARAM_VIDEO_CHANNEL:
-    return stream->video_channel;
+    ret = stream->video_channel;
+    break;
 
   case XINE_PARAM_AUDIO_VOLUME:
+    stream->xine->port_ticket->acquire(stream->xine->port_ticket, 0);
     if (!stream->audio_out)
-      return -1;
-    return stream->audio_out->get_property (stream->audio_out, AO_PROP_MIXER_VOL);
+      ret = -1;
+    else
+      ret = stream->audio_out->get_property (stream->audio_out, AO_PROP_MIXER_VOL);
+    stream->xine->port_ticket->release(stream->xine->port_ticket, 0);  
+    break;
 
   case XINE_PARAM_AUDIO_MUTE:
+    stream->xine->port_ticket->acquire(stream->xine->port_ticket, 0);
     if (!stream->audio_out)
-      return -1;
-    return stream->audio_out->get_property (stream->audio_out, AO_PROP_MUTE_VOL);
+      ret = -1;
+    else
+      ret = stream->audio_out->get_property (stream->audio_out, AO_PROP_MUTE_VOL);
+    stream->xine->port_ticket->release(stream->xine->port_ticket, 0);  
+    break;
 
   case XINE_PARAM_AUDIO_COMPR_LEVEL:
+    stream->xine->port_ticket->acquire(stream->xine->port_ticket, 0);
     if (!stream->audio_out)
-      return -1;
-    return stream->audio_out->get_property (stream->audio_out, AO_PROP_COMPRESSOR);
+      ret = -1;
+    else
+      ret = stream->audio_out->get_property (stream->audio_out, AO_PROP_COMPRESSOR);
+    stream->xine->port_ticket->release(stream->xine->port_ticket, 0);  
+    break;
+
+  case XINE_PARAM_AUDIO_AMP_LEVEL:
+    stream->xine->port_ticket->acquire(stream->xine->port_ticket, 0);
+    if (!stream->audio_out)
+      ret = -1;
+    else
+      ret = stream->audio_out->get_property (stream->audio_out, AO_PROP_AMP);
+    stream->xine->port_ticket->release(stream->xine->port_ticket, 0);  
+    break;
+
+  case XINE_PARAM_AUDIO_AMP_MUTE:
+    stream->xine->port_ticket->acquire(stream->xine->port_ticket, 0);
+    if (!stream->audio_out)
+      ret = -1;
+    else
+      ret = stream->audio_out->get_property (stream->audio_out, AO_PROP_AMP_MUTE);
+    stream->xine->port_ticket->release(stream->xine->port_ticket, 0);  
+    break;
+
+  case XINE_PARAM_EQ_30HZ:
+  case XINE_PARAM_EQ_60HZ:
+  case XINE_PARAM_EQ_125HZ:
+  case XINE_PARAM_EQ_250HZ:
+  case XINE_PARAM_EQ_500HZ:
+  case XINE_PARAM_EQ_1000HZ:
+  case XINE_PARAM_EQ_2000HZ:
+  case XINE_PARAM_EQ_4000HZ:
+  case XINE_PARAM_EQ_8000HZ:
+  case XINE_PARAM_EQ_16000HZ:
+    stream->xine->port_ticket->acquire(stream->xine->port_ticket, 0);
+    if (!stream->audio_out)
+      ret = -1;
+    else
+      ret=  stream->audio_out->get_property (stream->audio_out, 
+					     param - XINE_PARAM_EQ_30HZ + AO_PROP_EQ_30HZ);
+    stream->xine->port_ticket->release(stream->xine->port_ticket, 0);  
+    break;
 
   case XINE_PARAM_VERBOSITY:
-    return stream->xine->verbosity;
-    
+    ret = stream->xine->verbosity;
+    break;
+
   case XINE_PARAM_VO_HUE:
   case XINE_PARAM_VO_SATURATION:
   case XINE_PARAM_VO_CONTRAST:
-  case XINE_PARAM_VO_BRIGHTNESS: {
-    int v, min_v, max_v, range_v;
-
-
-    stream->video_driver->get_property_min_max (stream->video_driver,
-						param & 0xffffff,
-						&min_v, &max_v);
-
-    v = stream->video_driver->get_property (stream->video_driver, param & 0xffffff);
-
-    range_v = max_v - min_v;
-
-    /* printf ("xine_interface: %d [%d %d]\n", v, min_v, max_v); */
-
-    if (range_v > 0)
-      return (v-min_v) * 65535 / range_v;
-    else 
-      return 0;
-  }
-    break;
-
-
+  case XINE_PARAM_VO_BRIGHTNESS:
   case XINE_PARAM_VO_DEINTERLACE:
   case XINE_PARAM_VO_ASPECT_RATIO:
   case XINE_PARAM_VO_ZOOM_X:
   case XINE_PARAM_VO_ZOOM_Y:
-  case XINE_PARAM_VO_PAN_SCAN:
   case XINE_PARAM_VO_TVMODE:
-    return stream->video_driver->get_property(stream->video_driver, param & 0xffffff);
+  case XINE_PARAM_VO_CROP_LEFT:
+  case XINE_PARAM_VO_CROP_RIGHT:
+  case XINE_PARAM_VO_CROP_TOP:
+  case XINE_PARAM_VO_CROP_BOTTOM:
+    stream->xine->port_ticket->acquire(stream->xine->port_ticket, 0);
+    ret = stream->video_out->get_property(stream->video_out, param);
+    stream->xine->port_ticket->release(stream->xine->port_ticket, 0);  
+    break;
+  
+  case XINE_PARAM_IGNORE_VIDEO:
+    ret = _x_stream_info_get_public(stream, XINE_STREAM_INFO_IGNORE_VIDEO);
+    break;
+    
+  case XINE_PARAM_IGNORE_AUDIO:
+    ret = _x_stream_info_get_public(stream, XINE_STREAM_INFO_IGNORE_AUDIO);
+    break;
+  
+  case XINE_PARAM_IGNORE_SPU:
+    ret = _x_stream_info_get_public(stream, XINE_STREAM_INFO_IGNORE_SPU);
+    break;
+
+  case XINE_PARAM_METRONOM_PREBUFFER:
+    ret = stream->metronom->get_option(stream->metronom, METRONOM_PREBUFFER);
+    break;
+  
+  case XINE_PARAM_BROADCASTER_PORT:
+    if( stream->broadcaster )
+      ret = _x_get_broadcaster_port(stream->broadcaster);
+    else
+      ret = 0;
     break;
 
   default:
-    printf ("xine_interface: unknown param %d\n", param);
+    xprintf (stream->xine, XINE_VERBOSITY_DEBUG,
+	     "xine_interface: unknown or deprecated stream param %d requested\n", param);
+    ret = 0;
   }
 
-  return 0;
+  return ret;
 }
 
 uint32_t xine_get_stream_info (xine_stream_t *stream, int info) {
@@ -488,8 +641,12 @@ uint32_t xine_get_stream_info (xine_stream_t *stream, int info) {
     return 0;
 
   case XINE_STREAM_INFO_HAS_CHAPTERS:
+    if (stream->demux_plugin)
+      if (stream->demux_plugin->get_capabilities (stream->demux_plugin) & DEMUX_CAP_CHAPTERS)
+        return 1;
     if (stream->input_plugin)
-      return stream->input_plugin->get_capabilities (stream->input_plugin) & INPUT_CAP_CHAPTERS;
+      if (stream->input_plugin->get_capabilities (stream->input_plugin) & INPUT_CAP_CHAPTERS)
+        return 1;
     return 0;
 
   case XINE_STREAM_INFO_BITRATE:
@@ -514,34 +671,45 @@ uint32_t xine_get_stream_info (xine_stream_t *stream, int info) {
   case XINE_STREAM_INFO_IGNORE_AUDIO:
   case XINE_STREAM_INFO_IGNORE_SPU:
   case XINE_STREAM_INFO_VIDEO_HAS_STILL:
-    return stream->stream_info[info];
+  case XINE_STREAM_INFO_VIDEO_AFD:
+  case XINE_STREAM_INFO_DVD_TITLE_NUMBER:
+  case XINE_STREAM_INFO_DVD_TITLE_COUNT:
+  case XINE_STREAM_INFO_DVD_CHAPTER_NUMBER:
+  case XINE_STREAM_INFO_DVD_CHAPTER_COUNT:
+  case XINE_STREAM_INFO_DVD_ANGLE_NUMBER:
+  case XINE_STREAM_INFO_DVD_ANGLE_COUNT:
+    return _x_stream_info_get_public(stream, info);
 
   case XINE_STREAM_INFO_MAX_AUDIO_CHANNEL:
     return stream->audio_track_map_entries;
 
   case XINE_STREAM_INFO_MAX_SPU_CHANNEL:
     return stream->spu_track_map_entries;
-
+  
   default:
-    printf ("xine_interface: error, unknown stream info (%d) requested\n",
-	    info);
+    xprintf (stream->xine, XINE_VERBOSITY_DEBUG,
+	     "xine_interface: unknown or deprecated stream info %d requested\n", info);
   }
   return 0;
 }
 
 const char *xine_get_meta_info (xine_stream_t *stream, int info) {
-
-  return stream->meta_info[info];
+  return _x_meta_info_get_public(stream, info);
 }
 
 xine_osd_t *xine_osd_new(xine_stream_t *stream, int x, int y, int width, int height) {
   xine_osd_t *this = (xine_osd_t *)stream->osd_renderer->new_object(stream->osd_renderer, width, height);
   this->osd.renderer->set_position(&this->osd, x, y);
+  this->osd.renderer->set_encoding(&this->osd, "");
   return this;
 }
-  
+
+uint32_t xine_osd_get_capabilities(xine_osd_t *this) {
+  return this->osd.renderer->get_capabilities(&this->osd);
+} 
+
 void xine_osd_draw_point(xine_osd_t *this, int x, int y, int color) {
-  this->osd.area[x + y * this->osd.width] = color;
+  this->osd.renderer->point(&this->osd, x, y, color);
 } 
 
 void xine_osd_draw_line(xine_osd_t *this, int x1, int y1, int x2, int y2, int color) {
@@ -567,8 +735,12 @@ void xine_osd_get_text_size(xine_osd_t *this, const char *text, int *width, int
   this->osd.renderer->get_text_size(&this->osd, text, width, height);
 }
 
-void xine_osd_set_font(xine_osd_t *this, const char *fontname, int size) {
-  this->osd.renderer->set_font(&this->osd, fontname, size);
+int xine_osd_set_font(xine_osd_t *this, const char *fontname, int size) {
+  return this->osd.renderer->set_font(&this->osd, fontname, size);
+}
+
+void xine_osd_set_encoding(xine_osd_t *this, const char *encoding) {
+  this->osd.renderer->set_encoding(&this->osd, encoding);
 }
 
 void xine_osd_set_position(xine_osd_t *this, int x, int y) {
@@ -579,6 +751,10 @@ void xine_osd_show(xine_osd_t *this, int64_t vpts) {
   this->osd.renderer->show(&this->osd, vpts);
 }
 
+void xine_osd_show_unscaled(xine_osd_t *this, int64_t vpts) {
+  this->osd.renderer->show_unscaled(&this->osd, vpts);
+}
+
 void xine_osd_hide(xine_osd_t *this, int64_t vpts) {
   this->osd.renderer->hide(&this->osd, vpts);
 }
@@ -603,6 +779,12 @@ void xine_osd_get_palette(xine_osd_t *this, uint32_t *color, uint8_t *trans) {
   this->osd.renderer->get_palette(&this->osd, color, trans);
 }
 
+void xine_osd_draw_bitmap(xine_osd_t *this, uint8_t *bitmap,
+			    int x1, int y1, int width, int height,
+			    uint8_t *palette_map) {
+  this->osd.renderer->draw_bitmap(&this->osd, bitmap, x1, y1, width, height, palette_map);
+}
+
 const char *const *xine_post_list_inputs(xine_post_t *this_gen) {
   post_plugin_t *this = (post_plugin_t *)this_gen;
   return this->input_ids;
@@ -613,7 +795,7 @@ const char *const *xine_post_list_outputs(xine_post_t *this_gen) {
   return this->output_ids;
 }
 
-const xine_post_in_t *xine_post_input(xine_post_t *this_gen, char *name) {
+xine_post_in_t *xine_post_input(xine_post_t *this_gen, const char *name) {
   post_plugin_t  *this = (post_plugin_t *)this_gen;
   xine_post_in_t *input;
   
@@ -626,7 +808,7 @@ const xine_post_in_t *xine_post_input(xine_post_t *this_gen, char *name) {
   return NULL;
 }
 
-const xine_post_out_t *xine_post_output(xine_post_t *this_gen, char *name) {
+xine_post_out_t *xine_post_output(xine_post_t *this_gen, const char *name) {
   post_plugin_t   *this = (post_plugin_t *)this_gen;
   xine_post_out_t *output;
   
@@ -686,3 +868,96 @@ xine_post_out_t * xine_get_audio_source(xine_stream_t *stream) {
   return &stream->audio_source;
 }
 
+/* report error/message to UI. may be provided with several
+ * string parameters. last parameter must be NULL.
+ */
+int _x_message(xine_stream_t *stream, int type, ...) {
+  xine_ui_message_data_t *data;
+  xine_event_t            event;
+  char                   *explanation;
+  int                     size;
+  int                     n;
+  va_list                 ap;
+  char                   *s, *params;
+  char                   *args[1024];
+  static char            *std_explanation[] = {
+    "",
+    "Warning:",
+    "Unknown host:",
+    "Unknown device:",
+    "Network unreachable",
+    "Connection refused:",
+    "File not found:",
+    "Read error from:",
+    "Error loading library:",
+    "Encrypted media stream detected",
+    "Security message:",
+    "Audio device unavailable",
+    "Permission error",
+  };
+
+  if (!stream) return 0;
+
+  if( type >= 0 && type < sizeof(std_explanation)/
+                           sizeof(std_explanation[0]) ) {
+    explanation = std_explanation[type];
+    size = strlen(explanation)+1;
+  } else {
+    explanation = NULL;
+    size = 0;
+  }
+
+  n = 0;
+  va_start(ap, type);
+  while(((s = va_arg(ap, char *)) != NULL) && (n < 1024)) {
+    size += strlen(s) + 1;
+    args[n] = s;
+    n++;
+  }
+  va_end(ap);
+
+  args[n] = NULL;
+  
+  size += sizeof(xine_ui_message_data_t) + 1;
+  data = xine_xmalloc( size );
+
+  strcpy(data->compatibility.str, "Upgrade your frontend to see the error messages");
+  data->type           = type;
+  data->num_parameters = n;
+  
+  if( explanation ) {
+    strcpy(data->messages, explanation);
+    data->explanation = data->messages - (char *)data;
+    params = data->messages + strlen(explanation) + 1;
+  } else {
+    data->explanation = 0;
+    params            = data->messages;
+  }
+
+  data->parameters = params - (char *)data;
+  
+  n       = 0;
+  *params = '\0';
+  
+  while(args[n]) {
+    strcpy(params, args[n]);
+    params += strlen(args[n]) + 1;
+    n++;
+  }
+
+  *params = '\0';
+
+  event.type        = XINE_EVENT_UI_MESSAGE;
+  event.stream      = stream;
+  event.data_length = size;
+  event.data        = data;
+  xine_event_send(stream, &event);
+
+  free(data);
+
+  return 1;
+}
+
+int64_t xine_get_current_vpts(xine_stream_t *stream) {
+  return stream->xine->clock->get_current_time(stream->xine->clock);
+}
